unit Dclimate;

{-------------------------------------------------------------------}
{                    Unit:    Dclimate.pas                          }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/29/05      (5.0.005)                }
{                             9/5/05       (5.0.006)                }
{                             1/21/09      (5.0.014)                }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form unit containing the Climatology properties editor.  }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, GridEdit, StdCtrls, ComCtrls, Grids, NumEdit, Buttons,
  Uproject, Uglobals, Mask, ExtCtrls;

type
  TClimatologyForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    OKBtn: TButton;
    CancelBtn: TButton;
    HelpBtn: TButton;
    NoADBtn1: TButton;
    NatADBtn1: TButton;
    NoADBtn2: TButton;
    NatADBtn2: TButton;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    RadioButton5: TRadioButton;
    RadioButton6: TRadioButton;
    RadioButton7: TRadioButton;
    RadioButton8: TRadioButton;
    RadioButton9: TRadioButton;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    NumEdit1: TNumEdit;
    NumEdit2: TNumEdit;
    NumEdit3: TNumEdit;
    NumEdit4: TNumEdit;
    NumEdit5: TNumEdit;
    NumEdit6: TNumEdit;
    TempSeriesCombo: TComboBox;
    TempFileBtn: TBitBtn;
    TempFileEdit: TEdit;
    WindGrid1: TGridEditFrame;
    WindGrid2: TGridEditFrame;
    WindSpeedLabel: TLabel;
    EvapValueEdit: TNumEdit;
    EvapSeriesCombo: TComboBox;
    EvapGrid1: TGridEditFrame;
    EvapGrid2: TGridEditFrame;
    EvapGridLabel: TLabel;
    EvapUnitsLabel: TLabel;
    SnowTempUnitsLabel: TLabel;
    ElevUnitsLabel: TLabel;
    ADCGrid: TGridEditFrame;
    TempTseriesBtn: TBitBtn;
    EvapTseriesBtn: TBitBtn;
    Label3: TLabel;
    TempStartDateBox: TCheckBox;
    TempStartDateEdit: TMaskEdit;
    RecoveryCombo: TComboBox;
    RecoveryBtn1: TBitBtn;
    RecoveryBtn2: TBitBtn;
    SoilRecoveryLabel: TLabel;
    Bevel1: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure RadioBtnClick(Sender: TObject);
    procedure TempFileBtnClick(Sender: TObject);
    procedure TempSeriesComboClick(Sender: TObject);
    procedure TempFileEditChange(Sender: TObject);
    procedure NoADBtn1Click(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure TempTseriesBtnClick(Sender: TObject);
    procedure EvapTseriesBtnClick(Sender: TObject);
    procedure TempSeriesComboChange(Sender: TObject);
    procedure EvapValueEditChange(Sender: TObject);
    procedure EvapSeriesComboChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TempStartDateBoxClick(Sender: TObject);
    procedure RecoveryComboDblClick(Sender: TObject);
    procedure RecoveryBtn2Click(Sender: TObject);
    procedure RecoveryComboChange(Sender: TObject);
  private
    { Private declarations }
    FileDir: String;
  public
    { Public declarations }
    HasChanged: Boolean;
    procedure SetData(I: Integer);
    procedure GetData;
  end;

//var
//  ClimatologyForm: TClimatologyForm;

implementation

{$R *.dfm}

uses Fmain, Uedit;

const
  MonthLabels1: PChar = 'Jan'#13'Feb'#13'Mar'#13'Apr'#13'May'#13'Jun';
  MonthLabels2: PChar = 'Jul'#13'Aug'#13'Sep'#13'Oct'#13'Nov'#13'Dec';
  ADCColLabels: PChar = 'Depth Ratio'#13'Impervious'#13'Pervious';
  ADCRowLabels: PChar = ''#13'0.0'#13'0.1'#13'0.2'#13'0.3'#13'0.4'#13'0.5'#13+
                        '0.6'#13'0.7'#13'0.8'#13'0.9';
  TXT_SELECT_FILE = 'Select a Climatological File';
  TXT_FILE_FILTER = 'Data files (*.DAT)|*.DAT|Text files (*.TXT)|*.TXT' +
                    '|All files|*.*';
  TXT_MONTHLY_EVAP = 'Monthly Evaporation ';
  TXT_PAN_COEFFS   = 'Pan Coefficients';

procedure TClimatologyForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnCreate handler. Sets up the form's grid and combo box entries.
//-----------------------------------------------------------------------------
begin
  Uglobals.SetFont(self);
  FileDir := ProjectDir;
  with EvapGrid1.Grid do
  begin
    Rows[0].SetText(MonthLabels1);
  end;
  with EvapGrid2.Grid do
  begin
    Rows[0].SetText(MonthLabels2);
  end;
  with WindGrid1.Grid do
  begin
    Rows[0].SetText(MonthLabels1);
  end;
  with WindGrid2.Grid do
  begin
    Rows[0].SetText(MonthLabels2);
  end;
  with ADCGrid.Grid do
  begin
    Cols[0].SetText(ADCRowLabels);
    Rows[0].SetText(ADCColLabels);
    ClientWidth := ColCount*(DefaultColWidth+1);
  end;
  TempSeriesCombo.Items := Project.Lists[TIMESERIES];
  EvapSeriesCombo.Items := Project.Lists[TIMESERIES];
  RecoveryCombo.Items := Project.Lists[PATTERN];                                //(5.0.014 - LR)
  if UnitSystem = usSI then
  begin
    EvapUnitsLabel.Caption     := '(mm/day)';
    WindSpeedLabel.Caption     := 'Monthly Average Wind Speed (km/hr)';
    SnowTempUnitsLabel.Caption := '(degrees C)';
    ElevUnitsLabel.Caption     := '(meters)';

//// Added by FM - 9/5/05 ////
    EvapGridLabel.Caption      := 'Monthly Evaporation (mm/day)';
  end;
end;

procedure TClimatologyForm.TempSeriesComboChange(Sender: TObject);
//-----------------------------------------------------------------------------
//  Checks off Time Series radio button when an entry in the Time Series combo
//  box is changed.
//-----------------------------------------------------------------------------
begin
  RadioButton2.Checked := True;
  HasChanged := True;
end;

procedure TClimatologyForm.TempSeriesComboClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  Checks off Time Series radio button when an entry in the Time Series combo
//  box is selected.
//-----------------------------------------------------------------------------
begin
  RadioButton2.Checked := True;
  HasChanged := True;
end;

procedure TClimatologyForm.TempTseriesBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  Launches the Time Series Editor when the Temp Tseries button is clicked.
//-----------------------------------------------------------------------------
var
  I: Integer;
  S: String;
begin
  // Extract name of time series from Temp. Time Series combo box
  S := Trim(TempSeriesCombo.Text);

  // Edit the time series with this name
  I := Project.Lists[TIMESERIES].IndexOf(S);
  S := Uedit.EditTimeseries(I);

  // If name was changed, then change it in the Temp. Time Series combo box
  if Length(S) > 0 then
  begin
      TempSeriesCombo.Text := S;
      TempSeriesCombo.Items := Project.Lists[TIMESERIES];
  end;
end;

procedure TClimatologyForm.TempFileBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  Uses the MainForm's OpenTextFileDialog to obtain the name of a
//  temperature data file when the TempFileBtn is clicked.
//-----------------------------------------------------------------------------
begin
  with MainForm.OpenTextFileDialog do
  begin
    Filename := TempFileEdit.Text;
    if Length(Filename) = 0 then Filename := '*.dat';
    InitialDir := FileDir;
    Title := TXT_SELECT_FILE;
    Filter := TXT_FILE_FILTER;
    FilterIndex := 1;
    if Execute then
    begin
      TempFileEdit.Text := Filename;
      FileDir := ExtractFileDir(Filename);
    end;
  end;
end;

procedure TClimatologyForm.TempFileEditChange(Sender: TObject);
//-----------------------------------------------------------------------------
//  Checks off the External Climate File radio button when contents of
//  TempFileEdit edit box is changed.
//-----------------------------------------------------------------------------
begin
  RadioButton3.Checked := True;
  HasChanged := True;
  TempFileEdit.Hint := TempFileEdit.Text;
end;

procedure TClimatologyForm.TempStartDateBoxClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  Enables the Temperature File Start Date edit control when the
//  TempStartDateBox check box is checked.
//-----------------------------------------------------------------------------
begin
  TempStartDateEdit.Enabled := TempStartDateBox.Checked;
end;

procedure TClimatologyForm.EvapValueEditChange(Sender: TObject);
//-----------------------------------------------------------------------------
//  Checks off the Evap. Constant Value radio button when the contents of the
//  EvapValueEdit edit box is changed.
//-----------------------------------------------------------------------------
begin
  RadioButton4.Checked := True;
  HasChanged := True;
end;

procedure TClimatologyForm.EvapSeriesComboChange(Sender: TObject);
//-----------------------------------------------------------------------------
//  Checks off the Evap. Time Series radio button when the contents of the
//  EvapSeriesCombo combo box is changed.
//-----------------------------------------------------------------------------
begin
  RadioButton5.Checked := True;
  HasChanged := True;
end;

procedure TClimatologyForm.EvapTseriesBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  Launches the Time Series Editor when the Evap. Tseries button is clicked.
//-----------------------------------------------------------------------------
var
  I: Integer;
  S: String;
begin
  // Extract name of time series from Evap. Time Series combo box
  S := Trim(EvapSeriesCombo.Text);

  // Edit the time series with this name
  I := Project.Lists[TIMESERIES].IndexOf(S);
  S := Uedit.EditTimeseries(I);

  // If name was changed, then change it in the Evap. Time Series combo box
  if Length(S) > 0 then
  begin
      EvapSeriesCombo.Text := S;
      EvapSeriesCombo.Items := Project.Lists[TIMESERIES];
  end;
end;

procedure TClimatologyForm.RadioBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  Modifies the Evaporation page of form when either Evaporation from Climate
//  File button or Monthly Evaporation button is clicked.
//-----------------------------------------------------------------------------
var
  J: Integer;
begin
  if TRadioButton(Sender) = RadioButton6 then with Project.Climatology do
  begin
    EvapGridLabel.Caption := TXT_PAN_COEFFS;
    for J := 0 to 5 do
    begin
      EvapGrid1.Grid.Cells[J,1] := PanData[J];
      EvapGrid2.Grid.Cells[J,1] := PanData[J+6];
    end;
  end
  else if TRadioButton(Sender) = RadioButton7 then with Project.Climatology do
  begin
    EvapGridLabel.Caption := TXT_MONTHLY_EVAP + EvapUnitsLabel.Caption;
    for J := 0 to 5 do
    begin
      EvapGrid1.Grid.Cells[J,1] := EvapData[J];
      EvapGrid2.Grid.Cells[J,1] := EvapData[J+6];
    end;
  end;
  HasChanged := True;
end;

procedure TClimatologyForm.OKBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the OK button.
//-----------------------------------------------------------------------------
begin
  ModalResult := mrOK;
  if EvapGrid1.Modified then HasChanged := True;
  if EvapGrid2.Modified then HasChanged := True;
  if WindGrid1.Modified then HasChanged := True;
  if WindGrid2.Modified then HasChanged := True;
  if ADCGrid.Modified then HasChanged := True;
end;

procedure TClimatologyForm.NoADBtn1Click(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the four Areal Depletion buttons on the Areal
//  Deplaetion page of the form.
//-----------------------------------------------------------------------------
var
  K: Integer;
begin
  with ADCGrid.Grid do
  begin
    // No areal depletion for impervious area
    if Sender = NoADBtn1 then
      for K := 1 to 10 do Cells[1,K] := DefADCurve[K]
    // No areal depletion for pervious area
    else if Sender = NoADBtn2 then
      for K := 1 to 10 do Cells[2,K] := DefADCurve[K]
    // Natural areal depletion for impervious area
    else if Sender = NatADBtn1 then
      for K := 1 to 10 do Cells[1,K] := NatADCurve[K]
    // Natural area depletion for pervious area
    else if Sender = NatADBtn2 then
      for K := 1 to 10 do Cells[2,K] := NatADCurve[K]
    else exit;
  end;
  HasChanged := True;
end;

procedure TClimatologyForm.SetData(I: Integer);
//-----------------------------------------------------------------------------
//  Loads current climatology data into form and displays page I.
//-----------------------------------------------------------------------------
var
  J: Integer;
  K: Integer;
begin
  with Project.Climatology do
  begin
    // Temperature data
    TempSeriesCombo.Text := TempTseries;
    TempFileEdit.Text    := TempFile;

////  New lines added. (LR - 9/5/05)  ////
    if (Length(TempStartDate) > 0) then
    begin
      TempStartDateBox.Checked := True;
      TempStartDateEdit.Enabled := True;
      TempStartDateEdit.Text := TempStartDate;
    end;

    FileDir := ExtractFileDir(TempFile);
    if Length(FileDir) = 0 then FileDir := ProjectDir;
    case TempDataSource of
    NO_TEMP     : RadioButton1.Checked := True;
    TSERIES_TEMP: RadioButton2.Checked := True;
    FILE_TEMP   : RadioButton3.Checked   := True;
    end;

    // Evaporation data
    EvapValueEdit.Text := EvapDatA[0];
    EvapSeriesCombo.Text := EvapTseries;
    case EvapType of
    CONSTANT_EVAP: RadioButton4.Checked := True;
    TSERIES_EVAP : RadioButton5.Checked := True;
    FILE_EVAP:     RadioButton6.Checked := True;
    MONTHLY_EVAP : RadioButton7.Checked := True;
    end;
    RecoveryCombo.Text := RecoveryPat;                                          //(5.0.014 - LR)

    // Wind speed data
    with WindGrid1.Grid do
    begin
      for J := 0 to 5 do Cells[J,1] := WindSpeed[J+1];
    end;
    with WindGrid2.Grid do
    begin
      for J := 0 to 5 do Cells[J,1] := WindSpeed[J+7];
    end;
    if WindType = FILE_WINDSPEED then RadioButton8.Checked := True
    else RadioButton9.Checked := True;

    // Snow melt data
    for J := 1 to 6 do
    begin
      with FindComponent('NumEdit'+IntToStr(J)) as TNumEdit do
        Text := SnowMelt[J];
    end;
    with ADCGrid.Grid do
    begin
      for J := 1 to 2 do
      begin
        for K := 1 to 10 do
          Cells[J,K] := ADCurve[J][K];
      end;
    end;
  end;
  PageControl1.ActivePageIndex := I;
  EvapGrid1.Modified := False;
  EvapGrid2.Modified := False;
  WindGrid1.Modified := False;
  WindGrid2.Modified := False;
  ADCGrid.Modified   := False;
  HasChanged := False;
end;

procedure TClimatologyForm.GetData;
//-----------------------------------------------------------------------------
//  Retrieves climatology data from the form.
//-----------------------------------------------------------------------------
var
  I: Integer;
  J: Integer;
  S: String;
begin
  with Project.Climatology do
  begin

    // Temperature data
    if RadioButton1.Checked then TempDataSource := NO_TEMP;
    if RadioButton2.Checked then
    begin
      TempDataSource := TSERIES_TEMP;
      TempTseries    := TempSeriesCombo.Text;
    end;
    if RadioButton3.Checked then
    begin
      TempDataSource := FILE_TEMP;
      TempFile       := TempFileEdit.Text;

////  New lines added. (LR - 9/5/05)  ////
      if TempStartDateBox.Checked
      then TempStartDate := TempStartDateEdit.Text
      else TempStartDate := '';

    end;

    // Evaporation data
    if RadioButton4.Checked then
    begin
      EvapType := CONSTANT_EVAP;
      S := Trim(EvapValueEdit.Text);
      if Length(S) > 0 then for I := 0 to 11 do EvapData[I] := S;
    end;
    if RadioButton5.Checked then
    begin
      EvapType := TSERIES_EVAP;
      EvapTseries := EvapSeriesCombo.Text;
    end;
    if RadioButton6.Checked then
    begin
      EvapType := FILE_EVAP;
      for I := 0 to 5 do
      begin
        S := Trim(EvapGrid1.Grid.Cells[I,1]);
        if Length(S) = 0 then S := '0.0';
        PanData[I] := S;
        S := Trim(EvapGrid2.Grid.Cells[I,1]);
        if Length(S) = 0 then S := '0.0';
        PanData[I+6] := S;
      end;
    end;
    if RadioButton7.Checked then
    begin
      EvapType := MONTHLY_EVAP;
      for I := 0 to 5 do
      begin
        S := Trim(EvapGrid1.Grid.Cells[I,1]);
        if Length(S) = 0 then S := '0.0';
        EvapData[I] := S;
        S := Trim(EvapGrid2.Grid.Cells[I,1]);
        if Length(S) = 0 then S := '0.0';
        EvapData[I+6] := S;
      end;
    end;
    RecoveryPat := Trim(RecoveryCombo.Text);                                    //(5.0.014 - LR)

    // Wind speed data
    with WindGrid1.Grid do
    begin
      for I := 0 to 5 do
      begin
        S := Trim(Cells[I,1]);
        if Length(S) = 0 then S := '0.0';
        WindSpeed[I+1] := S;
      end;
    end;
    with WindGrid2.Grid do
    begin
      for I := 0 to 5 do
      begin
        S := Trim(Cells[I,1]);
        if Length(S) = 0 then S := '0.0';
        WindSpeed[I+7] := S;
      end;
    end;
    if RadioButton8.Checked then WindType := FILE_WINDSPEED
    else WindType := MONTHLY_WINDSPEED;

    // Snow melt data
    for I := 1 to 6 do
    begin
      with FindComponent('NumEdit'+IntToStr(I)) as TNumEdit do
      begin
        S := Trim(Text);
        if Length(S) > 0 then SnowMelt[I] := S;
      end;
    end;
    with ADCGrid.Grid do
    begin
      for I := 1 to 2 do
      begin
        for J := 1 to 10 do
        begin
           S := Trim(Cells[I,J]);
           if Length(S) > 0 then ADCurve[I][J] := S;
        end;
      end;
    end;
  end;
end;

procedure TClimatologyForm.HelpBtnClick(Sender: TObject);
begin
  with PageControl1 do
    if ActivePage = TabSheet1 then
       Application.HelpCommand(HELP_CONTEXT, 212340)
    else if ActivePage = TabSheet2 then
       Application.HelpCommand(HELP_CONTEXT, 212360)
    else if ActivePage = TabSheet3 then
       Application.HelpCommand(HELP_CONTEXT, 212370)
    else if ActivePage = TabSheet4 then
       Application.HelpCommand(HELP_CONTEXT, 212380)
    else if ActivePage = TabSheet5 then
       Application.HelpCommand(HELP_CONTEXT, 212350)
end;

procedure TClimatologyForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtnClick(Sender);
end;


//================== Added to 5.0.014 =============================

procedure TClimatologyForm.RecoveryComboDblClick(Sender: TObject);
var
  I: Integer;
  S: String;
begin
  with RecoveryCombo do
  begin
    I := Project.Lists[PATTERN].IndexOf(Trim(Text));
    S := Uedit.EditPattern(I);
    if Length(S) > 0 then Text := S;
  end;
end;

procedure TClimatologyForm.RecoveryBtn2Click(Sender: TObject);
begin
  RecoveryCombo.Text := '';
end;

procedure TClimatologyForm.RecoveryComboChange(Sender: TObject);
begin
  HasChanged := True;
end;

end.
