unit Dgrpdel;

{-------------------------------------------------------------------}
{                    Unit:    Dgrpdel.pas                           }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/29/05    (5.0.005)                  }
{                             6/19/07    (5.0.010)                  }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form for deleting a group of objects bounded by a user-  }
{   drawn fenceline.                                                }
{                                                                   }
{   Updated by adding the object's Tag property as a deletion       }
{   criterion. (5/11/07 - LR).                                      }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Uproject, Uglobals, ExtCtrls;

type
  TGroupDeleteForm = class(TForm)
    RainGageCheckBox: TCheckBox;
    SubcatchCheckBox: TCheckBox;
    NodeCheckBox: TCheckBox;
    LabelCheckBox: TCheckBox;
    OkBtn: TButton;
    CancelBtn: TButton;
    RainGageTagEdit: TEdit;
    SubcatchTagEdit: TEdit;
    NodeTagEdit: TEdit;
    Label1: TLabel;
    RainGageTagCheck: TCheckBox;
    SubcatchTagCheck: TCheckBox;
    NodeTagCheck: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure OkBtnClick(Sender: TObject);
  private
    { Private declarations }
    theRegion: HRgn;
    NumDeletions: Integer;
    procedure GroupDelete;
    procedure DeleteRaingages;
    procedure DeleteSubcatchments;
    procedure DeleteNodes;
    procedure DeleteLabels;
  public
    { Public declarations }
  end;

//var
//  GroupDeleteForm: TGroupDeleteForm;

implementation

{$R *.dfm}

uses Fmain, Fmap, Ubrowser;

procedure TGroupDeleteForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnCreate handler.
//-----------------------------------------------------------------------------
begin
  Uglobals.SetFont(self);
end;

procedure TGroupDeleteForm.OkBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the OK button.
//-----------------------------------------------------------------------------
begin
  GroupDelete;
end;

procedure TGroupDeleteForm.GroupDelete;
//-----------------------------------------------------------------------------
//  Deletes all objects of a selected type that lie within the user-drawn
//  fenceline.
//-----------------------------------------------------------------------------
begin
  //Create a GDI region from user's fenceline region
  theRegion := CreatePolygonRgn(MapForm.Fenceline, MapForm.NumFencePts - 1,
                                WINDING);
  try
    // Delete selected classes of objects
    NumDeletions := 0;
    if RainGageCheckBox.Checked then DeleteRaingages;
    if SubcatchCheckBox.Checked then DeleteSubcatchments;
    if NodeCheckBox.Checked then DeleteNodes;
    if LabelCheckBox.Checked then DeleteLabels;

    // Update browser item list box and map
    if NumDeletions > 0 then with MainForm do
    begin
      SetChangeFlags;
      ItemListBox.Count := Project.Lists[CurrentList].Count;
      ItemListBox.Refresh;
      Ubrowser.BrowserUpdate(CurrentList, Project.CurrentItem[CurrentList]);
      MapForm.NumFencePts := 0;
      MapForm.RedrawMap;
    end;
  finally
    DeleteObject(theRegion);
  end;
end;

procedure TGroupDeleteForm.DeleteRaingages;
//-----------------------------------------------------------------------------
//  Deletes all rain gages that lie within the deletion polygon area.
//-----------------------------------------------------------------------------
var
  J: Integer;
  Xp: Integer;
  Yp: Integer;
  Tag: String;                                                                  //(5.0.010 - LR)
  Checked: Boolean;                                                             //(5.0.010 - LR)
  G: TRaingage;                                                                 //(5.0.010 - LR)
begin
  // Try to delete raingages from highest to lowest list index
  Checked := RainGageTagCheck.Checked;                                          //(5.0.010 - LR)
  Tag := RainGageTagEdit.Text;                                                  //(5.0.010 - LR)
  for J := Project.Lists[RAINGAGE].Count-1 downto 0 do
  begin
    G := Project.GetGage(J);                                                    //(5.0.010 - LR)
    Xp := MapForm.Map.GetXpix(G.X);                                             //(5.0.010 - LR)
    Yp := MapForm.Map.GetYpix(G.Y);                                             //(5.0.010 - LR)
    if (PtInRegion(theRegion, Xp, Yp)) then
    begin
      if Checked and not SameText(Tag, G.Data[TAG_INDEX]) then continue;        //(5.0.010 - LR)
      Inc(NumDeletions);
      Project.DeleteItem(RAINGAGE, J);
    end;
  end;
end;

procedure TGroupDeleteForm.DeleteSubcatchments;
//-----------------------------------------------------------------------------
//  Deletes all subcatchments that lie within the deletion polygon area.
//-----------------------------------------------------------------------------
var
  J: Integer;
  Xp: Integer;
  Yp: Integer;
  Tag: String;                                                                  //(5.0.010 - LR)
  Checked: Boolean;                                                             //(5.0.010 - LR)
  S: TSubcatch;                                                                 //(5.0.010 - LR)
begin
  // Try to delete subcatchments from highest to lowest list index
  Checked := SubcatchTagCheck.Checked;                                          //(5.0.010 - LR)
  Tag := SubcatchTagEdit.Text;                                                  //(5.0.010 - LR)
  for J := Project.Lists[SUBCATCH].Count-1 downto 0 do
  begin
    S := Project.GetSubcatch(SUBCATCH, J);                                      //(5.0.010 - LR)
    Xp := MapForm.Map.GetXpix(S.X);                                             //(5.0.010 - LR)
    Yp := MapForm.Map.GetYpix(S.Y);                                             //(5.0.010 - LR)
    if (PtInRegion(theRegion, Xp, Yp)) then
    begin
      if Checked and not SameText(Tag, S.Data[TAG_INDEX]) then continue;        //(5.0.010 - LR)
      Inc(NumDeletions);
      // Remove this subcatchment as the outlet of any other subcatchment       //(5.0.010 - LR)
      Project.DeleteSubcatchOutSubcatch(S);                                     //(5.0.010 - LR)
      Project.DeleteItem(SUBCATCH, J);
    end;
  end;
end;

procedure TGroupDeleteForm.DeleteNodes;
//-----------------------------------------------------------------------------
//  Deletes all nodes and their connecting links that lie within the
//  deletion polygon area.
//-----------------------------------------------------------------------------
var
  I, J: Integer;
  P: TPoint;
  N: TNode;                                                                     //(5.0.010 - LR)
  Tag: String;                                                                  //(5.0.010 - LR)
  Checked: Boolean;                                                             //(5.0.010 - LR)
begin
  Checked := NodeTagCheck.Checked;                                              //(5.0.010 - LR)
  Tag := NodeTagEdit.Text;                                                      //(5.0.010 - LR)
  for I := 0 to MAXCLASS do
  begin
    if Project.IsNode(I) then
    begin
      // Must delete nodes from highest to lowest list index
      for J := Project.Lists[I].Count-1 downto 0 do
      begin
        N := Project.GetNode(I, J);                                             //(5.0.010 - LR)
        P := MapForm.Map.GetNodePoint(N);                                       //(5.0.010 - LR)
        if (PtInRegion(theRegion, P.X, P.Y)) then
        begin
          if Checked and not SameText(Tag, N.Data[TAG_INDEX]) then continue;    //(5.0.010 - LR)
          Inc(NumDeletions);

          // Delete any references to node as a subcatchment outlet or          //(5.0.010 - LR)
          // as a label's anchor point                                          //(5.0.010 - LR)
          Project.DeleteSubcatchOutNode(N);                                     //(5.0.010 - LR)
          Project.DeleteLabelAnchors(N);                                        //(5.0.010 - LR)

          // Delete all links adjacent to deleted node
          MapForm.Map.GetAdjacencyRect(I, J, True);
          Project.DeleteItem(I, J);
        end;
      end;
    end;
  end;
end;

procedure TGroupDeleteForm.DeleteLabels;
//-----------------------------------------------------------------------------
//  Deletes all map labels that lie within the deletion polygon area.
//-----------------------------------------------------------------------------
var
  J: Integer;
  Xp: Integer;
  Yp: Integer;
begin
  for J := Project.Lists[MAPLABEL].Count-1 downto 0 do
  begin
    Xp := MapForm.Map.GetXpix(Project.GetMapLabel(J).X);
    Yp := MapForm.Map.GetYpix(Project.GetMapLabel(J).Y);
    if (PtInRegion(theRegion, Xp, Yp)) then
    begin
      Inc(NumDeletions);
      Project.DeleteItem(MAPLABEL, J);
    end;
  end;
end;

end.
