unit Dgwater;

{-------------------------------------------------------------------}
{                    Unit:    Dgwater.pas                           }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/29/05      (5.0.005)                }
{                             9/5/05       (5.0.006)                }
{                             1/21/09      (5.0.014)                }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form unit used to edit the groundwater flow properties   }
{   of a subcatchment.                                              }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Uproject, Uglobals, Uutils, PropEdit;

////  New constant added. (LR - 9/5/05)  ////
const
  MAX_GW_PROPS = 9;

type
  TGroundWaterForm = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    Panel3: TPanel;
    OKBtn: TButton;
    CancelBtn: TButton;
    HelpBtn: TButton;
    Panel2: TPanel;
    HintLabel: TLabel;
    procedure OKBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
  private
    { Private declarations }
    SubCatchIndex: Integer;

////  New GW property added. (LR - 9/5/05)  ////
    GwaterProps: array[0..MAX_GW_PROPS] of TPropRecord;

    PropEdit1: TPropEdit;
    PropList: TStringlist;
    procedure ShowPropertyHint(Sender: TObject; aRow: LongInt);
    procedure ValidateEntry(Sender: TObject; Index: Integer; var S: String;
              var Errmsg: String; var IsValid: Boolean);
  public
    { Public declarations }
    HasChanged: Boolean;
    procedure SetData(const Index: Integer);
    procedure GetData(const Index: Integer);
    procedure SetGroupData;                                                     //(5.0.014 - LR)
    procedure GetGroupData(var GWData: array of String);                        //(5.0.014 - LR)
  end;

//var
//  GroundWaterForm: TGroundWaterForm;

implementation

{$R *.dfm}

const
  TXT_PROPERTY = 'Property';
  TXT_VALUE = 'Value';
  MSG_NO_DATA = 'This data field cannot be blank.';

////  New property added to the arrays below. (LR - 9/5/05)  ////

  PropNames: array[0..MAX_GW_PROPS] of String =
    ('Aquifer Name',
     'Receiving Node',
     'Surface Elevation',
     'Groundwater Flow Coeff.',
     'Groundwater Flow Expon.',
     'Surface Water Flow Coeff.',
     'Surface Water Flow Expon.',
     'Surface-GW Interaction Coeff.',
     'Fixed Surface Water Depth',
     'Threshold Groundwater Elev.');                                            //(5.0.014 - LR)

  DefaultProps: array[0..MAX_GW_PROPS] of String =
    ('', '*', '0', '0', '0', '0', '0', '0', '0', '');

  GroundWaterHint: array[0..MAX_GW_PROPS] of String =
    ('Name of aquifer object that lies below subcatchment ' +
     '(leave blank for no groundwater).',
     'Name of node that receives groundwater from the aquifer.',
     'Elevation of ground surface for the subcatchment (ft or m).',
     'Multiplier of distance between actual and threshold GW levels.',                    //(5.0.014 - LR)
     'Exponent of distance between actual and threshold level GW levels.',                //(5.0.014 - LR)
     'Multiplier of distance between surface water level and threshold GW level.',        //(5.0.014 - LR)
     'Exponent of distance between surface water level and threshold GW level.',          //(5.0.014 - LR)
     'Multiplier of product of GW height and surface water height above aquifer bottom.', //(5.0.014 - LR)
     'Fixed depth of surface water at receiving node (ft or m) ' +
     '(set to zero if surface water depth will vary as computed by flow routing).',
     'Groundwater elevation that must be reached before any flow occurs ' +               //(5.0.014 - LR)
     '(feet or meters). Leave blank to use the receiving node''s invert elevation.');     //(5.0.014 - LR)

procedure TGroundWaterForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnCreate handler.
//-----------------------------------------------------------------------------
var
  i: Integer;
begin
  // Set font size & style
  Uglobals.SetFont(self);

  // Create Property Editor
  PropEdit1 := TPropEdit.Create(self);
  with PropEdit1 do
  begin
    Parent := Panel1;
    Align := alClient;
    BorderStyle := bsNone;
    ColHeading1 := TXT_PROPERTY;
    ColHeading2 := TXT_VALUE;
    HeaderSplit := 67;
    ValueColor := clNavy;
    OnValidate := ValidateEntry;
    OnRowSelect := ShowPropertyHint;
  end;

  // Create Property stringlist
  PropList := TStringlist.Create;

  // Set the attributes of the Aquifer name property
  GwaterProps[0].Name   := PropNames[0];
  GwaterProps[0].Style  := esComboEdit;
  GwaterProps[0].Mask   := emNoSpace;
  GwaterProps[0].Length := 0;
  GwaterProps[0].List   := Project.Lists[AQUIFER].Text;

  // Set the attributes of the Node name property
  GwaterProps[1].Name   := PropNames[1];
  GwaterProps[1].Style  := esEdit;
  GwaterProps[1].Mask   := emNoSpace;
  GwaterProps[1].Length := 0;

  // Set the attributes of the Surface Elev. & Flow Coeffs. properties
  for i := 2 to High(GwaterProps) do
  begin
    GwaterProps[i].Name   := PropNames[i];
    GwaterProps[i].Style  := esEdit;
    GwaterProps[i].Mask   := emNumber;
    GwaterProps[i].Length := 0;
  end;
end;

procedure TGroundWaterForm.FormShow(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnShow handler.
//-----------------------------------------------------------------------------
begin
  PropEdit1.SetProps(GwaterProps, PropList);
  PropEdit1.Edit;
end;

procedure TGroundWaterForm.FormDestroy(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnDestroy handler.
//-----------------------------------------------------------------------------
begin
  PropList.Free;
  PropEdit1.Free;
end;

procedure TGroundWaterForm.OKBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the OK button.
//-----------------------------------------------------------------------------
begin
  if (not PropEdit1.IsValid) then
  begin
    ModalResult := mrNone;
    PropEdit1.Edit;
  end
  else ModalResult := mrOK;
end;

procedure TGroundWaterForm.ShowPropertyHint(Sender: TObject; aRow: LongInt);
//-----------------------------------------------------------------------------
//  Property Editor's OnRowSelect handler.
//-----------------------------------------------------------------------------
begin
  HintLabel.Caption := GroundWaterHint[aRow];
end;

procedure TGroundWaterForm.SetData(const Index: Integer);
//-----------------------------------------------------------------------------
//  Loads the current GW flow properties associated with subcatchment
//  of given index into the form's Property Editor control.
//-----------------------------------------------------------------------------
var
  K: Integer;
  S: TSubcatch;
begin
  // Save index of subcatchment
  SubcatchIndex := Index;

  // First load default property values into the editor
  for K := 0 to High(DefaultProps) do PropList.Add(DefaultProps[K]);

  // If subcatchment exists, load its properties into the editor
  if (Index >= 0) and (Index < Project.Lists[SUBCATCH].Count) then
  begin
    S := Project.GetSubcatch(SUBCATCH, Index);
    for K := 0 to S.Groundwater.Count-1 do
      PropList[K] := S.Groundwater[K];
  end;
  HasChanged := False;
end;

procedure TGroundWaterForm.GetData(const Index: Integer);
//-----------------------------------------------------------------------------
//  Unloads the GW flow properties from the form's Property Editor control
//  into subcatchment of given index.
//-----------------------------------------------------------------------------
var
  K: Integer;
  S: TSubcatch;
begin
  if (Index >= 0) and (Index < Project.Lists[SUBCATCH].Count) then
  begin
    S := Project.GetSubcatch(SUBCATCH, Index);
    S.Groundwater.Clear;
    if Length(Trim(PropList[0])) > 0
    then for K := 0 to PropList.Count-1 do S.Groundwater.Add(PropList[K]);
    HasChanged := PropEdit1.Modified;
  end;
end;

////  New procedure added for 5.0.014  ////                                     //(5.0.014 - LR)
procedure TGroundWaterForm.SetGroupData;
//-----------------------------------------------------------------------------
//  Initializes the Groundwater properties when the form is being
//  used to group edit a collection of subcatchments.
//-----------------------------------------------------------------------------
var
  K: Integer;
begin
  Caption := 'Group Groundwater Editor';
  for K := 0 to High(DefaultProps) do PropList.Add('*');
  PropEdit1.OnRowSelect := nil;
  HintLabel.Caption :=
    'Edit only the properties you wish to change. ' +
    'Leave Aquifer Name blank to remove Groundwater from the ' +
    'subcatchment group.';
end;

procedure TGroundWaterForm.GetGroupData(var GWData: array of String);
//-----------------------------------------------------------------------------
//  Copies the form's Groundwater property values to a string array.
//-----------------------------------------------------------------------------
var
  K: Integer;
begin
  for K := 0 to PropList.Count-1 do
    GWData[K] := PropList[K];
end;

procedure TGroundWaterForm.ValidateEntry(Sender: TObject; Index: Integer;
 var S: String; var Errmsg: String; var IsValid: Boolean);
begin
  IsValid := True;
end;

procedure TGroundWaterForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTEXT, 212220);
end;

end.
