unit Dmap;

{-------------------------------------------------------------------}
{                    Unit:    Dmap.pas                              }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/29/05     (5.0.005)                 }
{                             2/4/08      (5.0.012)                 }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form unit for changing Study Area Map display options.   }
{                                                                   }
{   The form consists of a listbox on the left and a notebook       }
{   next to it. The notebook has 8 pages, one for each category     }
{   of map display options, which are selected from the listbox.    }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, ExtCtrls, ComCtrls, Math, Uglobals, Umap, Grids, CheckLst;

type
  TMapOptionsForm = class(TForm)
    BtnOK: TButton;
    BtnCancel: TButton;
    BtnHelp: TButton;
    Panel1: TPanel;
    Notebook1: TNotebook;
    NodesBySize: TCheckBox;
    NodeBorder: TCheckBox;
    GroupBox2: TGroupBox;
    NodeShape: TShape;
    NodeSpin: TSpinEdit;
    LinksBySize: TCheckBox;
    GroupBox3: TGroupBox;
    LinkShape: TShape;
    LinkSpin: TSpinEdit;
    Label4: TLabel;
    SpinEdit1: TSpinEdit;
    Label7: TLabel;
    SpinEdit4: TSpinEdit;
    NotationTransparent: TCheckBox;
    Label3: TLabel;
    LinkSymbols: TCheckBox;
    SpinEdit2: TSpinEdit;
    Label6: TLabel;
    Label8: TLabel;
    SpinEdit3: TSpinEdit;
    LinkArrows: TRadioGroup;
    ArrowSpin: TSpinEdit;
    Label5: TLabel;
    Shape1: TShape;
    Shape2: TShape;
    Shape3: TShape;
    Shape4: TShape;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    ListBox1: TListBox;
    LabelsTransparent: TCheckBox;
    Label1: TLabel;
    NotationFontSize: TSpinEdit;
    SubcatchFill: TRadioGroup;
    SubcatchLink: TCheckBox;
    SubcatchLine: TSpinEdit;
    Label2: TLabel;
    LinkBorder: TCheckBox;
    Label10: TLabel;
    SubcatchSymbol: TSpinEdit;
    NodeSymbols: TCheckBox;
    RadioButton5: TRadioButton;
    Shape5: TShape;
    NotationListBox: TCheckListBox;

    // Additional background color choices added.                               //(5.0.012 - LR)
    RadioButton6: TRadioButton;
    Shape6: TShape;
    RadioButton7: TRadioButton;
    Shape7: TShape;
    RadioButton8: TRadioButton;
    Shape8: TShape;
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure SpinEdit1Change(Sender: TObject);
    procedure NodeSpinChange(Sender: TObject);
    procedure LinkSpinChange(Sender: TObject);
    procedure Shape1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnHelpClick(Sender: TObject);
    procedure NodeBorderClick(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    { Private declarations }
    RadioButtons: array[1..MAXMAPCOLORINDEX-1] of TRadioButton;                 //(5.0.012 - LR)
    ZoomLevel: array[1..4] of Integer;
    procedure ResizeNodeShape;
    procedure ResizeLinkShape;
    procedure UpdateBoolean(var Value: Boolean; const NewValue: Boolean);
    procedure UpdateInteger(var Value: Integer; const NewValue: Integer);
  public
    { Public declarations }
    HasChanged: Boolean;
    procedure GetActivePage(var aPage: Integer);
    procedure SetOptions(const Options: TMapOptions);
    procedure SetActivePage(aPage: Integer);
    procedure GetOptions(var Options: TMapOptions);
  end;

var
  MapOptionsForm: TMapOptionsForm;

implementation

{$R *.DFM}

procedure TMapOptionsForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
// OnCreate handler for form.
//----------------------------------------------------------------------------
var
  i: Integer;
begin
  Uglobals.SetFont(self);
  for i := 1 to MAXMAPCOLORINDEX-1 do                                           //(5.0.012 - LR)
    RadioButtons[i] := FindComponent('RadioButton' + IntToStr(i))
      as TRadioButton;
  with ListBox1 do
  begin
    ItemHeight := (ClientHeight) div Items.Count;
    ItemIndex := 0;
  end;
  ListBox1Click(Sender);
end;

procedure TMapOptionsForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//-----------------------------------------------------------------------------
// OnKeyDown handler for form (KeyPreview was set to True).
// Calls OnClick handler for Help button when F1 is pressed.
//-----------------------------------------------------------------------------
begin
  if Key = VK_F1 then BtnHelpClick(Sender);
end;

procedure TMapOptionsForm.SetOptions(const Options: TMapOptions);
//-----------------------------------------------------------------------------
// Loads current map display options into the form.
//-----------------------------------------------------------------------------
var
  I: Integer;
  Bstyle: TBrushStyle;
begin
  with Options do
  begin
    SubcatchLink.Checked := ShowSubcatchLinks;
    Bstyle := TBrushStyle(SubcatchFillStyle);
    if      Bstyle = bsDiagCross then I := 3
    else if Bstyle = bsBDiagonal then I := 2
    else if Bstyle = bsSolid     then I := 1
    else                              I := 0;
    SubcatchFill.ItemIndex := I;
    SubcatchSymbol.Value := SubcatchSize;
    SubcatchLine.Value := SubcatchLineSize;

    NodeSpin.Value := NodeSize;
    NodesBySize.Checked := ShowNodesBySize;
    NodeBorder.Checked := ShowNodeBorder;

    LinkSpin.Value := LinkSize;
    LinksBySize.Checked := ShowLinksBySize;
    LinkBorder.Checked := ShowLinkBorder;

    NodeSymbols.Checked := ShowNodeSymbols;
    LinkSymbols.Checked := ShowLinkSymbols;

    LinkArrows.ItemIndex := Ord(ArrowStyle);
    ArrowSpin.Value := ArrowSize;

    LabelsTransparent.Checked := LabelsTranspar;

    with NotationListBox do
    begin
      Checked[0] := ShowGageIDs;
      Checked[1] := ShowSubcatchIDs;
      Checked[2] := ShowNodeIDs;
      Checked[3] := ShowLinkIDs;
      Checked[4] := ShowSubcatchValues;
      Checked[5] := ShowNodeValues;
      Checked[6] := ShowLinkValues;
    end;
    NotationTransparent.Checked := NotationTranspar;
    NotationFontSize.Value := NotationSize;

    ZoomLevel[1] := LabelZoom;
    ZoomLevel[2] := SymbolZoom;
    ZoomLevel[3] := ArrowZoom;
    ZoomLevel[4] := NotationZoom;

    for I := 1 to 4 do
      with FindComponent('SpinEdit' + IntToStr(I))
        as TSpinEdit do
          Value := ZoomLevel[I];
    for I := 1 to MAXMAPCOLORINDEX-1 do                                         //(5.0.012 - LR)
      RadioButtons[i].Checked := False;
    RadioButtons[ColorIndex].Checked := True;
  end;
  ResizeNodeShape;
  ResizeLinkShape;
  NodeBorderClick(self);
  NodeSymbols.Enabled := not QueryFlag;
  HasChanged := False;
end;

procedure TMapOptionsForm.GetOptions(var Options: TMapOptions);
//-----------------------------------------------------------------------------
// Unloads contents of form into map display options.
//-----------------------------------------------------------------------------
var
  I, J  : Integer;
  Astyle: TArrowStyle;
begin
  with Options do
  begin
    UpdateBoolean(ShowSubcatchLinks, SubcatchLink.Checked);
    I := 0;
    case SubcatchFill.ItemIndex of
    0: I := Ord(bsClear);
    1: I := Ord(bsSolid);
    2: I := Ord(bsBDiagonal);
    3: I := Ord(bsDiagCross);
    end;
    UpdateInteger(SubcatchFillStyle, I);
    UpdateInteger(SubcatchSize, SubcatchSymbol.Value);
    UpdateInteger(SubcatchLineSize, SubcatchLine.Value);

    UpdateInteger(NodeSize, NodeSpin.Value);
    UpdateBoolean(ShowNodesBySize, NodesBySize.Checked);
    UpdateBoolean(ShowNodeBorder,NodeBorder.Checked);

    UpdateInteger(LinkSize, LinkSpin.Value);
    UpdateBoolean(ShowLinksBySize, LinksBySize.Checked);
    UpdateBoolean(ShowLinkBorder, LinkBorder.Checked);

    UpdateBoolean(ShowNodeSymbols, NodeSymbols.Checked);
    UpdateBoolean(ShowLinkSymbols, LinkSymbols.Checked);

    Astyle := TArrowStyle(LinkArrows.ItemIndex);
    if Astyle <> ArrowStyle then HasChanged := true;
    ArrowStyle := Astyle;
    UpdateInteger(ArrowSize, ArrowSpin.Value);

    UpdateBoolean(LabelsTranspar, LabelsTransparent.Checked);

    with NotationListBox do
    begin
      ShowGageIDs := Checked[0];
      ShowSubcatchIDs := Checked[1];
      ShowNodeIDs := Checked[2];
      ShowLinkIDs := Checked[3];
      ShowSubcatchValues := Checked[4];
      ShowNodeValues := Checked[5];
      ShowLinkValues := Checked[6];
    end;
    UpdateBoolean(NotationTranspar, NotationTransparent.Checked);
    UpdateInteger(NotationSize, NotationFontSize.Value);

    UpdateInteger(LabelZoom, ZoomLevel[1]);
    UpdateInteger(SymbolZoom, ZoomLevel[2]);
    UpdateInteger(ArrowZoom, ZoomLevel[3]);
    UpdateInteger(NotationZoom, ZoomLevel[4]);

    J := 0;
    for I := 1 to MAXMAPCOLORINDEX-1 do                                         //(5.0.012 - LR)
      if RadioButtons[I].Checked then J := I;
    UpdateInteger(ColorIndex, J);
  end;
end;

procedure TMapOptionsForm.UpdateBoolean(var Value: Boolean;
  const NewValue: Boolean);
//-----------------------------------------------------------------------------
// Utility function which changes the value of a boolean variable.
//-----------------------------------------------------------------------------
begin
  if Value <> NewValue then HasChanged := true;
  Value := NewValue;
end;

procedure TMapOptionsForm.UpdateInteger(var Value: Integer;
  const NewValue: Integer);
//-----------------------------------------------------------------------------
// Utility function which changes the value of an integer variable.
//-----------------------------------------------------------------------------
begin
  if Value <> NewValue then HasChanged := true;
  Value := NewValue;
end;

procedure TMapOptionsForm.GetActivePage(var aPage: Integer);
//-----------------------------------------------------------------------------
// Retrieves index of notebook page of options currently displayed.
//-----------------------------------------------------------------------------
begin
  aPage := ListBox1.ItemIndex;
end;

procedure TMapOptionsForm.SetActivePage(aPage: Integer);
//-----------------------------------------------------------------------------
// Displays a specific page of map options.
//-----------------------------------------------------------------------------
begin
  with ListBox1 do
  begin
    if aPage < Items.Count then
    begin
      ItemIndex := aPage;
      ListBox1Click(Self);
    end;
  end;
end;

procedure TMapOptionsForm.ListBox1Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the option category listbox.
//-----------------------------------------------------------------------------
begin
  NoteBook1.PageIndex := ListBox1.ItemIndex;
end;

procedure TMapOptionsForm.ListBox1DrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
//-----------------------------------------------------------------------------
// OnDrawItem handler for the item category listbox.
// Centers the text within the item's drawing rectangle.
//-----------------------------------------------------------------------------
var
  ht: Integer;
  dy: Integer;
  s:  String;
begin
  with Control as TListBox do
  begin
    s := Items[Index];
    ht := Canvas.TextHeight(s);
    Canvas.FillRect(Rect);
    dy := (Rect.Bottom - Rect.Top - ht) div 2;
    Canvas.TextOut(0,Rect.Top+dy,s);
  end;
end;

procedure TMapOptionsForm.SpinEdit1Change(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for all of the SpinEdit controls associated with
// Zoom Level options. The Tag property assigned to these controls
// determines which Zoom Level property is being changed (1 = Labels,
// 2 = Symbols, 3 = Arrows, 4 = Notation).
//-----------------------------------------------------------------------------
begin
  with Sender as TSpinEdit do
    ZoomLevel[Tag] := Value;
end;

procedure TMapOptionsForm.NodeSpinChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for Node Size SpinEdit control.
//-----------------------------------------------------------------------------
begin
  ResizeNodeShape;
end;

procedure TMapOptionsForm.ResizeNodeShape;
//-----------------------------------------------------------------------------
// Resizes the NodeShape control.
//-----------------------------------------------------------------------------
var
  newsize : Integer;
  aRect   : TRect;
begin
  newsize := 3*NodeSpin.Value+2;
  aRect := NodeShape.BoundsRect;
  aRect.Top := NodeSpin.Top + (NodeSpin.Height div 2)
               - (newsize div 2);
  aRect.Bottom := aRect.Top + newsize;// + 1;
  aRect.Right := aRect.Left + newsize;// + 1;
  NodeShape.BoundsRect := aRect;
end;

procedure TMapOptionsForm.LinkSpinChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for Link Size SpinEdit control.
//-----------------------------------------------------------------------------
begin
  ResizeLinkShape;
end;

procedure TMapOptionsForm.ResizeLinkShape;
//-----------------------------------------------------------------------------
// Resizes the LinkShape control.
//-----------------------------------------------------------------------------
begin
  LinkShape.Top := LinkSpin.Top + (LinkSpin.Height - LinkSpin.Value) div 2;
  LinkShape.Pen.Width := LinkSpin.Value;
  LinkShape.Height := LinkSpin.Value;
end;

procedure TMapOptionsForm.Shape1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
//-----------------------------------------------------------------------------
// OnMouseDown handler for Shape controls used to display background color
// choices. Causes the corresponding RadioButton control to be selected.
//-----------------------------------------------------------------------------
var
  i: Integer;
begin
  with Sender as TShape do
    i := Tag;
  with FindComponent('RadioButton' + IntToStr(i))
    as TRadioButton do
      Checked := True;
end;

procedure TMapOptionsForm.NodeBorderClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for NodeBorder checkbox. Causes the node shape symbol
// to be drawn with/without a border.
//-----------------------------------------------------------------------------
begin
  if NodeBorder.Checked then
    NodeShape.Pen.Color := clBlack
  else
    NodeShape.Pen.Color := clRed;
end;

procedure TMapOptionsForm.BtnHelpClick(Sender: TObject);
begin
  case NoteBook1.PageIndex of
    0: Application.HelpCommand(HELP_CONTEXT, 211120);
    1: Application.HelpCommand(HELP_CONTEXT, 211130);
    2: Application.HelpCommand(HELP_CONTEXT, 211140);
    3: Application.HelpCommand(HELP_CONTEXT, 211150);
    4: Application.HelpCommand(HELP_CONTEXT, 211160);
    5: Application.HelpCommand(HELP_CONTEXT, 211170);
    6: Application.HelpCommand(HELP_CONTEXT, 211180);
    7: Application.HelpCommand(HELP_CONTEXT, 211190);
  end;
end;

end.
