unit Doptions;

{-------------------------------------------------------------------}
{                    Unit:    Doptions.pas                          }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    5/9/05       (5.0.005)                }
{                             9/5/05       (5.0.006)                }
{                             3/10/06      (5.0.007)                }
{                             7/5/06       (5.0.008)                }
{                             6/19/07      (5.0.010)                }
{                             2/4/08       (5.0.012)                }
{                             1/21/09      (5.0.014)                }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form unit that edits a project's simulation options.     }
{                                                                   }
{   The form consists of a PageControl component with 5 pages,      }
{   one for each category of simulation options.                    }
{                                                                   }
{   ////  Components on form modified. (LR - 9/5/05)  ////          }
{   ////                               (LR - 3/10/06) ////          }
{   ////                               (LR - 7/5/06)  ////          }
{   ////                               (5.0.010 - LR) ////          }
{   ////                               (5.0.012 - LR) ////          }
{   ////                               (5.0.014 - LR) ////          }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Spin, Mask, ComCtrls, Math,  NumEdit, Buttons,
  ImgList, Grids, Uproject, Uglobals, Uutils;

type
  TAnalysisOptionsForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    InfilModelsGroup: TRadioGroup;
    RoutingMethodsGroup: TRadioGroup;
    InertialTermsGroup: TRadioGroup;
    NormalFlowGroup: TRadioGroup;
    DryDaysEdit: TNumEdit;
    RouteStepEdit: TNumEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label21: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label27: TLabel;
    OKBtn: TButton;
    CancelBtn: TButton;
    HelpBtn: TButton;
    AddBtn: TButton;
    EditBtn: TButton;
    DeleteBtn: TButton;
    StartDatePicker: TDateTimePicker;
    RptDatePicker: TDateTimePicker;
    EndDatePicker: TDateTimePicker;
    StartTimePicker: TDateTimePicker;
    RptTimePicker: TDateTimePicker;
    EndTimePicker: TDateTimePicker;
    DryDaysPicker: TDateTimePicker;
    WetDaysPicker: TDateTimePicker;
    RptDaysPicker: TDateTimePicker;
    DryStepPicker: TDateTimePicker;
    WetStepPicker: TDateTimePicker;
    RptStepPicker: TDateTimePicker;
    SweepStartPicker: TDateTimePicker;
    SweepEndPicker: TDateTimePicker;
    FileListBox: TListBox;
    GroupBox1: TGroupBox;
    VarTimeStepBox: TCheckBox;
    SFLabel1: TLabel;
    VarStepEdit: TSpinEdit;
    GroupBox2: TGroupBox;
    LengthenStepEdit: TNumEdit;
    Label14: TLabel;
    Label19: TLabel;
    GroupBox3: TGroupBox;
    Label22: TLabel;
    MinSurfAreaEdit: TNumEdit;
    MinSurfAreaLabel: TLabel;
    MiscGroup: TGroupBox;
    AllowPondingBox: TCheckBox;
    ReportControlsBox: TCheckBox;
    ReportInputBox: TCheckBox;
    SkipSteadyBox: TCheckBox;
    ForceMainGroup: TRadioGroup;
    Label8: TLabel;
    MinSlopeEdit: TNumEdit;
    ModelsGroup: TGroupBox;
    RainfallBox: TCheckBox;
    SnowMeltBox: TCheckBox;
    GroundwaterBox: TCheckBox;
    FlowRoutingBox: TCheckBox;
    WaterQualityBox: TCheckBox;
    Label13: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EditChange(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure AddBtnClick(Sender: TObject);
    procedure EditBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure FileListBoxDblClick(Sender: TObject);
    procedure FileListBoxKeyPress(Sender: TObject; var Key: Char);
    procedure VarTimeStepBoxClick(Sender: TObject);
  private
    { Private declarations }
    OldInfilIndex: Integer;                                                     //(5.0.014 - LR)
    procedure EditFileSelection(var S: String);
    function  GetStepTime(DaysPicker: TDateTimePicker;
      TimePicker: TDateTimePicker): String;
    procedure SetStepTime(const S: String; DaysPicker: TDateTimePicker;
      TimePicker: TDateTimePicker);
  public
    { Public declarations }
    HasChanged: Boolean;
    procedure SetOptions(Page: Integer);
    procedure GetOptions;
  end;

//var
//  AnalysisOptionsForm: TAnalysisOptionsForm;

implementation

{$R *.dfm}

uses
  Fmain, Diface, Uupdate;

const
  TXT_SQUARE_FEET = 'Square Feet';
  TXT_SQUARE_METERS = 'Square Meters';

procedure TAnalysisOptionsForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnCreate handler.
//-----------------------------------------------------------------------------
var
  DateFmt: String;
begin
  // Set font size and style
  Uglobals.SetFont(self);

  // Set properties of the interface file list box
  FileListBox.Font.Style := Font.Style;
  FileListBox.ScrollWidth := 10*FileListBox.ClientWidth;

  // Set date format for date picker controls
  DateFmt := 'MM' + DateSeparator + 'dd' + DateSeparator + 'yyyy';
  StartDatePicker.Format := DateFmt;
  RptDatePicker.Format := DateFmt;
  EndDatePicker.Format := DateFmt;

  if Uglobals.UnitSystem = usUS then
    MinSurfAreaLabel.Caption := TXT_SQUARE_FEET
  else
    MinSurfAreaLabel.Caption := TXT_SQUARE_METERS;
end;

procedure TAnalysisOptionsForm.FormShow(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnShow handler.
//-----------------------------------------------------------------------------
begin
  case PageControl1.TabIndex of
  1: ActiveControl := StartDatePicker;
  2: ActiveControl := RptDaysPicker;
  3: ActiveControl := InertialTermsGroup;
  4: ActiveControl := AddBtn;
  end;
end;

procedure TAnalysisOptionsForm.SetStepTime(const S: String;
  DaysPicker: TDateTimePicker; TimePicker: TDateTimePicker);
//-----------------------------------------------------------------------------
//  Sets the values displayed in a pair of DateTimePicker controls to the
//  hrs:min:sec time step specified by string S, where hrs can be > 23.
//-----------------------------------------------------------------------------
var
  StepTime: TDateTime;
  StepDays: Integer;
begin
  StepTime := Uutils.StrHoursToTime(S);
  StepDays := Floor(StepTime);
  StepTime := StepTime - StepDays;
  DaysPicker.Time := EncodeTime(0, StepDays, 0, 0);
  TimePicker.Time := StepTime;
end;

function  TAnalysisOptionsForm.GetStepTime(DaysPicker: TDateTimePicker;
  TimePicker: TDateTimePicker): String;
//-----------------------------------------------------------------------------
//  Converts the contents of a pair of DateTimePicker controls
//  into hrs:min:sec format, where hrs can be > 23.
//-----------------------------------------------------------------------------
var
  Days, Hr, Min, Sec, Msec: Word;
begin
  DecodeTime(DaysPicker.Time, Hr, Days, Sec, Msec);
  DecodeTime(TimePicker.Time, Hr, Min, Sec, Msec);
  Hr := Hr + 24*Days;
  Result := Format('%.2d:%.2d:%.2d', [Hr,Min,Sec]);
end;

procedure TAnalysisOptionsForm.SetOptions(Page: Integer);
//-----------------------------------------------------------------------------
//  Loads current simulation options into form and displays the
//  specified page of the PageControl component.
//-----------------------------------------------------------------------------
var
  S: String;
  I: Integer;
begin
  S := Project.Options.Data[INFILTRATION_INDEX];
  I := Uutils.FindKeyword(S, InfilOptions, 4);
  if I < 0 then I := 0;
  InfilModelsGroup.ItemIndex := I;
  OldInfilIndex := I;                                                           //(5.0.014 - LR)

  S := Project.Options.Data[ROUTING_MODEL_INDEX];
  I := Uutils.FindKeyWord(S, RoutingOptions, 3);
  if I < 0 then I := 0;
  RoutingMethodsGroup.ItemIndex := I;
  with Project.Options do
  begin
    StartDatePicker.Date  := StrToDate(Data[START_DATE_INDEX], FormatSettings);
    StartTimePicker.Time  := StrToTime(Data[START_TIME_INDEX], FormatSettings);
    RptDatePicker.Date    := StrToDate(Data[REPORT_START_DATE_INDEX], FormatSettings);
    RptTimePicker.Time    := StrToTime(Data[REPORT_START_TIME_INDEX], FormatSettings);
    EndDatePicker.Date    := StrToDate(Data[END_DATE_INDEX], FormatSettings);
    EndTimePicker.Time    := StrToTime(Data[END_TIME_INDEX], FormatSettings);
    SweepStartPicker.Date := StrToDate(Data[SWEEP_START_INDEX] + '/1947', FormatSettings);
    SweepEndPicker.Date   := StrToDate(Data[SWEEP_END_INDEX] + '/1947', FormatSettings);
    DryDaysEdit.Text      := Data[DRY_DAYS_INDEX];

    SetStepTime(Data[REPORT_STEP_INDEX], RptDaysPicker, RptStepPicker);
    SetStepTime(Data[DRY_STEP_INDEX], DryDaysPicker, DryStepPicker);
    SetStepTime(Data[WET_STEP_INDEX], WetDaysPicker, WetStepPicker);
    RouteStepEdit.Text := Data[ROUTING_STEP_INDEX];

    AllowPondingBox.Checked := SameText(Data[ALLOW_PONDING_INDEX], 'YES');
    ReportControlsBox.Checked := SameText(Data[REPORT_CONTROLS_INDEX], 'YES');
    ReportInputBox.Checked := SameText(Data[REPORT_INPUT_INDEX], 'YES');
    SkipSteadyBox.Checked := SameText(Data[SKIP_STEADY_INDEX], 'YES');

////  New code added for 5.0.014  ////                                          //(5.0.014 - LR)
    if Project.Lists[RAINGAGE].Count > 0
    then RainfallBox.Checked := not SameText(Data[IGNORE_RAINFALL_INDEX], 'YES')
    else RainfallBox.Enabled := False;
    if Project.Lists[SNOWPACK].Count > 0
    then SnowMeltBox.Checked := not SameText(Data[IGNORE_SNOWMELT_INDEX], 'YES')
    else SnowMeltBox.Enabled := False;
    if Project.Lists[AQUIFER].Count > 0
    then GroundwaterBox.Checked := not SameText(Data[IGNORE_GRNDWTR_INDEX], 'YES')
    else GroundwaterBox.Enabled := False;
    if Project.GetLinkCount > 0
    then FlowRoutingBox.Checked := not SameText(Data[IGNORE_ROUTING_INDEX], 'YES')
    else FlowRoutingBox.Enabled := False;
    if Project.Lists[POLLUTANT].Count > 0
    then WaterQualityBox.Checked := not SameText(Data[IGNORE_QUALITY_INDEX], 'YES')
    else WaterQualityBox.Enabled := False;

    with InertialTermsGroup do
    begin
      if SameText(Data[INERTIAL_DAMPING_INDEX], 'NONE')
      then ItemIndex := 0
      else if Sametext(Data[INERTIAL_DAMPING_INDEX], 'FULL')
      then ItemIndex := 2
      else ItemIndex := 1;
    end;

////  New choices for Normal Flow Limited option. (5.0.010 - LR)  ////
    with NormalFlowGroup do
    begin
      if SameText(Data[NORMAL_FLOW_LTD_INDEX], 'SLOPE')
      then ItemIndex := 0
      else if SameText(Data[NORMAL_FLOW_LTD_INDEX], 'FROUDE')
      then ItemIndex := 1
      else ItemIndex := 2;
    end;

////  New Force Main Equation option. (5.0.010 - LR)  ////
    with ForceMainGroup do
    begin
      if SameText(Project.Options.Data[FORCE_MAIN_EQN_INDEX], 'H-W')
      then ItemIndex := 0
      else ItemIndex := 1;
    end;

    I := StrToInt(Data[VARIABLE_STEP_INDEX]);
    if I >= VarStepEdit.MinValue then
    begin
      VarTimeStepBox.Checked := True;
      VarStepEdit.Value := I;
      VarStepEdit.Enabled := True;
      SFLabel1.Enabled := True;
    end;

    LengthenStepEdit.Text := Data[LENGTHEN_STEP_INDEX];
    MinSurfAreaEdit.Text := Data[MIN_SURFAREA_INDEX];
    MinSlopeEdit.Text := Data[MIN_SLOPE_INDEX];                                 //(5.0.014 - LR)
  end;

  for I := 0 to Project.IfaceFiles.Count-1 do
    FileListBox.Items.Add(Project.IfaceFiles[I]);
  if FileListBox.Items.Count = 0 then
  begin
    EditBtn.Enabled := False;
    DeleteBtn.Enabled := False;
  end
  else FileListBox.ItemIndex := 0;

  HasChanged := False;
  if Page < 0 then Page := 0;
  if Page >= PageControl1.PageCount
  then Page := PageControl1.PageCount - 1;
  PageControl1.TabIndex := Page;
end;

procedure TAnalysisOptionsForm.GetOptions;
//-----------------------------------------------------------------------------
//  Unloads current contents of form into project's simulation options.
//-----------------------------------------------------------------------------
var
  I: Integer;
  Yr, Mon, Day: Word;
begin
  if  InfilModelsGroup.ItemIndex <> OldInfilIndex then                          //(5.0.014 - LR)
  begin
    Project.Options.Data[INFILTRATION_INDEX] :=
      InfilOptions[InfilModelsGroup.ItemIndex];
    case InfilModelsGroup.ItemIndex of
    0: CopyStringArray(DefHortonInfil, Uproject.DefInfil);
    1: CopyStringArray(DefGreenAmptInfil, Uproject.DefInfil);
    2: CopyStringArray(DefCurveNumInfil, Uproject.DefInfil);
    end;
  end;

  Project.Options.Data[ROUTING_MODEL_INDEX] :=
     RoutingOptions[RoutingMethodsGroup.ItemIndex];

  with Project.Options do
  begin
    Data[START_DATE_INDEX] := DateToStr(StartDatePicker.Date, FormatSettings);
    Data[START_TIME_INDEX] := TimeToStr(StartTimePicker.Time, FormatSettings);
    Data[REPORT_START_DATE_INDEX] := DateToStr(RptDatePicker.Date, FormatSettings);
    Data[REPORT_START_TIME_INDEX] := TimeToStr(RptTimePicker.Time, FormatSettings);
    Data[END_DATE_INDEX] := DateToStr(EndDatePicker.Date, FormatSettings);
    Data[END_TIME_INDEX] := TimeToStr(EndTimePicker.Time, FormatSettings);

    DecodeDate(SweepStartPicker.Date, Yr, Mon, Day);
    Data[SWEEP_START_INDEX] := Format('%.2d/%.2d', [Mon, Day]);
    DecodeDate(SweepEndPicker.Date, Yr, Mon, Day);
    Data[SWEEP_END_INDEX] := Format('%.2d/%.2d', [Mon, Day]);

    Data[DRY_DAYS_INDEX] := DryDaysEdit.Text;

    Data[REPORT_STEP_INDEX] := GetStepTime(RptDaysPicker, RptStepPicker);
    Data[DRY_STEP_INDEX] := GetStepTime(DryDaysPicker, DryStepPicker);
    Data[WET_STEP_INDEX] := GetStepTime(WetDaysPicker, WetStepPicker);
    Data[ROUTING_STEP_INDEX] := RouteStepEdit.Text;

    if AllowPondingBox.Checked then
      Data[ALLOW_PONDING_INDEX] := 'YES'
    else
      Data[ALLOW_PONDING_INDEX] := 'NO';

    if ReportControlsBox.Checked then
      Data[REPORT_CONTROLS_INDEX] := 'YES'
    else
      Data[REPORT_CONTROLS_INDEX] := 'NO';

    if ReportInputBox.Checked then
      Data[REPORT_INPUT_INDEX] := 'YES'
    else
      Data[REPORT_INPUT_INDEX] := 'NO';

    if SkipSteadyBox.Checked then
      Data[SKIP_STEADY_INDEX] := 'YES'
    else
      Data[SKIP_STEADY_INDEX] := 'NO';

////  New code added for 5.0.014  ////                                          //(5.0.014 - LR)
    if RainfallBox.Enabled then
      if not RainfallBox.Checked then
        Data[IGNORE_RAINFALL_INDEX] := 'YES'
      else
        Data[IGNORE_RAINFALL_INDEX] := 'NO';
    if SnowMeltBox.Enabled then
      if not SnowMeltBox.Checked then
        Data[IGNORE_SNOWMELT_INDEX] := 'YES'
      else
        Data[IGNORE_SNOWMELT_INDEX] := 'NO';
    if GroundwaterBox.Enabled then
      if not GroundwaterBox.Checked then
        Data[IGNORE_GRNDWTR_INDEX] := 'YES'
      else
        Data[IGNORE_GRNDWTR_INDEX] := 'NO';
    if FlowRoutingBox.Enabled then
      if not FlowRoutingBox.Checked then
        Data[IGNORE_ROUTING_INDEX] := 'YES'
      else
        Data[IGNORE_ROUTING_INDEX] := 'NO';
    if WaterQUalityBox.Enabled then
      if not WaterQualityBox.Checked then
        Data[IGNORE_QUALITY_INDEX] := 'YES'
      else
        Data[IGNORE_QUALITY_INDEX] := 'NO';
///////////////////////////////////////////////

    with InertialTermsGroup do
    begin
      case ItemIndex of
      0: Data[INERTIAL_DAMPING_INDEX] := 'NONE';
      1: Data[INERTIAL_DAMPING_INDEX] := 'PARTIAL';
      2: Data[INERTIAL_DAMPING_INDEX] := 'FULL';
      end;
    end;

////  New choices for Normal Flow Limited option. (5.0.010 - LR)  ////
    with NormalFlowGroup do
    begin
      case ItemIndex of
      0: Data[NORMAL_FLOW_LTD_INDEX] := 'SLOPE';
      1: Data[NORMAL_FLOW_LTD_INDEX] := 'FROUDE';
      2: Data[NORMAL_FLOW_LTD_INDEX] := 'BOTH';
      end;
    end;

////  New Force Main Equation option. (5.0.010 - LR)  ////
    with ForceMainGroup do
    begin
      case ItemIndex of
      0: Data[FORCE_MAIN_EQN_INDEX] := 'H-W';
      1: Data[FORCE_MAIN_EQN_INDEX] := 'D-W';
      end;
    end;

    if VarTimeStepBox.Checked then
      Data[VARIABLE_STEP_INDEX] := IntToStr(VarStepEdit.Value)
    else
      Data[VARIABLE_STEP_INDEX] := '0';

    Data[LENGTHEN_STEP_INDEX] := LengthenStepEdit.Text;
    Data[MIN_SURFAREA_INDEX] := MinSurfAreaEdit.Text;
    Data[MIN_SLOPE_INDEX] := MinSlopeEdit.Text;                                 //(5.0.014 - LR)
  end;

  Project.IfaceFiles.Clear;
  with FileListBox do
  begin
    for I := 0 to Items.Count-1 do Project.IfaceFiles.Add(Items[I]);
  end;
  Uupdate.UpdateDefOptions;
end;

procedure TAnalysisOptionsForm.VarTimeStepBoxClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Variable Time Step check box.
//-----------------------------------------------------------------------------
begin
  HasChanged := True;
  VarStepEdit.Enabled := VarTimeStepBox.Checked;
  SFLabel1.Enabled := VarStepEdit.Enabled;
end;

procedure TAnalysisOptionsForm.OKBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the OK button.
//-----------------------------------------------------------------------------
begin
  ModalResult := mrOK;
end;

procedure TAnalysisOptionsForm.EditChange(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnChange handler for most of form's controls.
//-----------------------------------------------------------------------------
begin
  HasChanged := True;
end;

procedure TAnalysisOptionsForm.AddBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Interface File Add button.
//-----------------------------------------------------------------------------
var
  S: String;
begin
  S := '';
  EditFileSelection(S);
  if Length(S) > 0 then
  begin
    FileListBox.Items.Add(S);
    FileListBox.ItemIndex := FileListBox.Items.Count-1;
    FileListBox.SetFocus;
    EditBtn.Enabled := True;
    DeleteBtn.Enabled := True;
    HasChanged := True;
  end;
end;

procedure TAnalysisOptionsForm.EditBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Interface File Edit button.
//-----------------------------------------------------------------------------
var
  S: String;
begin
  with FileListBox do
  begin
    S := Items[ItemIndex];
    EditFileSelection(S);
    if Length(S) > 0 then
    begin
      Items[ItemIndex] := S;
      HasChanged := True;
    end;
    SetFocus;
  end;
end;

procedure TAnalysisOptionsForm.DeleteBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Interface File Delete button.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  with FileListBox do
  begin
    I := ItemIndex;
    if ItemIndex >= 0 then
    begin
      Items.Delete(ItemIndex);
      HasChanged := True;
    end;
    if Items.Count = 0 then
    begin
      ItemIndex := -1;
      AddBtn.SetFocus;
      EditBtn.Enabled := False;
      DeleteBtn.Enabled := False;
    end
    else
    begin
      if I >= Items.Count then ItemIndex := Items.Count-1
      else ItemIndex := I;
      SetFocus;
    end;
  end;
end;

procedure TAnalysisOptionsForm.FileListBoxDblClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnDblClick handler for the Interface File list box.
//-----------------------------------------------------------------------------
begin
  if EditBtn.Enabled then EditBtnClick(Sender);
end;

procedure TAnalysisOptionsForm.FileListBoxKeyPress(Sender: TObject;
  var Key: Char);
//-----------------------------------------------------------------------------
//  OnKeyPress handler for the Interface File list box.
//-----------------------------------------------------------------------------
begin
  if (Key = #13) and EditBtn.Enabled then EditBtnClick(Sender);
end;

procedure TAnalysisOptionsForm.EditFileSelection(var S: String);
//-----------------------------------------------------------------------------
//  Launches the Interface File selection dialog form.
//-----------------------------------------------------------------------------
begin
  with TIfaceFileForm.Create(self) do
  try
    SetData(S);
    if ShowModal = mrOK then GetData(S);
  finally
    Free;
  end;
end;

procedure TAnalysisOptionsForm.HelpBtnClick(Sender: TObject);
begin
  with PageControl1 do
    if ActivePage = TabSheet1 then
       Application.HelpCommand(HELP_CONTEXT, 211200)
    else if ActivePage = TabSheet2 then
       Application.HelpCommand(HELP_CONTEXT, 211210)
    else if ActivePage = TabSheet3 then
       Application.HelpCommand(HELP_CONTEXT, 211220)
    else if ActivePage = TabSheet4 then
       Application.HelpCommand(HELP_CONTEXT, 211230)
    else if ActivePage = TabSheet5 then
       Application.HelpCommand(HELP_CONTEXT, 212080);
end;

end.
