unit Dprefers;

{-------------------------------------------------------------------}
{                    Unit:    Dprefers.pas                          }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/31/05      (5.0.005)                }
{                             9/5/05       (5.0.006)                }
{                             1/21/09      (5.0.014)                }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form unit for setting program preferences.               }
{                                                                   }
{   Captions on TabSheets were changed (5.0.014).                   }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Spin, StdCtrls, ComCtrls, FileCtrl, Uglobals, Uutils, ExtCtrls, CheckLst,
  Buttons;

type
  TPreferencesForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Label5: TLabel;
    EditTempDir: TEdit;
    Label7: TLabel;
    Label8: TLabel;
    SubcatchVarBox: TComboBox;
    SubcatchVarSpin: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    NodeVarBox: TComboBox;
    NodeVarSpin: TSpinEdit;
    Label3: TLabel;
    Label4: TLabel;
    LinkVarBox: TComboBox;
    LinkVarSpin: TSpinEdit;
    BtnOK: TButton;
    BtnCancel: TButton;
    BtnHelp: TButton;
    CheckListBox1: TCheckListBox;
    DirSelectBtn: TBitBtn;
    Label6: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure SubcatchVarBoxChange(Sender: TObject);
    procedure SubcatchVarSpinChange(Sender: TObject);
    procedure LinkVarBoxChange(Sender: TObject);
    procedure LinkVarSpinChange(Sender: TObject);
    procedure NodeVarSpinChange(Sender: TObject);
    procedure NodeVarBoxChange(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure DirSelectBtnClick(Sender: TObject);
    procedure BtnHelpClick(Sender: TObject);
    procedure EditTempDirChange(Sender: TObject);
  private
    { Private declarations }
    SubcatchDigits: array[0..SUBCATCHVIEWS] of Integer;
    NodeDigits: array[0..NODEVIEWS] of Integer;
    LinkDigits: array[0..LINKVIEWS] of Integer;
    function SetPreferences: Boolean;
  public
    { Public declarations }
  end;

//var
//  PreferencesForm: TPreferencesForm;

implementation

{$R *.DFM}

uses
  Fmain;

const
  MSG_NO_DIRECTORY = ' - directory does not exist.';

////  Modified by LR - 5.0.010  ////
  MSG_DECIMALS = 'Select number of decimal places for computed results:';

////  Added by LR - 9/5/05  ////
////  Updated by LR - 5.0.010  ////
  PrefersList: array[0..9] of PChar =
    ('Bold Fonts', 'Large Fonts', 'Blinking Map Highlighter',
     'Flyover Map Labeling', 'Confirm Deletions', 'Automatic Backup File',
     'Tab Delimited Project File', 'Report Elapsed Time by Default',
     'Prompt to Save Results',  'Clear File List');

procedure TPreferencesForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
// OnCreate handler for form.
//-----------------------------------------------------------------------------
var
  I: Integer;
  J: Integer;
begin
  // Set font size & style
  Uglobals.SetFont(self);

  // Initialize general preferences
  with CheckListBox1 do
  begin

////  Added by LR - 9/5/05  ////
////  Updated by LR - 5.0.010  ////
    for I := 0 to High(PrefersList) do Items.Add(PrefersList[I]);
    Checked[6] := TabDelimited;
    Checked[7] := RptElapsedTime;
    Checked[8] := not AutoSave;

    Checked[0] := BoldFonts;
    Checked[1] := LargeFonts;
    Checked[2] := Blinking;
    Checked[3] := FlyOvers;
    Checked[4] := ConfirmDelete;
    Checked[5] := AutoBackup;

  end;

  EditTempDir.Text := TempDir;

  // Assign items to node & link variable combo boxes
  J := 0;
  for I := SUBCATCHOUTVAR1 to SUBCATCHVIEWS do
  begin
    SubcatchVarBox.Items.Add(SubcatchVariable[I].Name);
    SubcatchDigits[J] := SubcatchUnits[I].Digits;
    Inc(J);
  end;
  J := 0;
  for I := NODEOUTVAR1 to NODEVIEWS do
  begin
    NodeVarBox.Items.Add(NodeVariable[I].Name);
    NodeDigits[J] := NodeUnits[I].Digits;
    Inc(J);
  end;
  J := 0;
  for I := LINKOUTVAR1 to LINKVIEWS do
  begin
    LinkVarBox.Items.Add(LinkVariable[I].Name);
    LinkDigits[J] := LinkUnits[I].Digits;
    Inc(J);
  end;
  SubcatchVarBox.ItemIndex := 0;
  SubcatchVarSpin.Value := SubcatchDigits[0];
  NodeVarBox.ItemIndex := 0;
  NodeVarSpin.Value := NodeDigits[0];
  LinkVarBox.ItemIndex := 0;
  LinkVarSpin.Value := LinkDigits[0];

  Label6.Caption := MSG_DECIMALS;
  PageControl1.ActivePage := TabSheet1;
end;

procedure TPreferencesForm.EditTempDirChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for the Temp Directory edit box.
//-----------------------------------------------------------------------------
begin
  with EditTempDir do Hint := Text;
end;

procedure TPreferencesForm.SubcatchVarSpinChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for SpinEdit control that sets the number of
// decimal places for a subcatchment variable.
//-----------------------------------------------------------------------------
begin
  SubcatchDigits[SubcatchVarBox.ItemIndex] := SubcatchVarSpin.Value;
end;

procedure TPreferencesForm.NodeVarSpinChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for SpinEdit control that sets the number of
// decimal places for a node variable.
//-----------------------------------------------------------------------------
begin
  NodeDigits[NodeVarBox.ItemIndex] := NodeVarSpin.Value;
end;

procedure TPreferencesForm.LinkVarSpinChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for SpinEdit control that sets the number of
// decimal places for a link variable.
//-----------------------------------------------------------------------------
begin
  LinkDigits[LinkVarBox.ItemIndex] := LinkVarSpin.Value;
end;

procedure TPreferencesForm.SubcatchVarBoxChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for the combo box that selects a subcatchment variable.
//-----------------------------------------------------------------------------
begin
  SubcatchVarSpin.Value := SubcatchDigits[SubcatchVarBox.ItemIndex];
end;

procedure TPreferencesForm.NodeVarBoxChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for the combo box that selects a node variable.
//-----------------------------------------------------------------------------
begin
  NodeVarSpin.Value := NodeDigits[NodeVarBox.ItemIndex];
end;

procedure TPreferencesForm.LinkVarBoxChange(Sender: TObject);
//-----------------------------------------------------------------------------
// OnChange handler for the combo box that selects a link variable.
//-----------------------------------------------------------------------------
begin
  LinkVarSpin.Value := LinkDigits[LinkVarBox.ItemIndex];
end;

procedure TPreferencesForm.BtnOKClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for OK button.
//-----------------------------------------------------------------------------
begin
  if SetPreferences then ModalResult := mrOK;
end;

procedure TPreferencesForm.BtnCancelClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Cancel button.
//-----------------------------------------------------------------------------
begin
  ModalResult := mrCancel;
end;

procedure TPreferencesForm.DirSelectBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for button that searches for a choice of Temporary Folder.
// Uses built-in Delphi function SelectDirectory to display a directory
// selection dialog box.
//-----------------------------------------------------------------------------
var
  S: String;
begin
  S := EditTempDir.Text;
  if SelectDirectory(S, [sdAllowCreate, sdPerformCreate, sdPrompt], 0)
  then EditTempDir.Text := S;
end;

function TPreferencesForm.SetPreferences: Boolean;
//-----------------------------------------------------------------------------
// Transfers contents of form to program preference variables.
//-----------------------------------------------------------------------------
var
  I: Integer;
  J: Integer;
  Buffer: String;
begin
  // Use default temporary directory if entry field is blank
  Buffer := Trim(EditTempDir.Text);
  if (Length(Buffer) = 0) then
  begin
    TempDir := Uutils.GetTempFolder;
    if TempDir = '' then TempDir := EpaSwmmDir;
  end

  // Otherwise check that user's directory choice exists
  else
  begin
    if Buffer[Length(Buffer)] <> '\' then Buffer := Buffer + '\';
    if DirectoryExists(Buffer) then TempDir := Buffer
    else
    begin
      MessageDlg(Buffer + MSG_NO_DIRECTORY, mtWARNING, [mbOK], 0);
      EditTempDir.Text := TempDir;
      PageControl1.ActivePage := TabSheet1;
      EditTempDir.SetFocus;
      Result := False;
      Exit;
    end;
  end;

  // Save the other preferences to their respective global variables.
  with CheckListBox1 do
  begin
    BoldFonts     := Checked[0];
    LargeFonts    := Checked[1];
    Blinking      := Checked[2];
    FlyOvers      := Checked[3];
    ConfirmDelete := Checked[4];
    AutoBackup    := Checked[5];

////  Added by LR - 9/5/05  ////
////  Updated by LR - 5.0.010  ////
    TabDelimited := Checked[6];
    RptElapsedTime := Checked[7];
    AutoSave := not Checked[8];
  end;

  // Clear Most Recently Used file list

////  Changed by LR - 5.0.010  ////
  if CheckListBox1.Checked[9] then

    for I := 0 to MainForm.MRUList.Count-1 do MainForm.MRUList[I] := '';

  // Save number of display decimal places for output view variables
  J := 0;
  for I := SUBCATCHOUTVAR1 to SUBCATCHVIEWS do
  begin
    SubcatchUnits[I].Digits := SubcatchDigits[J];
    Inc(J);
  end;
  J := 0;
  for I := NODEOUTVAR1 to NODEVIEWS do
  begin
    NodeUnits[I].Digits := NodeDigits[J];
    Inc(J);
  end;
  J := 0;
  for I := LINKOUTVAR1 to LINKVIEWS do
  begin
    LinkUnits[I].Digits := LinkDigits[J];
    Inc(J);
  end;
  Result := True;
end;

procedure TPreferencesForm.BtnHelpClick(Sender: TObject);
begin
   with PageControl1 do
     if ActivePage = TabSheet1 then
       Application.HelpCommand(HELP_CONTEXT, 210910)
     else
       Application.HelpCommand(HELP_CONTEXT, 210920);
end;

end.
