unit Dproplot;

{-------------------------------------------------------------------}
{                    Unit:    Dproplot.pas                          }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/31/05     (5.0.005)                 }
{                             2/4/08      (5.0.012)                 }
{                             1/21/09     (5.0.014)                 }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form unit for editing the display options of a           }
{   profile plot.                                                   }
{                                                                   }
{   Checkbox for Include Ground Profile option added in             }
{   Release 5.0.012.                                                }
{                                                                   }
{   Vertical Scale page added in Release 5.0.014.                   }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Spin, StdCtrls, ExtCtrls, ComCtrls, Uglobals, NumEdit,
  Chart, TeEngine;

type
  TProfilePlotOptionsForm = class(TForm)
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    ColorBox1: TColorBox;
    ColorBox2: TColorBox;
    ColorBox3: TColorBox;
    ColorBox4: TColorBox;
    MainTitleEdit: TEdit;
    HorizAxisEdit: TEdit;
    VertAxisEdit: TEdit;
    MainTitleFontBtn: TButton;
    AxisFontBtn: TButton;
    HorizGridBox: TCheckBox;
    VertGridBox: TCheckBox;
    LabelsOnAxisBox: TCheckBox;
    LabelsOnPlotBox: TCheckBox;
    ArrowLengthSpin: TSpinEdit;
    LabelSizeSpin: TSpinEdit;
    DefaultBox: TCheckBox;
    OKBtn: TButton;
    CancelBtn: TButton;
    HelpBtn: TButton;
    FontDialog: TFontDialog;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    CheckBox1: TCheckBox;
    TabSheet4: TTabSheet;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    YminEdit: TNumEdit;
    YmaxEdit: TNumEdit;
    YincEdit: TNumEdit;
    YminLabel: TLabel;
    YmaxLabel: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure MainTitleFontBtnClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure AxisFontBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    LeftAxis: TChartAxis;
    procedure LoadOptions(theGraphOptions: TGraphOptions;
      theProfileOptions: TProfileOptions; theMainTitle: String;
      theXtitle: String; theYtitle: String; IncludeHGL: Boolean);               //(5.0.012 - LR)
    procedure UnloadOptions(var theGraphOptions: TGraphOptions;
      var theProfileOptions: TProfileOptions; var theMainTitle: String;
      var theXtitle: String; var theYtitle: String; var IncludeHGL: Boolean);   //(5.0.012 - LR)
    procedure SetLeftAxisScaling(Ymax: Double; theChart: TChart);               //(5.0.014 - LR)
    procedure GetLeftAxisScaling(theChart: TChart);                             //(5.0.014 - LR)
  end;

//var
//  ProfilePlotOptionsForm: TProfilePlotOptionsForm;

implementation

{$R *.dfm}

procedure TProfilePlotOptionsForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnCreate handler.
//-----------------------------------------------------------------------------
begin
  Uglobals.SetFont(self);
end;

procedure TProfilePlotOptionsForm.LoadOptions(theGraphOptions: TGraphOptions;
  theProfileOptions: TProfileOptions; theMainTitle: String; theXtitle: String;
  theYtitle: String; IncludeHGL: Boolean);                                      //(5.0.012 - LR)
//-----------------------------------------------------------------------------
//  Loads a set of graph options, profile plot options, main title, and
//  axis titles into the form.
//-----------------------------------------------------------------------------
begin
  with MainTitleEdit do
  begin
    Text := theMainTitle;
    Font.Color := theGraphOptions.TitleFontColor;
    Font.Name := theGraphOptions.TitleFontName;
    Font.Size := theGraphOptions.TitleFontSize;
    Font.Style := [];
    if theGraphOptions.TitleFontBold
    then Font.Style := Font.Style + [fsBold];
    if theGraphOptions.TitleFontItalic
    then Font.Style := Font.Style + [fsItalic];
  end;
  with HorizAxisEdit do
  begin
    Text := theXtitle;
    Font.Color := clBlack;
    Font.Name := theGraphOptions.AxisFontName;
    Font.Size := theGraphOptions.AxisFontSize;
    Font.Style := [];
    if theGraphOptions.AxisFontBold
    then Font.Style := Font.Style + [fsBold];
    if theGraphOptions.AxisFontItalic
    then Font.Style := Font.Style + [fsItalic];
  end;
  VertAxisEdit.Text := theYtitle;
  VertAxisEdit.Font.Assign(HorizAxisEdit.Font);
  with theProfileOptions do
  begin
    LabelsOnAxisBox.Checked :=  LabelsOnAxis;
    LabelsOnPlotBox.Checked := LabelsOnPlot;
    ArrowLengthSpin.Value := LabelsArrowLength;
    LabelSizeSpin.Value := LabelsFontSize;
  end;
  HorizGridBox.Checked := (theGraphOptions.AxisGridStyle[1] > 0);
  VertGridBox.Checked  := (theGraphOptions.AxisGridStyle[0] > 0);
  ColorBox1.Selected := theGraphOptions.PanelColor;
  ColorBox2.Selected := theGraphOptions.BackColor;
  ColorBox3.Selected := theProfileOptions.ConduitColor;
  ColorBox4.Selected := theProfileOptions.WaterColor;
  CheckBox1.Checked := not IncludeHGL;                                          //(5.0.012 - LR)
end;

procedure TProfilePlotOptionsForm.SetLeftAxisScaling(Ymax: Double; theChart: TChart);
//-----------------------------------------------------------------------------
//  Sets the entries in the edit boxes used for vertical axis scaling
//
////  New function added for Release 5.0.014  ////
//-----------------------------------------------------------------------------
begin
  YminEdit.Text := FloatToStr(theChart.LeftAxis.Minimum);
  YmaxEdit.Text := FloatToStr(theChart.LeftAxis.Maximum);
  YincEdit.Text := FloatToStr(theChart.LeftAxis.Increment);
  YminLabel.Caption := '(' + FloatToStrF(theChart.MinYValue(theChart.LeftAxis),
                             ffFixed, 15, 2) + ')';
  YmaxLabel.Caption := '(' + FloatToStrF(Ymax, ffFixed, 15, 2) + ')';
end;

procedure TProfilePlotOptionsForm.UnloadOptions(
  var theGraphOptions: TGraphOptions; var theProfileOptions: TProfileOptions;
  var theMainTitle: String; var theXtitle: String; var theYtitle: String;       //(5.0.012 - LR)
  var IncludeHGL: Boolean);                                                     //(5.0.012 - LR)
//-----------------------------------------------------------------------------
//  Unloads graph options, profile plot options, main title, and axis
//  titles from the form.
//-----------------------------------------------------------------------------
begin
  theMainTitle := MainTitleEdit.Text;
  theXtitle := HorizAxisEdit.Text;
  theYtitle := VertAxisEdit.Text;
  IncludeHGL := not Checkbox1.Checked;                                          //(5.0.012 - LR)
  with theProfileOptions do
  begin
    LabelsOnAxis := LabelsOnAxisBox.Checked;
    LabelsOnPlot := LabelsOnPlotBox.Checked;
    LabelsArrowLength := ArrowLengthSpin.Value;
    LabelsFontSize := LabelSizeSpin.Value;
    ConduitColor := ColorBox3.Selected;
    WaterColor := ColorBox4.Selected;
  end;
  with theGraphOptions do
  begin
    PanelColor := ColorBox1.Selected;
    BackColor  := ColorBox2.Selected;
    AxisGridStyle[1] := Ord(HorizGridBox.Checked);
    AxisGridStyle[0] := Ord(VertGridBox.Checked);
    with MainTitleEdit.Font do
    begin
      TitleFontName := Name;
      TitleFontColor := Color;
      TitleFontSize := Size;
      TitleFontBold := (fsBold in Style);
      TitleFontItalic := (fsItalic in Style);
    end;
    with HorizAxisEdit.Font do
    begin
      AxisFontName := Name;
      AxisFontSize := Size;
      AxisFontBold := (fsBold in Style);
      AxisFontItalic := (fsItalic in Style);
    end;
  end;
end;

procedure TProfilePlotOptionsForm.GetLeftAxisScaling(theChart: TChart);
//-----------------------------------------------------------------------------
//  Retrieves vertical axis scaling options
//
////  New function added for Release 5.0.014  ////
//-----------------------------------------------------------------------------
begin
  with theChart.LeftAxis do
  begin
    AutomaticMinimum := False;
    Minimum :=  StrToFloat(YminEdit.Text);
    AutomaticMaximum := False;
    Maximum := StrToFloat(YmaxEdit.Text);
    Increment := StrToFloat(YincEdit.Text);
  end;
end;

procedure TProfilePlotOptionsForm.MainTitleFontBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the font button used to change the font properties
//  of the main title displayed on the profile plot.
//-----------------------------------------------------------------------------
begin
  with FontDialog do
  begin
    Font.Assign(MainTitleEdit.Font);
    Options := Options + [fdEffects];
    if Execute then MainTitleEdit.Font.Assign(Font);
  end;
end;

procedure TProfilePlotOptionsForm.AxisFontBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the font button used to change the font properties
//  of the axes displayed on the profile plot.
//-----------------------------------------------------------------------------
begin
  with FontDialog do
  begin
    Font.Assign(HorizAxisEdit.Font);
    Options := Options - [fdEffects];
    if Execute then
    begin
      HorizAxisEdit.Font.Assign(Font);
      VertAxisEdit.Font.Assign(Font);
    end;
  end;
end;

procedure TProfilePlotOptionsForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTEXT, 211380);
end;

end.
