unit Dreport;

{-------------------------------------------------------------------}
{                    Unit:    Dreport.pas                           }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    3/31/05     (5.0.005)                 }
{                             9/5/05      (5.0.006)                 }
{                             3/10/06     (5.0.007)                 }
{                             6/19/07     (5.0.010)                 }
{                             2/4/08      (5.0.012)                 }
{                             1/21/09     (5.0.014)                 }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Stay-on-top form unit used to define the contents of graphical  }
{   and tabular reports. It consists of a Notebook control with     }
{   three pages; one for time series plots and tables, one for      }
{   profile plots, and  one for scatter plots. When the user        }
{   clicks the OK button, the selections made on the form are       }
{   placed into a TReportSelection record (see Uglobals.pas for     }
{   description) and the MainForm's CreateReport procedure is       }
{   called.                                                         }
{-------------------------------------------------------------------}

////  The original LinksListBox used to list links selected for  ////
////  a profile plot was changed from a TMemo control to a       ////
////  TListBox control and speed buttons were added for          ////
////  inserting, deleting and moving items in the list.          ////
////  (LR - 9/5/05)                                              ////

////  A new speed button was added to clear the LinksListBox on  ////
///   the profile plot page of the dialog for Release 5.0.014.   ////

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons, CheckLst, Contnrs,
  Spin, NumEdit, Uglobals, Uproject;

type
  TReportSelectForm = class(TForm)
    BtnOK: TButton;
    BtnCancel: TButton;
    BtnHelp: TButton;

    Notebook1: TNotebook;
    Label1: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    StartDateCombo1: TComboBox;
    EndDateCombo1: TComboBox;
    Label3: TLabel;
    VariableListBox: TCheckListBox;
    CategoryCombo: TComboBox;

    GroupBox1: TGroupBox;
    Label7: TLabel;
    StartNodeEdit: TEdit;
    StartNodeBtn: TBitBtn;
    Label8: TLabel;
    EndNodeEdit: TEdit;
    EndNodeBtn: TBitBtn;
    FindPathBtn: TButton;
    ProfileLinksBox: TGroupBox;
    UseProfileBtn: TButton;
    SaveProfileBtn: TButton;

    Label2: TLabel;
    Label6: TLabel;
    StartDateCombo2: TComboBox;
    EndDateCombo2: TComboBox;
    XVariableBox: TGroupBox;
    Label9: TLabel;
    XCategoryCombo: TComboBox;
    Label10: TLabel;
    XObjectEdit: TEdit;
    XObjectBtn: TBitBtn;
    Label11: TLabel;
    XVariableCombo: TComboBox;
    Label15: TLabel;
    TimeAxisCombo: TComboBox;
    YVariableBox: TGroupBox;
    Label12: TLabel;
    YCategoryCombo: TComboBox;
    Label13: TLabel;
    YObjectEdit: TEdit;
    YObjectBtn: TBitBtn;
    Label14: TLabel;
    YVariableCombo: TComboBox;
    ObjectsPanel: TPanel;
    ObjectsLabel: TLabel;
    ItemsListBox: TListBox;
    BtnAdd: TBitBtn;
    BtnDelete: TBitBtn;
    BtnMoveUp: TBitBtn;
    BtnMoveDown: TBitBtn;

////  New components added. (LR - 9/5/05)  ////
    BtnAddLink: TBitBtn;
    BtnDelLink: TBitBtn;
    BtnLinkUp: TBitBtn;
    BtnLinkDown: TBitBtn;
    BtnClearLinks: TBitBtn;
    LinksListBox: TListBox;

    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BtnOKClick(Sender: TObject);
    procedure BtnCancelClick(Sender: TObject);
    procedure BtnHelpClick(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure BtnDeleteClick(Sender: TObject);
    procedure BtnMoveUpClick(Sender: TObject);
    procedure BtnMoveDownClick(Sender: TObject);
    procedure CategoryComboClick(Sender: TObject);
    procedure StartEndNodeBtnClick(Sender: TObject);
    procedure FindPathBtnClick(Sender: TObject);
    procedure SaveProfileBtnClick(Sender: TObject);
    procedure StartEndNodeEditChange(Sender: TObject);
    procedure UseProfileBtnClick(Sender: TObject);
    procedure VariableListBoxClick(Sender: TObject);
    procedure FormShow(Sender: TObject);

////  New event handlers added. (LR - 9/5/05)  ////
    procedure BtnAddLinkClick(Sender: TObject);
    procedure BtnDelLinkClick(Sender: TObject);
    procedure BtnLinkDownClick(Sender: TObject);
    procedure BtnLinkUpClick(Sender: TObject);
    procedure BtnClearLinksClick(Sender: TObject);
  private
    { Private declarations }
    ReportTypeChoice: Integer;         // Type of report being generated
    MaxObjects: Integer;               // Max. number of items being reported on
    MultiVariable: Boolean;            // True if > 1 variable being reported on
    procedure AddToItemList;
    procedure CreateReport;
    procedure FreeAdjLists;
    function  GetObjectType(const ItemIndex: Integer): Integer;
    procedure CreateAdjLists;

    procedure GetMinPath(StartNode: TNode; EndNode: TNode;                      //(5.0.010 - LR)
              NodeStack: TQueue; LinkStack: TStack);                            //(5.0.012 - LR)
    procedure Initialize;                                                       //(5.0.010 - LR)
    function  CheckProfile: Integer;                                            //(5.0.014 - LR)


  public
    { Public declarations }
    procedure SetReportType(const aReportType: Integer);
  end;

var
  ReportSelectForm: TReportSelectForm;      // Do not comment this out.

implementation

{$R *.DFM}

uses
  Dprofile, Fmain, Fmap, Ubrowser, Uoutput;

const
  TXT_SYSTEM = 'System';
  TXT_TIME_SERIES_PLOT = 'Time Series Plot';
  TXT_SCATTER_PLOT = 'Scatter Plot';
  TXT_PROFILE_PLOT = 'Profile Plot';
  TXT_TABLE_BY_VARIABLE = 'Table by Variable';
  TXT_TABLE_BY_OBJECT = 'Table by Object';
  TXT_OBJECTS_LABEL1 =
  'Select an object from the Map or the Browser. ';
  TXT_OBJECTS_LABEL2 =
  'Then click the "+" button to add it to the ';
  TXT_OBJECTS_LABEL_PLOT = 'plot.';
  TXT_OBJECTS_LABEL_TABLE = 'table.';
  TXT_LINKS_IN_PROFILE = 'Links in Profile';
  TXT_PROFILE_NAME = 'Profile Name';

  MSG_NO_VARIABLE = 'No variable was selected';
  MSG_NO_ITEMS = 'No objects to display';
  MSG_TOO_FEW_ITEMS = 'Too few objects to display';
  MSG_TOO_MANY_ITEMS = 'Too many objects to display';
  MSG_NO_RESULTS = 'No results available to display';
  MSG_NO_SUBCATCH_SELECTED = 'Must select a subcatchment.';
  MSG_NO_NODE_SELECTED = 'Must select a node.';
  MSG_NO_LINK_SELECTED = 'Must select a link.';
  MSG_INVALID_DATES = 'End date comes before start date.';
  MSG_START_NODE_NOT_EXIST = 'Starting node does not exist.';
  MSG_END_NODE_NOT_EXIST = 'Ending node does not exist.';
  MSG_NO_VALID_PATH = 'Cannot find a path between start and end nodes.';
  MSG_PROFILE_EXISTS = 'A profile with that name already exists.';
  MSG_NO_RESULTS_AVAILABLE = 'There are no results available for this object.'; //(5.0.014 - LR)

  Categories: String = 'Subcatchments'#13'Nodes'#13'Links';                     //(5.0.010 - LR)

procedure TReportSelectForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
// Form's OnCreate event handler.
//-----------------------------------------------------------------------------
begin
////  Contents moved to new Initialize procedure.  ////                         //(5.0.010 - LR)
end;

procedure TReportSelectForm.FormShow(Sender: TObject);
//-----------------------------------------------------------------------------
// Form's OnShow event handler.
//-----------------------------------------------------------------------------
begin
  // Use global font currently in effect
  Uglobals.SetFont(self);
end;

procedure TReportSelectForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
//-----------------------------------------------------------------------------
// Form's OnClose event handler.
//-----------------------------------------------------------------------------
begin
  // Clear controls on the Profile Plot page                                    //(5.0.012 - LR)
  StartNodeEdit.Text := '';
  EndNodeEdit.Text := '';
  LinksListBox.Clear;
//  Action := caFree;                                                           //(5.0.010 - LR)
end;

///////////////////////////////////////////////////////////////////////////
////  This procedure contains code that was previously in FormCreate.  ////     //(5.0.010 - LR)
///////////////////////////////////////////////////////////////////////////
procedure TReportSelectForm.Initialize;
//-----------------------------------------------------------------------------
// Initializes the contents of the form.
//-----------------------------------------------------------------------------
begin
  // Add items to the object category combo box
  CategoryCombo.Items.Clear;
  CategoryCombo.Items.Text := Categories;

  // Select default type of object to report on
  if Project.IsSubcatch(CurrentList) then
  begin
    CategoryCombo.ItemIndex := 0;
    AddToItemList;
  end
  else if Project.IsNode(CurrentList) then
  begin
    CategoryCombo.ItemIndex := 1;
    AddToItemList;
  end
  else if Project.IsLink(CurrentList) then
  begin
    CategoryCombo.ItemIndex := 2;
    AddToItemList;
  end
  else CategoryCombo.ItemIndex := 0;

  // Select to report by elapsed time or by date/time
  if RptElapsedTime then TimeAxisCombo.ItemIndex := 0
  else TimeAxisCombo.ItemIndex := 1;
  MultiVariable := False;

  // Activate item selection buttons                                            //(5.0.014 - LR)
  BtnDelete.Enabled := True;
  BtnMoveUp.Enabled := True;
  BtnMoveDown.Enabled := True;

end;

procedure TReportSelectForm.SetReportType(const aReportType: Integer);
//-----------------------------------------------------------------------------
// Sets up the form to display choices for a particular type of report.
//-----------------------------------------------------------------------------
var
  S: String;
begin
  // Initialize the contents of the form                                        //(5.0.010 - LR)
  Initialize;                                                                   //(5.0.010 - LR)

  // Save the type of report being created
  ReportTypeChoice := aReportType;
  S := TXT_OBJECTS_LABEL1 + TXT_OBJECTS_LABEL2;

  // Report is a time series plot
  if ReportTypeChoice = TIMESERIESPLOT then
  begin
    Caption := TXT_TIME_SERIES_PLOT;
    Notebook1.PageIndex := 0;
    MaxObjects := MAXSERIES + 1;
    CategoryCombo.Items.Add(TXT_SYSTEM);

////  Next line moved to here from FormCreate procedure. (LR - 9/5/05)  ////
    CategoryComboClick(CategoryCombo);
  end;

  // Report is a scatter plot
  if ReportTypeChoice = SCATTERPLOT then
  begin
    Caption := TXT_SCATTER_PLOT;
    Notebook1.PageIndex := 2;
    XCategoryCombo.Items := CategoryCombo.Items;
    XCategoryCombo.ItemIndex := CategoryCombo.ItemIndex;
    CategoryComboClick(XCategoryCombo);
    YCategoryCombo.Items := CategoryCombo.Items;
    YCategoryCombo.ItemIndex := CategoryCombo.ItemIndex;
    CategoryComboClick(YCategoryCombo);
  end;

  // Report is a profile plot
  if ReportTypeChoice = PROFILEPLOT then
  begin
    Caption := TXT_PROFILE_PLOT;
    UseProfileBtn.Enabled := (Project.ProfileNames.Count > 0);
    SaveProfileBtn.Enabled := False;
    Notebook1.PageIndex := 1;
  end;

  // Report is a time series table for a specific variable
  if ReportTypeChoice = TABLEBYVARIABLE then
  begin
    Caption := TXT_TABLE_BY_VARIABLE;
    Notebook1.PageIndex := 0;
    MaxObjects := 11;  //Largest number of columns allowed in the table

////  Next line moved to here from FormCreate procedure. (LR - 9/5/05)  ////
    CategoryComboClick(CategoryCombo);

  end;
  if ReportTypeChoice = TABLEBYOBJECT then
  begin
    Caption := TXT_TABLE_BY_OBJECT;
    Notebook1.PageIndex := 0;
    MultiVariable := True;
    BtnDelete.Enabled := False;
    BtnMoveUp.Enabled := False;
    BtnMoveDown.Enabled := False;
    MaxObjects := 1;
    CategoryCombo.Items.Add(TXT_SYSTEM);

////  Next line moved to here from FormCreate procedure. (LR - 9/5/05)  ////
    CategoryComboClick(CategoryCombo);

  end;

  // Assign dates to date combo boxes
  if Notebook1.PageIndex = 0 then
  begin
    StartDateCombo1.Items := MainForm.DateListBox.Items;
    EndDateCombo1.Items := MainForm.DateListBox.Items;
    StartDateCombo1.ItemIndex := 0;
    EndDateCombo1.ItemIndex := EndDateCombo1.Items.Count - 1;
  end;
  if Notebook1.PageIndex = 2 then
  begin
    StartDateCombo2.Items := MainForm.DateListBox.Items;
    EndDateCombo2.Items := MainForm.DateListBox.Items;
    StartDateCombo2.ItemIndex := 0;
    EndDateCombo2.ItemIndex := EndDateCombo2.Items.Count - 1;
  end;
end;

procedure TReportSelectForm.CategoryComboClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick event handler for CategoryCombo control. Changes entries in the
// VariableCombo control depending on whether Subcatchments, Nodes, Links
// or System was selected.
//-----------------------------------------------------------------------------
var
  I: Integer;
  Receiver: TComboBox;
begin
  // Identify which combobox needs to be updated
  if      Sender = CategoryCombo  then Receiver := XVariableCombo
  else if Sender = XCategoryCombo then Receiver := XVariableCombo
  else if Sender = YCategoryCombo then Receiver := YVariableCombo
  else Exit;

  // Check which class of object was selected
  with Sender as TComboBox do
  begin
    Receiver.Items.Clear;
    Case ItemIndex of
    // Subcatchments were selected
    0: begin
         // Load subcatchment variables into Receiver combobox
         for I := Uglobals.SUBCATCHOUTVAR1 to
                  MainForm.SubcatchViewBox.Items.Count-1 do
           Receiver.Items.Add(MainForm.SubcatchViewBox.Items[I]);
         Receiver.ItemIndex := 0;

         // Place current subcatchment from Browser into the ItemsList listbox
         if Notebook1.PageIndex = 0 then
         begin
           ItemsListBox.Clear;
           if ItemsListBox.Enabled then
             if Project.IsSubcatch(CurrentList) then AddToItemList;
         end;
       end;

    // Nodes were selected
    1: begin
         // Load node variables into the Variable combobox
         for I := Uglobals.NODEOUTVAR1 to MainForm.NodeViewBox.Items.Count-1 do
           Receiver.Items.Add(MainForm.NodeViewBox.Items[I]);
         Receiver.ItemIndex := 0;

         // Place current node from Browser into the ItemsList listbox
         if Notebook1.PageIndex = 0 then
         begin
           ItemsListBox.Clear;
           if ItemsListBox.Enabled then
             if Project.IsNode(CurrentList) then AddToItemList;
         end;
       end;

    // Links were selected
    2: begin
         // Load link variables into the Variable combobox
         for I := Uglobals.LINKOUTVAR1 to MainForm.LinkViewBox.Items.Count-1 do
           Receiver.Items.Add(MainForm.LinkViewBox.Items[I]);
         Receiver.ItemIndex := 0;

         // Place current link from Browser into the ItemsList listbox
         if Notebook1.PageIndex = 0 then
         begin
           ItemsListBox.Clear;
           if ItemsListBox.Enabled then
             if Project.IsLink(CurrentList) then AddToItemList;
         end;
       end;

    // System was selected
    3: begin
         // Load system variables into Variable combobox
         for I := 0 to Uglobals.SYSVIEWS do
           Receiver.Items.Add(Uglobals.SysViewNames[I]);
         Receiver.ItemIndex := 0;
       end;
    end;
  end;

  // Finish setting up the time series plot/table page
  if Notebook1.PageIndex = 0 then
  begin
    if CategoryCombo.ItemIndex = SYS then ObjectsPanel.Visible := False
    else ObjectsPanel.Visible := True;
    ObjectsLabel.Caption := CategoryCombo.Text;
    VariableListBox.Items.Assign(XVariableCombo.Items);
    VariableListBox.ItemIndex := 0;
    if CategoryCombo.ItemIndex <> SYS then VariableListBox.Checked[0] := True;
  end;
end;

procedure TReportSelectForm.VariableListBoxClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the check list box used to select which variable(s)
//  to report on. Makes sure that only one variable is selected if the report
//  is not multi-variable.
//-----------------------------------------------------------------------------
var
  I: Integer;
  isMultiVariable: Boolean;
begin
  if CategoryCombo.ItemIndex = SYS
  then isMultiVariable := True
  else isMultiVariable := Multivariable;
  if not isMultiVariable then with VariableListBox do
  begin
    for I := 0 to Items.Count-1 do Checked[I] := (I = ItemIndex);
  end;
end;

procedure TReportSelectForm.BtnOKClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the OK button.
//-----------------------------------------------------------------------------
var
  I : Integer;
  VariableCount: Integer;
begin
  // Check that one or more objects were selected to report on
  // for profile plots and scatter plots
  if ReportTypeChoice = PROFILEPLOT then
  begin
    //if LinksListBox.Lines.Count <= 0 then
    if LinksListBox.Items.Count <= 0 then
    begin
      MessageDlg(MSG_NO_ITEMS, mtError, [mbOK], 0);
      Exit;
    end;
    I := CheckProfile;                                                          //(5.0.014 - LR)
    if I >= 0 then                                                              //(5.0.014 - LR)
    begin                                                                       //(5.0.014 - LR)
      MessageDlg('Link ' + LinksListBox.Items[I] +                              //(5.0.014 - LR)
                 ' either does not exist or is not connected' +                 //(5.0.014 - LR)
                 ' to the preceding link.', mtError, [mbOK], 0);                //(5.0.014 - LR)
      Exit;                                                                     //(5.0.014 - LR)
    end;                                                                        //(5.0.014 - LR)
  end
  else if ReportTypeChoice = SCATTERPLOT then
  begin
    if (Length(Trim(XObjectEdit.Text)) = 0)
    or (Length(Trim(YObjectEdit.Text)) = 0) then
    begin
      MessageDlg(MSG_TOO_FEW_ITEMS, mtError, [mbOK], 0);
      Exit;
    end;
  end

  // For time series plots/tables,
  else
  begin
    // Check that >= 1 objects were selected
    if  (ItemsListBox.Items.Count <= 0)
    and (CategoryCombo.ItemIndex <> SYS) then
    begin
      MessageDlg(MSG_NO_ITEMS, mtError, [mbOK], 0);
      Exit;
    end

    // Check that too many objects were not selected
    else if ItemsListBox.Items.Count > MaxObjects then
    begin
      MessageDlg(MSG_TOO_MANY_ITEMS, mtError, [mbOK], 0);
      Exit;
    end;

    // Check that >= 1 variable was selected
    VariableCount := 0;
    with VariableListBox do
      for I := 0 to Items.Count-1 do
       if Checked[I] then Inc(VariableCount);
    if VariableCount = 0 then
    begin
      MessageDlg(MSG_NO_VARIABLE, mtError, [mbOK], 0);
      Exit;
    end;
  end;

  // Check that End Date > Start Date
  if ( (Notebook1.PageIndex = 0) and
     (StartDateCombo1.ItemIndex > EndDateCombo1.ItemIndex) )
  or ( (Notebook1.PageIndex = 2) and
     (StartDateCombo2.ItemIndex > EndDateCombo2.ItemIndex) )
  then
  begin
    MessageDlg(MSG_INVALID_DATES, mtError, [mbOK], 0);
    Exit;
  end;

  // Save report selections and create the report
  Hide;
  Application.ProcessMessages;
  CreateReport;
  //Close;                                                                      //(5.0.010 - LR)
end;

procedure TReportSelectForm.BtnCancelClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Cancel button.
//-----------------------------------------------------------------------------
begin
  Hide;                                                                         //(5.0.010 - LR)
  //Close;                                                                      //(5.0.010 - LR)
end;

procedure TReportSelectForm.BtnAddClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Add button. Adds the currently selected object
// appearing in the Data Browser to the items to be plotted.
//-----------------------------------------------------------------------------
var
  theCategory: Integer;
begin
  // Determine the class of object to be reported on
  if      Sender = BtnAdd     then theCategory := CategoryCombo.ItemIndex
  else if Sender = XObjectBtn then theCategory := XCategoryCombo.ItemIndex
  else if Sender = YObjectBtn then theCategory := YCategoryCombo.ItemIndex
  else Exit;

  // Check that results are available for the object                            //(5.0.014 - LR)
  if not Project.HasResults(CurrentList, Project.CurrentItem[CurrentList]) then
  begin
    MessageDlg(MSG_NO_RESULTS_AVAILABLE, mtInformation, [mbOK], 0);
    Exit;
  end;

  // Check that the current object appearing in the Browser is of this class
  case theCategory of
  0: if not Project.IsSubcatch(CurrentList) then
     begin
       MessageDlg(MSG_NO_SUBCATCH_SELECTED, mtError, [mbOK], 0);
       Exit;
     end;
  1: if not Project.IsNode(CurrentList) then
     begin
       MessageDlg(MSG_NO_NODE_SELECTED, mtError, [mbOK], 0);
       Exit;
     end;
  2: if not Project.IsLink(CurrentList) then
     begin
       MessageDlg(MSG_NO_LINK_SELECTED, mtError, [mbOK], 0);
       Exit;
     end;
  end;

  // Add the object to the list of items to be plotted or tabulated
  // ...this is for time series
  if Sender = BtnAdd then
  begin
    with ItemsListBox do
      if (MaxObjects = 1) and (Items.Count = MaxObjects) then Items.Delete(0);
    AddToItemList;
  end
  // ...this is for the X-variable of a scatter plot
  else if Sender = XObjectBtn then with Project do
  begin
    if CurrentItem[CurrentList] >= 0 then
      XObjectEdit.Text := GetID(CurrentList, CurrentItem[CurrentList]);
  end
  // ...this is for the Y-variable of a scatter plot
  else if Sender = YObjectBtn then with Project do
  begin
    if CurrentItem[CurrentList] >= 0 then
      YObjectEdit.Text := GetID(CurrentList, CurrentItem[CurrentList]);
  end;
end;

procedure TReportSelectForm.BtnDeleteClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Delete button. Deletes selected item from Items listbox.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  with ItemsListBox do
  begin
    I := ItemIndex;
    if ItemIndex >= 0 then
      Items.Delete(ItemIndex);
    if Items.Count > 0 then
    begin
      if I < Items.Count then
        ItemIndex := I
      else
        ItemIndex := Items.Count-1;
    end;
  end;
end;

procedure TReportSelectForm.BtnMoveUpClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Move Up button. Moves the selected item in the Items
// list box up one level.
//-----------------------------------------------------------------------------
begin
  with ItemsListBox do
  begin
    if ItemIndex > 0 then Items.Exchange(ItemIndex,ItemIndex-1);
  end;
end;

procedure TReportSelectForm.BtnMoveDownClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Move Down button. Moves the selected item in the Items
// list box down one level.
//-----------------------------------------------------------------------------
begin
  with ItemsListBox do
  begin
    if ItemIndex < Items.Count-1 then Items.Exchange(ItemIndex,ItemIndex+1);
  end;
end;

procedure TReportSelectForm.AddToItemList;
//-----------------------------------------------------------------------------
// Adds a new item (subcatchment, node or link ID) to the Items listbox.
//-----------------------------------------------------------------------------
begin
  with Project do
  begin
    if not HasResults(CurrentList, CurrentItem[CurrentList]) then Exit;         //(5.0.014 - LR)
    if CurrentItem[CurrentList] >= 0 then with ItemsListBox do
    begin
      Items.Add(GetID(CurrentList, CurrentItem[CurrentList]));
      ItemIndex := Items.Count-1;
    end;
  end;
end;

procedure TReportSelectForm.CreateReport;
//-----------------------------------------------------------------------------
// Creates a graph or table based on the selections made on the form.
//-----------------------------------------------------------------------------
var
  I : Integer;
  ReportSelection: TReportSelection;
begin
  with ReportSelection do
  begin
    StartDateIndex := 0;
    EndDateIndex   := 0;
    if ReportTypeChoice = PROFILEPLOT then
    begin
      ObjectType     := LINKS;

////  Following lines replaced. (LR - 9/5/05)  ////
      //Items          := LinksListBox.Lines;
      //ItemCount      := LinksListBox.Lines.Count;
      Items          := LinksListBox.Items;
      ItemCount      := LinksListBox.Items.Count;

      VariableCount  := 0;
      ReportType     := ReportTypeChoice;
    end

    else if ReportTypeChoice = SCATTERPLOT then
    begin
      StartDateIndex := StartDateCombo2.ItemIndex;
      EndDateIndex   := EndDateCombo2.ItemIndex + 1;
      XObjectType    := GetObjectType(XCategoryCombo.ItemIndex);
      ObjectType     := GetObjectType(YCategoryCombo.ItemIndex);
      ItemsListBox.Clear;
      ItemsListBox.Items.Add(XObjectEdit.Text);
      ItemsListBox.Items.Add(YObjectEdit.Text);
      Items := ItemsListBox.Items;
      ItemCount := ItemsListBox.Items.Count;
      Variables[0] := Ubrowser.GetIndexOfVar(XObjectType, XVariableCombo.Text);
      Variables[1] := Ubrowser.GetIndexOfVar(ObjectType,  YVariableCombo.Text);
      VariableCount := 2;
      ReportType := ReportTypeChoice;
    end

    else
    begin
      StartDateIndex  := StartDateCombo1.ItemIndex;
      EndDateIndex    := EndDateCombo1.ItemIndex + 1;
      ObjectType      := GetObjectType(CategoryCombo.ItemIndex);
      Items           := ItemsListBox.Items;
      ItemCount       := ItemsListBox.Items.Count;
      ReportType      := ReportTypeChoice;
      DateTimeDisplay := (TimeAxisCombo.ItemIndex > 0);

      if  (ReportType = TABLEBYVARIABLE)
      or  ( (ReportType = TIMESERIESPLOT) and (ObjectType <> SYS) ) then
      begin
        Variables[0] := Ubrowser.GetIndexOfVar(ObjectType,
                        VariableListBox.Items[VariableListBox.ItemIndex]);
        VariableCount := 1;
      end;

      if (ReportType = TABLEBYOBJECT)
      or ( (ReportType = TIMESERIESPLOT) and (ObjectType = SYS) )
      then with VariableListBox do
      begin
        VariableCount := 0;
        for I := 0 to Items.Count-1 do
        begin
          if Checked[I] then
          begin
            if ObjectType = SYS then Variables[VariableCount] := I
            else Variables[VariableCount] :=
              Ubrowser.GetIndexOfVar(ObjectType, Items[I]);
            Inc(VariableCount);
          end;
        end;
      end;
    end;
  end;
  MainForm.CreateReport(ReportSelection);
end;

function TReportSelectForm.GetObjectType(const ItemIndex: Integer): Integer;
//-----------------------------------------------------------------------------
//  Determines the class of object selected in one of the object category
//  combo boxes.
//-----------------------------------------------------------------------------
begin
  if ItemIndex = 0 then Result := SUBCATCHMENTS
  else if ItemIndex = 1 then Result := NODES
  else if ItemIndex = 2 then Result := LINKS
  else Result := SYS;
end;

//====================== Profile Plot Page Methods ============================

procedure TReportSelectForm.StartEndNodeEditChange(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnChange handler for the Start/End Node edit box.
//-----------------------------------------------------------------------------
begin
  ProfileLinksBox.Caption := TXT_LINKS_IN_PROFILE;
  LinksListBox.Clear;
end;

procedure TReportSelectForm.StartEndNodeBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Start/End Node button. Makes the currently
//  selected node in the Data Browser the starting/ending node for the
//  profile plot.
//-----------------------------------------------------------------------------
var
  S: String;
begin
  with Project do
  begin
    if IsNode(CurrentList) then
    begin
      S := GetID(CurrentList, CurrentItem[CurrentList]);
      if Sender = StartNodeBtn then StartNodeEdit.Text := S;
      if Sender = EndNodeBtn then  EndNodeEdit.Text := S;
    end;
  end;
end;

////////////////////////////////////////////////////////////////////////
////  Start of new procedures for Links List buttons. (LR - 9/5/05) ////

procedure TReportSelectForm.BtnAddLinkClick(Sender: TObject);
var
  I: Integer;
begin
  if not Project.IsLink(CurrentList) then
  begin
    MessageDlg(MSG_NO_LINK_SELECTED, mtError, [mbOK], 0);
    Exit;
  end;
  I := LinksListBox.ItemIndex + 1;
  with Project do
  begin
    if CurrentItem[CurrentList] >= 0 then with LinksListBox do
    begin
      Items.Insert(I, GetID(CurrentList, CurrentItem[CurrentList]));
      ItemIndex := I;
    end;
  end;

////  Added (LR - 3/10/06)  ////
  SaveProfileBtn.Enabled := LinksListBox.Items.Count > 0;

end;

procedure TReportSelectForm.BtnDelLinkClick(Sender: TObject);
var
  I: Integer;
begin
  with LinksListBox do
  begin
    I := ItemIndex;
    if ItemIndex >= 0 then
      Items.Delete(ItemIndex);
    if Items.Count > 0 then
    begin
      if I < Items.Count then
        ItemIndex := I
      else
        ItemIndex := Items.Count-1;
    end;
  end;

////  Added (LR - 3/10/06)  ////
  SaveProfileBtn.Enabled := LinksListBox.Items.Count > 0;
end;

procedure TReportSelectForm.BtnLinkDownClick(Sender: TObject);
begin
  with LinksListBox do
  begin
    if ItemIndex < Items.Count-1 then Items.Exchange(ItemIndex,ItemIndex+1);
  end;
end;

procedure TReportSelectForm.BtnLinkUpClick(Sender: TObject);
begin
  with LinksListBox do
  begin
    if ItemIndex > 0 then Items.Exchange(ItemIndex,ItemIndex-1);
  end;
end;

//// New procedure added. (5.0.014 - LR)  ////
procedure TReportSelectForm.BtnClearLinksClick(Sender: TObject);
begin
  LinksListBox.Clear;
end;

////  End of new procedures for Links List buttons. (LR - 9/5/05) ////
////////////////////////////////////////////////////////////////////////

procedure TReportSelectForm.FindPathBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Find Path button. Finds a path of links between
//  the start and end nodes specified for the profile plot.
//-----------------------------------------------------------------------------
var
  PathStack:  TQueue;   // Delphi's queue & stack class are                     //(5.0.012 - LR)
  FinalStack: TStack;   // defined in the Contrns unit
  StartNode:  TNode;
  EndNode:    TNode;
  aLink:      TLink;
  I, J:       Integer;
begin
  // Get the node objects associated with the names of the start & end nodes
  if not Project.FindNode(StartNodeEdit.Text, I, J) then
  begin
    MessageDlg(MSG_START_NODE_NOT_EXIST, mtError, [mbOK], 0);
    exit;
  end;
  StartNode := Project.GetNode(I, J);
  if not Project.FindNode(EndNodeEdit.Text, I, J) then
  begin
    MessageDlg(MSG_END_NODE_NOT_EXIST, mtError, [mbOK], 0);
    exit;
  end;
  EndNode := Project.GetNode(I, J);

  // Populate link adjacency lists for each node of the project
  CreateAdjLists;
  LinksListBox.Clear;
  SaveProfileBtn.Enabled := False;

  // Create two stack objects to use in finding the profile plot path
  PathStack := TQueue.Create();                                                 //(5.0.012 - LR)
  FinalStack := TStack.Create();

  try
    // Trace a path between the start and end nodes
    GetMinPath(StartNode, EndNode, PathStack, FinalStack);                      //(5.0.010 - LR)

    // Check that a path was found
    if FinalStack.Count = 0
    then MessageDlg(MSG_NO_VALID_PATH, mtError, [mbOK], 0)

    // Unwind the FinalStack to extract the links on the path and add
    // their ID's to the Links On Path list box
    else
    begin
      while FinalStack.Count > 0 do
      begin
        aLink := FinalStack.Pop;
        //LinksListBox.Lines.Add(aLink.ID);
        LinksListBox.Items.Add(aLink.ID);
      end;
      SaveProfileBtn.Enabled := True;
    end;
  finally
    FreeAdjLists;
    PathStack.Free;
    FinalStack.Free;
  end;
end;

procedure TReportSelectForm.CreateAdjLists;
//-----------------------------------------------------------------------------
//  Loops through each link in the project, adding it to the link adjacency
//  list of the node on either end.
//-----------------------------------------------------------------------------
var
  K, L:   Integer;
  aLink:  TLink;
  aNode:  TNode;
begin
  for K := 0 to MAXCLASS do
  begin
    if Project.IsLink(K) then
    begin
      for L := 0 to Project.Lists[K].Count-1 do
      begin
        aLink := Project.GetLink(K,L);
        aLink.Marked := False;
        aNode := aLink.Node1;
        aNode.AdjList.Add(aLink);
        aNode := aLink.Node2;
        aNode.AdjList.Add(aLink);
      end;
    end;
  end;
end;

procedure TReportSelectForm.FreeAdjLists;
//-----------------------------------------------------------------------------
//  Clears the link adjacency list for each node of the project.
//-----------------------------------------------------------------------------
var
  I, J: Integer;
  aNode: TNode;
begin
  for I := 0 to MAXCLASS do
  begin
    if Project.IsNode(I) then
    begin
      for J := 0 to Project.Lists[I].Count-1 do
      begin
        aNode := Project.GetNode(I,J);
        aNode.AdjList.Clear;
      end;
    end;
  end;
end;

//////////////////////////////////////////////
//// New procedure added. (5.0.010 - LR)  ////
//////////////////////////////////////////////
procedure TReportSelectForm.GetMinPath(StartNode: TNode; EndNode: TNode;
  NodeStack: TQueue; LinkStack: TStack);                                        //(5.0.012 - LR)
//-----------------------------------------------------------------------------
//  Finds a path between StartNode and EndNode that consist of the
//  fewest links.
//-----------------------------------------------------------------------------
var
  I, J, L  : Integer;
  N1, N2   : TNode;
  aLink    : TLink;
  MovedBack: Boolean;
begin
  // Assign a high path length to each node
  for I := 0 to MAXCLASS do
  begin
    if Project.IsNode(I) then
    begin
      for J := 0 to Project.Lists[I].Count-1 do
        Project.GetNode(I,J).PathLen := MAXINT;
    end;
  end;

  // Place the start node on the node stack
  StartNode.PathLen := 0;
  NodeStack.Push(StartNode);

  // Repeat until the node stack is empty
  while NodeStack.Count > 0 do
  begin

    // Remove the top node from the stack and examine each connecting link
    N1 := NodeStack.Pop;
    for I := 0 to N1.AdjList.Count-1 do
    begin

      // Mark the link and find the node at its other end
      aLink := N1.AdjList.Items[I];
      if aLink.Marked then continue;
      aLink.Marked := true;
      N2 := aLink.Node2;
      if N1 = N2 then N2 := aLink.Node1;

      // Update the path length to that node
      L := N1.PathLen + 1;

      // If a shorter path results, place the node on the stack
      if L < N2.PathLen then
      begin
        if (N2.PathLen = MAXINT) and (N2 <> EndNode) then NodeStack.Push(N2);
        N2.PathLen := L;
      end;
    end;
  end;

  // If the end node was never reached then exit
  if EndNode.PathLen = MAXINT then Exit;

  // Start with the end node and trace back to the start node
  N1 := EndNode;
  while N1 <> StartNode do
  begin

    // Examine each link connected to the current node
    MovedBack := false;
    for I := 0 to N1.AdjList.Count-1 do
    begin

      // Unmark the link
      aLink := N1.AdjList.Items[I];
      if not aLink.Marked then continue;
      aLink.Marked := false;

      // Identify the node at the other end of the link
      N2 := aLink.Node2;
      if N2 = N1 then N2 := aLink.Node1;

      // If this node is on the shortest path then place the
      // link on the link stack and make this node the next one
      // to examine
      if N2.PathLen = N1.PathLen - 1 then
      begin
        LinkStack.Push(aLink);
        N1 := N2;
        MovedBack := true;
        break;
      end;
    end;
    if not MovedBack then break;
  end;
end;

procedure TReportSelectForm.SaveProfileBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Save Profile button. Saves the current profile
//  path to the project's list of profile paths.
//-----------------------------------------------------------------------------
var
  S: String;
  I: Integer;
begin
  S := InputBox(TXT_PROFILE_NAME, '', '');
  if Project.ProfileNames.Find(S, I)
  then MessageDlg(MSG_PROFILE_EXISTS, mtError, [mbOK], 0)
  else
  begin
    Project.ProfileNames.Add(S);
    ProfileLinksBox.Caption := S;

    //with LinksListBox do
    //begin
      //S := Lines[0];
      //for I := 1 to Lines.Count-1 do
      //  S := S + #13 + Lines[I];
    //end;
    with LinksListBox do
    begin
      S := Items[0];
      for I := 1 to Items.Count-1 do
        S := S + #13 + Items[I];
    end;

    Project.ProfileLinks.Add(S);
    SaveProfileBtn.Enabled := False;
    UseProfileBtn.Enabled := True;
    Uglobals.HasChanged := True;                                                //(5.0.014 - LR)
  end;
end;

procedure TReportSelectForm.UseProfileBtnClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick handler for the Use Profile button. Retrieves one of the
//  project's previously saved profile paths.
//-----------------------------------------------------------------------------
var
  ProfileIndex: Integer;
begin
  ProfileIndex := -1;
  if Project.ProfileNames.Count > 0 then
  with TProfileSelectionForm.Create(self) do
  try
    if ShowModal = mrOK then ProfileIndex := SelectedProfile;
  finally
    Free;
  end;
  if ProfileIndex >= 0 then
  begin
    StartNodeEdit.Text := '';
    EndNodeEdit.Text := '';
    ProfileLinksBox.Caption := Project.ProfileNames[ProfileIndex];
    LinksListBox.Clear;
    //LinksListBox.Lines.SetText(PChar(Project.ProfileLinks[ProfileIndex]));
    LinksListBox.Items.SetText(PChar(Project.ProfileLinks[ProfileIndex]));
    SaveProfileBtn.Enabled := False;
  end;
  if Project.ProfileNames.Count = 0 then UseProfileBtn.Enabled := False;
end;

//////////////////////////////////////////////
//// New procedure added. (5.0.014 - LR)  ////
//////////////////////////////////////////////
function TReportSelectForm.CheckProfile: Integer;
//-----------------------------------------------------------------------------
//  Checks that the links selected for a profile plot form a connected path.
//-----------------------------------------------------------------------------
var
  I, J, K: Integer;
  Link1: TLink;
  Link2: TLink;
  LastNode: TNode;
begin
  Result := 0;
  with LinksListBox do
  begin

  //Check that first two links are connected
    if not Project.FindLink(LinksListBox.Items[0], I, J) then Exit;
    if Items.Count = 1 then
    begin
      Result := -1;
      Exit;
    end;
    Result := 1;
    Link1 := Project.GetLink(I, J);
    if not Project.FindLink(LinksListBox.Items[1], I, J) then Exit;
    Link2 := Project.GetLink(I, J);
    if (Link2.Node1 = Link1.Node1) or (Link2.Node1 = Link1.Node2)
    then LastNode := Link2.Node2
    else if (Link2.Node2 = Link1.Node1) or (Link2.Node2 = Link1.Node2)
    then LastNode := Link2.Node1
    else Exit;

  //Check that remaining links are connected
    for K := 2 to Items.Count-1 do
    begin
      Result := K;
      if not Project.FindLink(LinksListBox.Items[K], I, J) then Exit;
      Link2 := Project.GetLink(I, J);
      if Link2.Node1 = LastNode then LastNode := Link2.Node2
      else if Link2.Node2 = LastNode then LastNode := Link2.Node1
      else Exit;
    end;
    Result := -1;
  end;
end;

procedure TReportSelectForm.BtnHelpClick(Sender: TObject);
var
  HC: Integer;
begin
  case ReportTypeChoice of
    TIMESERIESPLOT:  HC := 211390;
    PROFILEPLOT:     HC := 211370;
    SCATTERPLOT:     HC := 212460;
    TABLEBYOBJECT:   HC := 211400;
    TABLEBYVARIABLE: HC := 211410;
    else HC := 0;
  end;
  if HC > 0 then Application.HelpCommand(HELP_CONTEXT, HC);
end;

end.
