unit Dviewer;

{-------------------------------------------------------------------}
{                    Unit:    Dviewer.pas                           }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    6/19/07     (5.0.010)                 }
{                             2/4/08      (5.0.012)                 }
{                             4/10/09     (5.0.015)                 }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Form unit with an Orpheus TextFileViewer component used to      }
{   view a project's input file.                                    }
{                                                                   }
{   This unit was re-written to support split views. (5.0.012 - LR) }
{   Initial size & position of the form was modified (5.0.015 - LR) }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Menus, Xprinter, VirtList, Grids, StrUtils,
  ovcbase, ovcviewr, ovcdata;               //Orpheus units

type
  TDataViewForm = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    PageSetup1: TMenuItem;
    PrintPreview1: TMenuItem;
    Print1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    Search1: TMenuItem;
    Find1: TMenuItem;
    SearchAgain1: TMenuItem;
    FindDialog1: TFindDialog;
    DataViewer1: TOvcTextFileViewer;
    MnuWindow: TMenuItem;
    MnuSplit: TMenuItem;
    Splitter1: TSplitter;
    DataViewer2: TOvcTextFileViewer;
    procedure PageSetup1Click(Sender: TObject);
    procedure PrintPreview1Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Find1Click(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MnuSplitClick(Sender: TObject);
    procedure DataViewer1Enter(Sender: TObject);
    procedure DataViewer2Enter(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    theViewer: TOvcTextFileViewer;
    procedure Print(Destination: TDestination);
  public
    { Public declarations }
    procedure SetDataFile(const Fname: String);
  end;

//var
//  DataViewForm: TDataViewForm;

implementation

{$R *.DFM}

uses
  Fmain;

const
  MSG_TEXT_NOT_FOUND = 'Text not found.';
  MSG_LOADING_FAILURE = 'Unable to display project data.';
  TXT_SPLIT = 'Split';
  TXT_REMOVE_SPLIT = 'Remove Split';

procedure TDataViewForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnCreate handler.
//-----------------------------------------------------------------------------
begin
  // Inactivate the split view
  theViewer := DataViewer1;
  Splitter1.Visible := False;
  DataViewer2.Visible := False;

////  Size and position of form modified for release 5.0.015  ////              //(5.0.015 - LR)
  Width := MainForm.Width - MainForm.BrowserPanel1.Width;
  Height := MainForm.ClientHeight;
  Left := MainForm.Left + MainForm.BrowserPanel1.Width;
  Top := MainForm.Top + MainForm.DockAreaTop.Height;
end;

procedure TDataViewForm.FormShow(Sender: TObject);
//-----------------------------------------------------------------------------
//  Form's OnShow handler.
//-----------------------------------------------------------------------------
begin
  // Make the split view occupy 1/2 the form's height
  DataViewer2.Height := (ClientHeight div 2) - Splitter1.Height;
end;

procedure TDataViewForm.DataViewer1Enter(Sender: TObject);
//-----------------------------------------------------------------------------
//  Makes DataViewer1 the current viewer when it receives focus.
//-----------------------------------------------------------------------------
begin
  theViewer := DataViewer1;
end;

procedure TDataViewForm.DataViewer2Enter(Sender: TObject);
//-----------------------------------------------------------------------------
//  Makes DataViewer2 the current viewer when it receives focus.
//-----------------------------------------------------------------------------
begin
  theViewer := DataViewer2;
end;

procedure TDataViewForm.SetDataFile(const Fname: String);
//-----------------------------------------------------------------------------
//  Assigns a data file to the DataViewer control.
//-----------------------------------------------------------------------------
begin
  // Make sure that the data file exists
  DataViewer1.IsOpen := False;
  DataViewer2.IsOpen := False;
  if FileExists(Fname) then
  try
    // Load the contents of the file into the DataViewer
    DataViewer1.FileName := Fname;
    DataViewer2.FileName := Fname;
    DataViewer1.IsOpen := True;
  except
    On E: Exception do
      MessageDlg(MSG_LOADING_FAILURE, mtWarning, [mbOK], 0);
  end;
end;

procedure TDataViewForm.PageSetup1Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Page Setup menu option.
//-----------------------------------------------------------------------------
begin
  MainForm.MnuPageSetupClick(Sender);
end;

procedure TDataViewForm.PrintPreview1Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Print Preview menu option.
//-----------------------------------------------------------------------------
begin
  // Print to the preview form
  Print(dPreview);
end;

procedure TDataViewForm.Print1Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Print menu option.
//-----------------------------------------------------------------------------
begin
  // Print to the printer
  Print(dPrinter);
end;

procedure TDataViewForm.MnuSplitClick(Sender: TObject);
//-----------------------------------------------------------------------------
//  OnClick hander for the Window | Split menu item.
//-----------------------------------------------------------------------------
begin
  // Case where split view is currently active
  if Splitter1.Visible then
  begin
    Splitter1.Visible := False;
    DataViewer2.Visible := False;
    DataViewer2.IsOpen := False;
    theViewer := DataViewer1;
    MnuSplit.Caption := TXT_SPLIT;
  end

  // Case where the split view is currently inactive
  else
  begin
    DataViewer2.IsOpen := True;
    DataViewer2.TopLine := DataViewer1.TopLine;
    DataViewer2.Visible := True;
    Splitter1.Visible := True;
    MnuSplit.Caption := TXT_REMOVE_SPLIT;
  end;
end;

procedure TDataViewForm.Print(Destination: TDestination);
//-----------------------------------------------------------------------------
// Prints the contents of the viewer to the Destination device.
//-----------------------------------------------------------------------------
var
  I    : Integer;
  S    : array[0..1024] of AnsiChar;
begin
  with MainForm.thePrinter do
  begin
    BeginJob;
    SetDestination(Destination);
    with theViewer.FixedFont do
      SetFontInformation(Name, Size, Style);
    with theViewer do
      for I := 0 to LineCount - 1 do
      begin
        GetPrintableLine(I, S, SizeOf(S));
        PrintLine(S);
      end;
    EndJob;
  end;
end;

procedure TDataViewForm.Close1Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Close menu option.
//-----------------------------------------------------------------------------
begin
  Close;
end;

procedure TDataViewForm.Find1Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Find menu option.
//-----------------------------------------------------------------------------
begin
  FindDialog1.Execute;
end;

procedure TDataViewForm.FindDialog1Find(Sender: TObject);
//-----------------------------------------------------------------------------
// Launches a system Find dialog form to locate specific text in the viewer.
//-----------------------------------------------------------------------------
var
  OptionSet: TSearchOptionSet;  //Defined in the Orpheus OvcData unit.
begin
  OptionSet := [];
  OptionSet := OptionSet + [soFind] + [soSelText];
  with FindDialog1 do
  begin
    if frWholeWord in Options then OptionSet := OptionSet + [soWholeWord];
    if frMatchCase in Options then OptionSet := OptionSet + [soMatchCase];
    if not (frDown in Options) then OptionSet := OptionSet + [soBackward];
    if not theViewer.Search(FindText, OptionSet) then
      MessageDlg('Search string ''' + FindText + ''' not found',
        mtInformation, [mbOk], 0);
  end;
end;

end.
