unit Fmain;

{-------------------------------------------------------------------}
{                    Unit:    Fmain.pas                             }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    5/6/05      (5.0.005)                 }
{                             9/5/05      (5.0.006)                 }
{                             7/5/06      (5.0.008)                 }
{                             6/19/07     (5.0.010)                 }
{                             2/4/08      (5.0.012)                 }
{                             1/21/09     (5.0.014)                 }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Delphi form unit containing the main MDI form for EPA SWMM.     }
{                                                                   }
{   This unit contains the main MDI parent form, MainForm.          }
{   It contains the program's main menu, its tool bars, a           }
{   status panel and a Browser (i.e., control) panel.               }
{                                                                   }
{   The MDI Child forms consist of:                                 }
{     MapForm         - study area map                              }
{     StatusForm      - simulation status report                    }
{     GraphForm       - time series graph                           }
{     ProfilePlotForm - water elevation profile plot                }
{     TableForm       - tabular listing of results                  }
{     StatsReportForm - statistical analysis report                 }
{                                                                   }
{   Consult the files Uproject.pas, Uglobals.pas, Objprops.txt,     }
{   and Viewvars.txt for a description of the constants, classes,   }
{   and global variables used throughout the program.               }
{                                                                   }
{-------------------------------------------------------------------}

////  The following components were either modified or added  ////
////  using Delphi's IDE:                                     ////
////  MnuFindObject, MnuFindText, FindPopupMenu,              ////
////  PopupFindObject, PopupFindText, OpenTextFileDialog.     ////
////  (LR - 9/5/05)                                           ////

////  New Curves item added to ObjectTreeView control and     ////
////  all Curve items re-ordered. (5.0.010 - LR)              ////

////  Status Bar re-designed. (5.0.012 - LR)                  ////

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, ExtCtrls, Buttons, StdCtrls, ComCtrls,
  Printers, PgSetup, OpenDlg, TB97, Chart, ChartDlg, Xprinter,
  ExtDlgs, ImgList, Grids, ToolWin, VirtList, Spin, XPMan,
  ehshelprouter, Uglobals, Uproject, Uutils, Animator;

const
  MSG_NO_MAP_FILE = 'Could not read map file ';
  MSG_NO_CALIB_DATA = 'No calibration data has been registered for this project.';
  MSG_NO_INPUT_FILE = 'Input file no longer exists.';
  MSG_NOT_EPASWMM_FILE = 'Not an EPA-SWMM file.';
  MSG_READONLY = ' is read-only.'#10+
    'Use File >> Save As command to save it under a different name.';
  MSG_NO_BACKDROP = 'Could not find backdrop file ';
  MSG_FIND_BACKDROP = '. Do you want to search for it?';

  TXT_MAIN_CAPTION = 'SWMM 5';
  TXT_STATUS_REPORT = 'Status Report';
  TXT_SAVE_CHANGES = 'Save changes made to current project?';
  TXT_SAVE_RESULTS = 'Save current simulation results?';
  TXT_WARNING = '  WARNING:';
  TXT_TITLE = 'TITLE:';
  TXT_NOTES = 'NOTES:';
  TXT_HIDE = '&Hide';
  TXT_SHOW = '&Show';
  TXT_HOW_DO_I = 'How do I';
  TXT_OPEN_PROJECT_TITLE = 'Open a Project';
  TXT_OPEN_MAP_TITLE = 'Open a Map';
  TXT_SAVE_PROJECT_TITLE = 'Save Project As';
  TXT_OPEN_PROJECT_FILTER =
   'Input file (*.INP)|*.INP|' + 'Backup files (*.BAK)|*.BAK|All files|*.*';
  TXT_SAVE_PROJECT_FILTER = 'Input files (*.INP)|*.INP|All files|*.*';
  TXT_SCENARIO_FILTER = 'Scenario files (*.SCN)|*.SCN|All files|*.*';
  TXT_MAP_FILTER  = 'Map files (*.MAP)|*.MAP|All files|*.*';

  RunStatusHint: array[0..2] of String =
    ('No Results Available','Results Are Current','Results Need Updating');

  // Max. index of Object & Map Toolbar buttons
  MAXOBJTOOLBARINDEX = 12;
  MAXMAPTOOLBARINDEX = 8;                                                       //(5.0.012 - LR)

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;

    MnuFile: TMenuItem;
      MnuNew: TMenuItem;
      MnuOpen: TMenuItem;
      MnuReopen: TMenuItem;
      N13: TMenuItem;
      MnuSave: TMenuItem;
      MnuSaveAs: TMenuItem;
      N1: TMenuItem;
      MnuExport: TMenuItem;
        MnuExportMap: TMenuItem;
      MnuCombine: TMenuItem;
      N10: TMenuItem;
      MnuPageSetup: TMenuItem;
      MnuPrintPreview: TMenuItem;
      MnuPrint: TMenuItem;
      N11: TMenuItem;
      N2: TMenuItem;
      MnuExit: TMenuItem;

    MnuEdit: TMenuItem;
      MnuCopy: TMenuItem;
      N3: TMenuItem;
      MnuSelectObject: TMenuItem;
      MnuSelectVertex: TMenuItem;
      MnuSelectRegion: TMenuItem;
      MnuSelectAll: TMenuItem;
      N16: TMenuItem;
      MnuFindObject: TMenuItem;
      MnuFindText: TMenuItem;
      N17: TMenuItem;
      MnuGroupEdit: TMenuItem;
      MnuGroupDelete: TMenuItem;

    MnuView: TMenuItem;
      MnuDimensions: TMenuItem;
      MnuBackdrop: TMenuItem;
        MnuBackdropLoad: TMenuItem;
        MnuBackdropUnload: TMenuItem;
        N15: TMenuItem;
        MnuBackdropAlign: TMenuItem;
        MnuBackdropResize: TMenuItem;
        N18: TMenuItem;
        MnuBackdropWatermark: TMenuItem;
      N5: TMenuItem;
      MnuPan: TMenuItem;
      MnuZoomIn: TMenuItem;
      MnuZoomOut: TMenuItem;
      MnuFullExtent: TMenuItem;
      N14: TMenuItem;
      MnuQuery: TMenuItem;
      N12: TMenuItem;
      MnuOVMap: TMenuItem;
      MnuObjects: TMenuItem;
        MnuShowGages: TMenuItem;
        MnuShowSubcatch: TMenuItem;
        MnuShowNodes: TMenuItem;
        MnuShowLinks: TMenuItem;
        MnuShowLabels: TMenuItem;
        MnuShowBackdrop: TMenuItem;
      MnuLegends: TMenuItem;
        MnuSubcatchLegend: TMenuItem;
        MnuNodeLegend: TMenuItem;
        MnuLinkLegend: TMenuItem;
        MnuTimeLegend: TMenuItem;
        N6: TMenuItem;
        MnuModifyLegend: TMenuItem;
          MnuModifySubcatchLegend: TMenuItem;
          MnuModifyNodeLegend: TMenuItem;
          MnuModifyLinkLegend: TMenuItem;
      MnuToolbars: TMenuItem;
        MnuStdToolbar: TMenuItem;
        MnuMapToolbar: TMenuItem;
        MnuObjectToolbar: TMenuItem;

    MnuProject: TMenuItem;
      MnuProjectSummary: TMenuItem;
      MnuProjectDetails: TMenuItem;
      MnuProjectDefaults: TMenuItem;
      MnuProjectCalibData: TMenuItem;
      N9: TMenuItem;
      MnuProjectRunSimulation: TMenuItem;

    MnuReport: TMenuItem;
      MnuReportStatus: TMenuItem;
      MnuReportGraph: TMenuItem;
        MnuGraphTimeSeries: TMenuItem;
        MnuGraphScatter: TMenuItem;
        MnuGraphProfile: TMenuItem;
      MnuReportTable: TMenuItem;
        MnuTableByObject: TMenuItem;
        MnuTableByVariable: TMenuItem;
      MnuReportStatistics: TMenuItem;
      N8: TMenuItem;
      MnuReportOptions: TMenuItem;

////  New Tools menu item added. (5.0.010 - LR)  ////
    MnuTools: TMenuItem;
      MnuProgramOptions: TMenuItem;
      MnuMapDisplayOptions: TMenuItem;
      N7: TMenuItem;
      MnuConfigureTools: TMenuItem;

    MnuWindow: TMenuItem;
      MnuWindowTile: TMenuItem;
      MnuWindowCascade: TMenuItem;
      MnuWindowCloseAll: TMenuItem;

    MnuHelp: TMenuItem;
      MnuHelpTopics: TMenuItem;
      MnuHowdoI: TMenuItem;
      MnuHelpUnits: TMenuItem;
      MnuHelpErrors: TMenuItem;
      N19: TMenuItem;
      MnuHelpTutorial: TMenuItem;
      N4: TMenuItem;
      MnuAbout: TMenuItem;

////  This popup menu is no longer used. (LR - 9/5/05)  ////
//    GraphPopupMenu: TPopupMenu;
//      PopupGraphTimeSeries: TMenuItem;
//      PopupGraphScatter: TMenuItem;
//      PopupGraphProfile: TMenuItem;

    TablePopupMenu: TPopupMenu;
      PopupTableByObject: TMenuItem;
      PopupTableByVariable1: TMenuItem;
    FindPopupMenu: TPopupMenu;
      PopupFindObject: TMenuItem;
      PopupFindText: TMenuItem;

    OpenPictureDialog: TOpenPictureDialog;
    OpenTextFileDialog: TOpenTxtFileDialog;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    FontDialog: TFontDialog;
    PageSetupDialog: TPageSetupDialog;
    ChartDialog: TChartDialog;
    thePrinter: TPrintControl;

    ImageList: TImageList;
    BrowserImageList: TImageList;

    DockAreaTop: TDock97;

    StdToolBar: TToolbar97;
      TBNew: TToolbarButton97;
      TBOpen: TToolbarButton97;
      TBSave: TToolbarButton97;
      TBPrint: TToolbarButton97;
      ToolbarSep971: TToolbarSep97;
      TBCopy: TToolbarButton97;
      TBFind: TToolbarButton97;
      ToolbarSep972: TToolbarSep97;
      TBRun: TToolbarButton97;
      ToolbarSep973: TToolbarSep97;
      TBQuery: TToolbarButton97;
      TBGraph: TToolbarButton97;
      TBTable: TToolbarButton97;
      TBStats: TToolbarButton97;
      TBOptions: TToolbarButton97;
      TBArrange: TToolbarButton97;

    MapToolBar: TToolbar97;
      MapButton1: TToolbarButton97;
      MapButton2: TToolbarButton97;
      MapButton3: TToolbarButton97;
      MapButton4: TToolbarButton97;
      MapButton5: TToolbarButton97;
      MapButton6: TToolbarButton97;
      MapButton7: TToolbarButton97;

    ProgressPanel: TPanel;
      ProgressBar: TProgressBar;

    DockAreaLeft: TDock97;
    ObjectToolbar: TToolbar97;
    ObjectButton7: TToolbarButton97;
    ObjectButton8: TToolbarButton97;
    ObjectButton6: TToolbarButton97;
    ObjectButton2: TToolbarButton97;
    ObjectButton3: TToolbarButton97;
    ObjectButton4: TToolbarButton97;
    ObjectButton12: TToolbarButton97;
    ObjectButton5: TToolbarButton97;
    ObjectButton9: TToolbarButton97;
    ObjectButton10: TToolbarButton97;
    ObjectButton1: TToolbarButton97;
    ObjectButton11: TToolbarButton97;

    BrowserPanel1: TPanel;
    BrowserSplitter: TSplitter;
    BrowserPageControl: TPageControl;
      BrowserDataPage: TTabSheet;
        ObjectTreeView: TTreeView;
        ItemListBox: TVirtualListBox;
        BrowserDataSplitter: TSplitter;
        BrowserToolBar: TToolBar;
          BrowserBtnNew: TToolButton;
          BrowserBtnDelete: TToolButton;
          BrowserBtnEdit: TToolButton;
          BrowserBtnUp: TToolButton;
          BrowserBtnDown: TToolButton;
          BrowserBtnSort: TToolButton;
      BrowserMapPage: TTabSheet;
    HelpRouter1: THelpRouter;

////  New toolbar buttons added. (LR - 9/5/05)  ////
    TBProfile: TToolbarButton97;
    TBScatter: TToolbarButton97;
    MnuExportHotstart: TMenuItem;
    XPManifest1: TXPManifest;

////  Browser panel re-organized (5.0.010 - LR)  ////
    MapScrollBox: TScrollBox;
    MapThemesBox: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    SubcatchViewBox: TComboBox;
    NodeViewBox: TComboBox;
    LinkViewBox: TComboBox;
    MapTimePeriodBox: TGroupBox;
    DateLabel: TLabel;
    TimeLabel: TLabel;
    ElapsedTimeLabel: TLabel;
    DateListBox: TComboBox;
    DateScrollBar: TScrollBar;
    TimeListBox: TComboBox;
    TimeScrollBar: TScrollBar;
    ElapsedTimePanel: TEdit;
    ElapsedTimeUpDown: TUpDown;
    DockAreaRight: TDock97;
    AnimatorFrame: TAnimatorFrame;
    ItemsLabel: TLabel;
      AutoLengthMnu: TPopupMenu;
        AutoLengthOnMnu: TMenuItem;
        AutoLengthOffMnu: TMenuItem;
      FlowUnitsMnu: TPopupMenu;
        CFSMnuItem: TMenuItem;
        GPMMnuItem: TMenuItem;
        MGDMnuItem: TMenuItem;
        N20: TMenuItem;
        CMSMnuItem: TMenuItem;
        LPSMnuItem: TMenuItem;
        MLDMnuItem: TMenuItem;
      OffsetsMnu: TPopupMenu;
        OffsetsMnuDepth: TMenuItem;
        OffsetsMnuElev: TMenuItem;
    MapButton8: TToolbarButton97;
    DockAreaBottom: TDock97;
    StatusBar: TToolbar97;
    AutoLengthBtn: TToolbarButton97;
    ToolbarSep974: TToolbarSep97;
    OffsetsBtn: TToolbarButton97;
    ToolbarSep975: TToolbarSep97;
    FlowUnitsBtn: TToolbarButton97;
    ToolbarSep976: TToolbarSep97;
    RunStatusImage: TImage;
    ToolbarSep977: TToolbarSep97;
    ZoomLevelLabel: TLabel;
    ToolbarSep978: TToolbarSep97;
    XYLabel: TLabel;

    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);

    procedure MnuFileClick(Sender: TObject);
    procedure MnuNewClick(Sender: TObject);
    procedure MnuReopenClick(Sender: TObject);
    procedure MnuOpenClick(Sender: TObject);
    procedure MnuSaveClick(Sender: TObject);
    procedure MnuSaveAsClick(Sender: TObject);
    procedure MnuExportMapClick(Sender: TObject);
    procedure MnuCombineClick(Sender: TObject);
    procedure MnuPageSetupClick(Sender: TObject);
    procedure MnuPrintPreviewClick(Sender: TObject);
    procedure MnuPrintClick(Sender: TObject);
    procedure MnuPreferencesClick(Sender: TObject);
    procedure MnuExitClick(Sender: TObject);

    procedure MnuEditClick(Sender: TObject);
    procedure MnuCopyClick(Sender: TObject);
    procedure MnuSelectObjectClick(Sender: TObject);
    procedure MnuSelectVertexClick(Sender: TObject);
    procedure MnuSelectRegionClick(Sender: TObject);
    procedure MnuSelectAllClick(Sender: TObject);
    procedure MnuGroupEditClick(Sender: TObject);
    procedure MnuGroupDeleteClick(Sender: TObject);
    procedure MnuFindObjectClick(Sender: TObject);
    procedure MnuFindTextClick(Sender: TObject);

    procedure MapActionClick(Sender: TObject);
    procedure MnuBackdropClick(Sender: TObject);
    procedure MnuBackdropLoadClick(Sender: TObject);
    procedure MnuDimensionsClick(Sender: TObject);
    procedure MnuBackdropUnloadClick(Sender: TObject);
    procedure MnuBackdropAlignClick(Sender: TObject);
    procedure MnuBackdropResizeClick(Sender: TObject);
    procedure MnuBackdropWatermarkClick(Sender: TObject);
    procedure MnuQueryClick(Sender: TObject);
    procedure MnuOVMapClick(Sender: TObject);
    procedure MnuObjectsClick(Sender: TObject);
    procedure MnuShowObjectsClick(Sender: TObject);
    procedure MnuShowBackdropClick(Sender: TObject);
    procedure MnuLegendsClick(Sender: TObject);
    procedure MnuSubcatchLegendClick(Sender: TObject);
    procedure MnuNodeLegendClick(Sender: TObject);
    procedure MnuLinkLegendClick(Sender: TObject);
    procedure MnuTimeLegendClick(Sender: TObject);
    procedure MnuModifyLegendClick(Sender: TObject);
    procedure MnuModifySubcatchLegendClick(Sender: TObject);
    procedure MnuModifyLinkLegendClick(Sender: TObject);
    procedure MnuModifyNodeLegendClick(Sender: TObject);
    procedure MnuStdToolbarClick(Sender: TObject);
    procedure MnuMapToolbarClick(Sender: TObject);
    procedure MnuObjectToolbarClick(Sender: TObject);
    procedure MnuViewOptionsClick(Sender: TObject);

    procedure MnuProjectSummaryClick(Sender: TObject);
    procedure MnuProjectDefaultsClick(Sender: TObject);
    procedure MnuProjectDetailsClick(Sender: TObject);
    procedure MnuProjectCalibDataClick(Sender: TObject);
    procedure MnuProjectRunSimulationClick(Sender: TObject);

    procedure MnuReportClick(Sender: TObject);
    procedure MnuReportStatusClick(Sender: TObject);
    procedure MnuReportCustomizeClick(Sender: TObject);
    procedure MnuGraphTimeSeriesClick(Sender: TObject);
    procedure MnuGraphProfileClick(Sender: TObject);
    procedure MnuGraphScatterClick(Sender: TObject);
    procedure MnuTableByVariableClick(Sender: TObject);
    procedure MnuTableByObjectClick(Sender: TObject);
    procedure MnuReportStatisticsClick(Sender: TObject);

    procedure MnuWindowClick(Sender: TObject);
    procedure MnuWindowCascadeClick(Sender: TObject);
    procedure MnuWindowTileClick(Sender: TObject);
    procedure MnuWindowCloseAllClick(Sender: TObject);

    procedure MnuHelpTopicsClick(Sender: TObject);
    procedure MnuHowdoIClick(Sender: TObject);
    procedure MnuHelpUnitsClick(Sender: TObject);
    procedure MnuHelpErrorsClick(Sender: TObject);
    procedure MnuHelpTutorialClick(Sender: TObject);
    procedure MnuAboutClick(Sender: TObject);

    procedure FindPopupMenuPopup(Sender: TObject);

    procedure MapButtonClick(Sender: TObject);
    procedure MapButton7Click(Sender: TObject);
    procedure TBOptionsClick(Sender: TObject);
    procedure StdToolBarClose(Sender: TObject);
    procedure MapToolBarClose(Sender: TObject);
    procedure ObjectToolbarClose(Sender: TObject);
    procedure ObjectButtonClick(Sender: TObject);
    procedure BrowserBtnNewClick(Sender: TObject);
    procedure BrowserBtnDeleteClick(Sender: TObject);
    procedure BrowserBtnEditClick(Sender: TObject);
    procedure BrowserBtnUpClick(Sender: TObject);
    procedure BrowserBtnDownClick(Sender: TObject);
    procedure BrowserBtnSortClick(Sender: TObject);

    procedure ObjectTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure ObjectTreeViewClick(Sender: TObject);

    procedure ItemListBoxClick(Sender: TObject);
    procedure ItemListBoxDblClick(Sender: TObject);
    procedure ItemListBoxKeyPress(Sender: TObject; var Key: Char);
    procedure ItemListBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ItemListBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ItemListBoxGetItem(Sender: TObject; Index: Integer;
      var Value: String; var aColor: TColor);

    procedure MapViewBoxChange(Sender: TObject);
    procedure TimeListBoxClick(Sender: TObject);
    procedure TimeScrollBarChange(Sender: TObject);
    procedure TimeScrollBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure DateScrollBarChange(Sender: TObject);
    procedure DateScrollBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure DateListBoxClick(Sender: TObject);
    procedure BrowserPageControlResize(Sender: TObject);
    procedure ElapsedTimeUpDownChangingEx(Sender: TObject;
      var AllowChange: Boolean; NewValue: Smallint;
      Direction: TUpDownDirection);

////  New event handling procedures added. (LR - 9/5/05)  ////
    procedure MnuReportGraphClick(Sender: TObject);
    procedure TBGraphClick(Sender: TObject);

////  Procedures for new main form components. (5.0.010 - LR)  ////
    procedure MnuExportHotstartClick(Sender: TObject);
    //procedure AutoLengthCheckBoxClick(Sender: TObject);
    procedure MnuConfigureToolsClick(Sender: TObject);

////  Procedures for re-designed Status Bar components. (5.0.012 - LR)  ////
    procedure AutoLengthOnMnuClick(Sender: TObject);
    procedure AutoLengthOffMnuClick(Sender: TObject);
    procedure FlowUnitsMnuItemClick(Sender: TObject);
    procedure OffsetsMnuItemClick(Sender: TObject);

  private
    { Private declarations }

////  New array of menu items added for MRU file support. (LR - 9/5/05)  ////
    MnuMRU: array[0..Uglobals.MAXMRUINDEX] of TMenuItem;

    Startup: Boolean;
    procedure ClearAll;
    procedure CloseForms;
    procedure CreateTempFiles;
    procedure DeleteTempFiles;
    procedure FindBackdropFile;
    procedure InitPageLayout;
    procedure MRUDisplay(Sender: TObject);
    procedure MRUUpdate(Sender: TObject; const AddFileName: String);
    procedure Print(Dest: TDestination);
    procedure ReadCmdLine;
    procedure RecenterControl(aControl: TControl);
    procedure ResizeControl(aControl: TControl);
    procedure RunSimulation;
    procedure SaveFile(Fname: String);
    function  SaveFileDlg(Sender: TObject): Integer;

////  Procedure moved here for revised MRU file support. (LR - 9/5/05) ////
    procedure MRUClick(Sender: TObject);

//// New procedure added. (5.0.010 - LR) ////
    procedure SetItemsLabelFont;

 public
    { Public declarations }
    MRUList:  TStringList;
    procedure CreateReport(ReportSelection: TReportSelection);
    function  FormExists(const Name: String): Boolean;
    procedure HideProgressBar;
    procedure OpenFile(Sender: TObject; const Fname: String);
    procedure PageSetup;
    procedure PanButtonClick;
    procedure RefreshMapForm;
    procedure RefreshForms;
    procedure RefreshResults;
    procedure SelectorButtonClick;
    procedure SetChangeFlags;
    procedure ShowProgressBar(const Msg: String);
    procedure ShowRunStatus;
    procedure UpdateProfilePlots;
    procedure UpdateProgressBar(var Count: Integer; const StepSize: Integer);
    procedure ToolItemClick(Sender:TObject);
end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}
{$R mycurs32.res}  // Resource file containing custom cursors

uses
  Fmap, Fovmap, Fproped, Fstatus, Fgraph, Fproplot, Ftable, Fstats,
  Fsimul, Fsummary, Dabout, Dcalib1, Dcombine, Ddefault, Dgrpdel, Dprefers,
  Dviewer, Dreport, Dstats, Dgrouped, Dfind, Dquery, Dmapexp, Dbackdrp,
  Dbackdim, Dtools1, Ubrowser, Uinifile, Umap, Uimport, Uexport, Uoutput,
  Utools,                                                                       //(5.0.010 - LR)
  Uupdate,                                                                      //(5.0.012 - LR)
  Dreporting;                                                                   //(5.0.014 - LR)


//============================================================================
//            Form Creation, Resizing, & Closing Handlers
//============================================================================

procedure TMainForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
// Main form's OnCreate handler.
//-----------------------------------------------------------------------------
var
  I  : Integer;
begin
  // Load custom cursors from the mycurs32.res file
  Screen.Cursors[crXHAIR]   := LoadCursor(HInstance, PChar('xhair'));
  Screen.Cursors[crZOOMIN]  := LoadCursor(HInstance, PChar('zoomin'));
  Screen.Cursors[crZOOMOUT] := LoadCursor(HInstance, PChar('zoomout'));
  Screen.Cursors[crFIST]    := LoadCursor(HInstance, PChar('fist'));
  Screen.Cursors[crMOVE]    := LoadCursor(HInstance, PChar('move'));
  Screen.Cursors[crPENCIL]  := LoadCursor(HInstance, PChar('pencil'));
  Screen.Cursors[crARROWTIP]:= LoadCursor(HInstance, PChar('arrowtip'));
  Screen.Cursors[crSQUARE]  := LoadCursor(HInstance, PChar('square'));

  // Identify various directories
  EpaSwmmDir := ExtractFilePath(Application.ExeName);
  ProjectDir := EpaSwmmDir;
  WindowsDir := Uutils.GetWindowsDir;
  Application.HelpFile := EpaSwmmDir + HLPFILE;
  TempDir := Uutils.GetTempFolder;
  if TempDir = '' then TempDir := EpaSwmmDir;

  // Revised method for identifying Ini file and home directories.              //(5.0.010 - LR)
  IniFileDir := Uutils.GetAppDataDir('EPASWMM', EpaSwmmDir);
  HomeDir := Uutils.GetHomeDir;
  Caption := TXT_MAIN_CAPTION;

  // Rename any old INI files located in SWMM or home directories               //(5.0.010 - LR)
  RenameFile(EpaSwmmDir+INIFILE, IniFileDir+INIFILE);                           //(5.0.010 - LR)
  RenameFile(HomeDir+INIFILE , IniFileDir+INIFILE);                             //(5.0.010 - LR)

  // Process command line switches
  InputFileName := '';
  ReadCmdLine;

  // Set decimal separator and date/time format
  DecimalSeparator := '.';
  DateSeparator := '/';
  TimeSeparator := ':';
  ShortDateFormat := 'mm' + DateSeparator + 'dd' + DateSeparator + 'yyyy';
  LongTimeFormat := 'HH' + TimeSeparator + 'nn' + TimeSeparator + 'ss';
  GetLocaleFormatSettings(1033, FormatSettings);
  FormatSettings.DecimalSeparator := '.';
  FormatSettings.DateSeparator := '/';
  FormatSettings.TimeSeparator := ':';
  FormatSettings.ShortDateFormat := 'mm' + DateSeparator + 'dd' + DateSeparator + 'yyyy';
  FormatSettings.LongTimeFormat := 'HH' + TimeSeparator + 'nn' + TimeSeparator + 'ss';

  // Set the mouse & keyboard controls for zooming
  // & scrolling the TeeChart component
  TeeZoomMouseButton := mbLeft;
  TeeZoomKeyShift := [ssShift];
  TeeScrollMouseButton := mbLeft;
  TeeScrollKeyShift := [ssCtrl];

  // Create most-recently-used (MRU) file list
  MRUList := TStringList.Create;

////  Added for revised method of MRU file feature. (LR - 9/5/05)  ////
  // Create MRU file menu items
  for I := 0 to Uglobals.MAXMRUINDEX do
  begin
    MnuMRU[I] := TMenuItem.Create(self);
    MnuMRU[I].Tag     := I;
    MnuMRU[I].OnClick := MRUClick;
    MnuMRU[I].Name    := 'MRU' + IntToStr(I);
    MnuMRU[I].Visible := False;
    MnuReopen.Add(MnuMRU[I]);
  end;

  // Create list of add-on tools
  Utools.OpenToolList;

  // Use default number of decimal places
  for I := 0 to SUBCATCHVIEWS do SubcatchUnits[I].Digits := 2;
  for I := 0 to NODEVIEWS do NodeUnits[I].Digits := 2;
  for I := 0 to LINKVIEWS do LinkUnits[I].Digits := 2;

  // Create Project database object
  Project := TProject.Create;

  // Disable printing options if there are no printers
  thePrinter.SetShowProgress(True);
  if Printer.Printers.Count = 0 then
  begin
    MnuPageSetup.Enabled := False;
    MnuPrintPreview.Enabled := False;
    MnuPrint.Enabled := False;
    TBPrint.Enabled := False;
  end;

  // Set status flags
  Startup := False;
  RunStatus := rsNone;
  RunFlag := False;
  QueryFlag := False;
  QueryColor := clRed;
  HasChanged := False;
  MapButton1.Down := True;

  // Create other auxilary forms
  OVMapForm := TOVMapForm.Create(self);               // Overview map form
  PropEditForm := TPropEditForm.Create(self);         // Property Editor
  ReportSelectForm := TReportSelectForm.Create(self); // Report Selector        //(5.0.010 - LR)
  ReportingForm := TReportingForm.Create(self);       // Report Options         //(5.0.014 - LR)

  // Retrieve preferences from .INI file
  // (Must have created auxilary forms first and
  //  get MainForm size before Toolbar positions)
  Uinifile.ReadMainFormSize;
  IniLoadToolbarPositions(self, IniFileDir+INIFILE); // included in TToolbar97
  Uinifile.ReadIniFile;

  // Initialize checked status of view menu toolbar options
  MnuStdToolbar.Checked := StdToolBar.Visible;
  MnuMapToolbar.Checked := MapToolbar.Visible;
  MnuObjectToolbar.Checked := ObjectToolbar.Visible;

  // Display status panel, hide progress meter panel
  ProgressPanel.Visible := False;
  StatusBar.Visible := True;

  // Initialize Browser panel
  Ubrowser.InitBrowser;

  // Set font style
  Uglobals.SetFont(self);
  Uglobals.SetFont(PropEditForm);
  SetItemsLabelFont;                                                            //(5.0.010 - LR)

  // This hack sets the Browser's active page to the Map page to prevent
  // a 0 from appearing in the ElapsedTimeLabel box -- the active page
  // is then re-set to the Data page in the OnShow event handler below.
  BrowserPageControl.ActivePage := BrowserMapPage;

  // Prevent form from repainting itself for now
  LockWindowUpdate(Handle);
end;


//  The following FormShow, FormActivate, and FormResize methods are used
//  to begin an EPA SWMM session with the MapForm filling the entire
//  background of the MainForm and loading any project file that might
//  have been supplied on the command line. They take advantage of the
//  fact that on startup, the following events for the MainForm are
//  fired: OnCreate, OnShow, OnPaint, OnActivate, OnResize, OnPaint.


procedure TMainForm.FormShow(Sender: TObject);
//-----------------------------------------------------------------------------
// Main form's OnShow handler.
// Makes the Data page the active page of the Browser.
//-----------------------------------------------------------------------------
begin
  BrowserPageControl.ActivePage := BrowserDataPage;
end;


procedure TMainForm.FormActivate(Sender: TObject);
//-----------------------------------------------------------------------------
// Main form's OnActivate handler.
// Creates the Map form when the MainForm is first activated.
//-----------------------------------------------------------------------------
begin
  if not Assigned(MapForm) then
  begin
    // Create a Map child form
    MapForm := TMapForm.Create(self);
    with MapForm do
    begin
      PlaceLegends;
      RedrawOnResize := True;
      Startup := True;
    end;

    // Create the Map Finder and Query forms                                    //(5.0.014 - LR)
    FindForm := TFindForm.Create(self);
    QueryForm := TQueryForm.Create(self);                                       //(5.0.014 - LR)
  end;

  // Allow the main form to repaint itself
  LockWindowUpdate(0);
end;


procedure TMainForm.FormResize(Sender: TObject);
//-----------------------------------------------------------------------------
//  Main form's OnResize handler. Used only to open a project file supplied
//  on the command line or to begin with an empty project.
//-----------------------------------------------------------------------------
begin
  // Check if in startup mode
  if Startup then
  begin

    // Open a project file if one is provided on the command line,
    // otherwise simulate a click on File|New
    if Length(InputFileName) > 0 then
    begin
      ProjectDir := ExtractFileDir(InputFilename);
      OpenFile(Sender, InputFileName);
    end
    else MnuNewClick(Sender);

    // Reset the startup mode to false
    Startup := False;
  end;
end;


procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
//-----------------------------------------------------------------------------
// Main form's OnClose handler. Frees all allocated resources.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin

  // Save preferences to the .INI file                                          //(5.0.010 - LR)
  if not Uutils.IsReadOnly(IniFileDir) then                                     //(5.0.010 - LR)
  begin                                                                         //(5.0.010 - LR)
    Uinifile.SaveIniFile;
    Uinifile.SaveMainFormSize;
    IniSaveToolbarPositions(self, IniFileDir+INIFILE); // included in TToolbar97
  end;                                                                          //(5.0.010 - LR)

  // Clear any current simulation output results
  CloseForms;
  Uoutput.ClearOutput;
  DeleteTempFiles;

  // Free the forms created at startup
  MapForm.Free;
  OVMapForm.Free;
  QueryForm.Free;
  FindForm.Free;
  PropEditForm.Free;
  ReportSelectForm.Free;                                                        //(5.0.010 - LR)
  ReportingForm.Free;                                                           //(5.0.014 - LR)

  // Free the memory allocated for the project's data
  Project.Clear;
  Project.Free;

  // Free add-on tools list
  Utools.CloseToolList;

////  Added for new method of MRU files. (LR - 9/5/05)  ////
  // Free the most-recently-used file menu items
  for I := 0 to Uglobals.MAXMRUINDEX do MnuMRU[I].Free;

  // Free the most-recently-used file list
  MRUList.Free;
end;


procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
//-----------------------------------------------------------------------------
// OnCloseQuery handler for Main form.
// Checks if user wants to save the current project or cancel the
// Close request.
//-----------------------------------------------------------------------------
begin
  if SaveFileDlg(Sender) = mrCancel
  then CanClose := False
  else CanClose := True;
end;


//=============================================================================
//                        File Menu Handlers
//=============================================================================

procedure TMainForm.MnuFileClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for File menu
//-----------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////
////  This procedure was modified to accomodate the new
////  method of handling the most-recently-used file feature. (LR - 9/5/05)
///////////////////////////////////////////////////////////////////////////
var
  I: Integer;
  Enable: Boolean;
begin
  // Enable Reopen item if MRU list not empty
  Enable := False;
  for I := 0 to MRUList.Count-1 do
  begin
    if Length(MRUList[I]) > 0 then Enable := True;
  end;
  MnuReopen.Enabled := Enable;

  // Enable/disable printing (if printing is allowed)
  if MnuPageSetup.Enabled then
  begin
    MnuPrint.Enabled :=
      (ActiveMDIChild is TTableForm) or
      (ActiveMDIChild is TGraphForm) or
      (ActiveMDIChild is TProfilePlotForm) or
      (ActiveMDIChild is TStatsReportForm) or
      (ActiveMDIChild is TStatusForm) or
      (ActiveMDIChild is TMapForm);
    MnuPrintPreview.Enabled := MnuPrint.Enabled;
  end;

  // Enable/disable Export | Hotstart File
  MnuExportHotStart.Enabled := RunFlag;
end;


procedure TMainForm.MnuNewClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Creates a new project when File|New selected from main menu.
//-----------------------------------------------------------------------------
begin
  // Save current project data if it has changed
  if SaveFileDlg(Sender) = mrCancel then Exit;

  // Close any simulation output display forms
  CloseForms;

  // Re-set name of input project file
  InputFileName := '';
  InputFileType := iftINP;
  Caption := TXT_MAIN_CAPTION;
  ReadOnlyFlag := False;

  // Clear current project data
  ClearAll;             // Clears all project data
  ShowRunStatus;        // Resets run status icon
  PageSetup;            // Resets the printer page options

  // Make Title/Notes be the Browser's current data category
  Ubrowser.BrowserUpdate(NOTES, Project.CurrentItem[NOTES]);
end;


procedure TMainForm.MnuOpenClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Opens a project file when File|Open selected from main menu.
//-----------------------------------------------------------------------------
begin
  // Prompt the user to either save the current project data or to cancel
  if SaveFileDlg(Sender) = mrCancel then Exit;

  // Execute the Open File dialog
  with OpenTextFileDialog do
  begin

    // Set options for the Open File dialog control
    Title := TXT_OPEN_PROJECT_TITLE;
    Filter := TXT_OPEN_PROJECT_FILTER;
    InitialDir := ProjectDir;
    Filename := InputFileName;
    Options := Options - [ofHideReadOnly] + [ofFileMustExist];

    // If the user selects a file, then open it
    if Execute then
    begin
      ReadOnlyFlag := (ofReadOnly in Options) or
                      (HasAttr(FileName, faReadOnly));
      ProjectDir := ExtractFileDir(Filename);
      OpenFile(Sender, Filename);
      MapForm.ChangeHiliteObject(-1, -1);   // Remove object hilighting on map
    end;
    FilterIndex := 1;
  end;
end;


procedure TMainForm.MnuReopenClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for File|Reopen menu item.
//-----------------------------------------------------------------------------
begin
  MRUDisplay(Sender);
end;


procedure TMainForm.MnuSaveClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for File|Save menu item.
//-----------------------------------------------------------------------------
begin
  // For a new, un-named input file, implement the Save As command
  if (Length(InputFileName) = 0) then MnuSaveAsClick(Sender)

  // Otherwise save project data under the current input file name
  else SaveFile(InputFileName);
end;


procedure TMainForm.MnuSaveAsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Saves project to a new file when File|Save As selected from main menu.
//-----------------------------------------------------------------------------
begin
  with SaveDialog do
  begin
    Title := TXT_SAVE_PROJECT_TITLE;
    Filter := TXT_SAVE_PROJECT_FILTER;
    InitialDir := ProjectDir;
    DefaultExt := 'inp';
    if Length(InputFileName) > 0
    then Filename := ChangeFileExt(ExtractFileName(InputFileName),'.inp')
    else Filename := '*.inp';
    if Execute then SaveFile(Filename);
    DefaultExt := '';
  end;
end;


procedure TMainForm.MnuExportMapClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Launches the Map Export dialog when File|Export|Map selected.
// Allows the study area map image to be saved to file.
//-----------------------------------------------------------------------------
begin
  with TMapExportForm.Create(self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;


////  Added to support new File | Export | Hotstart menu item. (5.0.010 - LR)  ////
procedure TMainForm.MnuExportHotstartClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Allows output results at the current time period to be saved to a
// Hotstart file.
//-----------------------------------------------------------------------------
begin
  with SaveDialog do
  begin
    Title := 'Save Hotstart File As';
    Filter := 'Hotstart files (*.HSF)|*.HSF|All files|*.*';
    DefaultExt := 'hsf';
    Filename := '*.hsf';
    InitialDir := ProjectDir;
    if Execute then Uexport.SaveHotstartFile(Filename);
    DefaultExt := '';
  end;
end;


procedure TMainForm.MnuCombineClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Launches the Combine dialog form when File|Combine selected.
// Allows two SWMM Routing Interface results files to be combined into one
// (see Help file or Users Manual for description of a Routing Interface file.)
//-----------------------------------------------------------------------------
begin
  with TFileCombineForm.Create(self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;


procedure TMainForm.MnuPageSetupClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Page Setup dialog when File|Page Setup selected.
//-----------------------------------------------------------------------------
begin
  with PageSetupDialog, PageLayout do
  begin
    // Transfer current contents of the PageLayout structure to
    // the PageSetupDialog component
    Margins.Left   := LMargin;
    Margins.Right  := RMargin;
    Margins.Top    := TMargin;
    Margins.Bottom := BMargin;
    PageSetupDialog.BoldFont := BoldFonts;
    Printer.Orientation := TPrinterOrientation(Orientation);

    // Execute the dialog
    if PageSetupDialog.Execute then
    begin

      // Transfer new margins to the PageLayout structure
      LMargin := Margins.Left;
      RMargin := Margins.Right;
      TMargin := Margins.Top;
      BMargin := Margins.Bottom;

      // Setup the printer's page options
      Orientation := Ord(Printer.Orientation);
      PageSetup;
      HasChanged := True;
    end;
  end;
end;


procedure TMainForm.MnuPrintPreviewClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Prints the active window to a preview form when File|Print Preview selected.
//-----------------------------------------------------------------------------
begin
  Print(Xprinter.dPreview);
end;


procedure TMainForm.MnuPrintClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Prints the active window to the printer when File|Print selected.
//-----------------------------------------------------------------------------
begin
  Print(Xprinter.dPrinter);
end;


procedure TMainForm.Print(Dest: TDestination);
//-----------------------------------------------------------------------------
// Prints the active window to Dest (Preview window or printer).
//-----------------------------------------------------------------------------
begin
    if ActiveMDIChild is TMapForm
    then TMapForm(ActiveMDIChild).Print(Dest)

    else if ActiveMDIChild is TGraphForm
    then TGraphForm(ActiveMDIChild).Print(Dest)

    else if ActiveMDIChild is TProfilePlotForm
    then TProfilePlotForm(ActiveMDIChild).Print(Dest)

    else if ActiveMDIChild is TStatsReportForm
    then TStatsReportForm(ActiveMDIChild).Print(Dest)

    else if ActiveMDIChild is TTableForm
    then TTableForm(ActiveMDIChild).Print(Dest)

    else if ActiveMDIChild is TStatusForm
    then TStatusForm(ActiveMDIChild).Print(Dest);
end;


procedure TMainForm.MnuExitClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Shuts down program when File|Exit selected from main menu.
//-----------------------------------------------------------------------------
begin
  Close;
end;


//=============================================================================
//                         Edit Menu Handlers
//=============================================================================

procedure TMainForm.MnuEditClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Edit menu.
//-----------------------------------------------------------------------------
begin
  // Menu options Select Object, Select Vertex, & Select Region apply
  // only when the MapForm window is active
  MnuSelectObject.Enabled := ActiveMDIChild is TMapForm;
  MnuSelectVertex.Enabled := (ActiveMDIChild is TMapForm) and
                             MapButton2.Enabled;
  MnuSelectRegion.Enabled := ActiveMDIChild is TMapForm;

  // Select All applies when either the MapForm window, a TableForm window
  // or a StatsReportForm window is active
  MnuSelectAll.Enabled := (ActiveMDIChild is TMapForm) or
                          (ActiveMDIChild is TTableForm) or

                          ////  Added by LR - 7/5/06)  ////
                          (ActiveMDIChild is TStatusForm) or
                          
                          ((ActiveMDIChild is TStatsReportForm) and
                           (TStatsReportForm(ActiveMDIChild).
                            PageControl1.ActivePageIndex = 1));

  // Group editing applies only if a fenceline has been drawn on the
  // MapForm's map
  MnuGroupEdit.Enabled := (not MapForm.Linking) and
                          (MapForm.NumFencePts > 0);
  MnuGroupDelete.Enabled := MnuGroupEdit.Enabled;

  // Only allow certain windows to be copied
  MnuCopy.Enabled := (ActiveMDIChild is TMapForm) or
                     (ActiveMDIChild is TGraphForm) or
                     (ActiveMDIChild is TProfilePlotForm) or
                     (ActiveMDIChild is TStatusForm) or
                     (ActiveMDIChild is TTableForm) or
                     (ActiveMDIChild is TStatsReportForm);

////  Added to support text search on Status Form. (LR - 9/5/05) ////
  // Enable Find Text option if Status Report Form has focus
  MnuFindText.Enabled := (ActiveMDIChild is TStatusForm);
end;


procedure TMainForm.MnuCopyClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Copies active window to the clipboard or to a file when Edit|Copy To
// is selected.
//-----------------------------------------------------------------------------
begin
  if ActiveMDIChild is TMapForm
  then TMapForm(ActiveMDIChild).CopyTo

  else if ActiveMDIChild is TStatusForm
  then TStatusForm(ActiveMDIChild).CopyTo

  else if ActiveMDIChild is TGraphForm
  then TGraphForm(ActiveMDIChild).CopyTo

  else if ActiveMDIChild is TProfilePlotForm
  then TProfilePlotForm(ActiveMDIChild).CopyTo

  else if ActiveMDIChild is TTableForm
  then TTableForm(ActiveMDIChild).CopyTo

  else if ActiveMDIChild is TStatsReportForm
  then TStatsReportForm(ActiveMDIChild).CopyTo;
end;


procedure TMainForm.MnuSelectObjectClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Puts the MapForm into Object Selection mode when Edit|Select Object
// is selected.
//-----------------------------------------------------------------------------
begin
  SelectorButtonClick;
end;


procedure TMainForm.MnuSelectVertexClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Puts the MapForm into Vertex Selection mode when Edit|Select Vertex
// is selected.
//-----------------------------------------------------------------------------
begin
  MapButtonClick(MapButton2);
end;


procedure TMainForm.MnuSelectRegionClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Puts the MapForm into Region Selection mode when Edit|Select Region
// is selected.
//-----------------------------------------------------------------------------
begin
  MapButtonClick(MapButton3);
end;


procedure TMainForm.MnuSelectAllClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Selects all objects on a given form when Edit|Select All is selected.
//-----------------------------------------------------------------------------
begin
  if ActiveMDIChild is TMapForm then TMapForm(ActiveMDIChild).SelectAll;
  if ActiveMDIChild is TTableForm then TTableForm(ActiveMDIChild).SelectAll;
  if ActiveMDIChild is TStatsReportForm
  then TStatsReportForm(ActiveMDIChild).SelectAll;

////  Added by LR - 7/5/06  ////
  if ActiveMDIChild is TStatusForm
  then TStatusForm(ActiveMDIChild).SelectAll;
end;


procedure TMainForm.MnuFindObjectClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Activates the FindForm to locate a visual object by name.
//-----------------------------------------------------------------------------
begin
  if ActiveMDIChild is TStatusForm
  then TStatusForm(ActiveMDIChild).FindObject
  else FindForm.Visible := True;
end;


////  New function for searching for text on Status report. (LR - 9/5/05)  ////
procedure TMainForm.MnuFindTextClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Activates a Find text dialog to find text in the Status Report.
//-----------------------------------------------------------------------------
begin
  if ActiveMDIChild is TStatusForm
  then TStatusForm(ActiveMDIChild).FindText;
end;


procedure TMainForm.MnuGroupEditClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Group Edit dialog when Edit|Group Edit selected.
//-----------------------------------------------------------------------------
begin
  with TGroupEditForm.Create(self) do
  try
    PropEditForm.Hide;
    ShowModal;
  finally
    Free;
  end;
  if PropEditForm.Visible then Ubrowser.BrowserEditObject;
end;


procedure TMainForm.MnuGroupDeleteClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Deletes all objects in the selected region of the study area map when
// Edit|Group Delete is selected.
//-----------------------------------------------------------------------------
begin
  with TGroupDeleteForm.Create(self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;


//=============================================================================
//                         View Menu Handlers
//=============================================================================

procedure TMainForm.MnuDimensionsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Dimensions menu item.
//-----------------------------------------------------------------------------
begin
  MapForm.ModifyMapDimensions;
end;


procedure TMainForm.MnuBackdropClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Backdrop menu item.
//-----------------------------------------------------------------------------
var
  EnableFlag: Boolean;
begin
  // EnableFlag is true if the project has a map backdrop file
  EnableFlag := (Length(MapForm.Map.Backdrop.Filename) > 0);

  // Enable the Load Backdrop option if there is no current backdrop file
  MnuBackdropLoad.Enabled := not EnableFlag;

  // Enable the other options if there is a current backdrop file
  MnuBackdropUnload.Enabled := EnableFlag;
  MnuBackdropAlign.Enabled := EnableFlag;
  MnuBackdropResize.Enabled := EnableFlag;
  MnuBackdropWatermark.Enabled := EnableFlag;

  // Check the Watermark option if the backdrop is currently displayed
  // as a watermark
  MnuBackdropWatermark.Checked := EnableFlag and MapForm.Map.Backdrop.Watermark;
end;


procedure TMainForm.MnuBackdropLoadClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the View|Backdrop|Load menu item.
// Loads a backdrop image file into the project's map display.
//-----------------------------------------------------------------------------
begin
  // Use the Backdrop File dialog to get the name of a backdrop file
  with TBackdropFileForm.Create(self) do

  try
    if ShowModal = mrOK then with MapForm do
    begin

      // Retrieve the backdrop's file name and coordinates from the dialog
      Application.ProcessMessages;
      with Map.Backdrop do
      begin
        Filename := GetBackdropFilename;
        GetBackdropCoords(LowerLeft, UpperRight);
        Visible  := True;
      end;

      // Display the backdrop image on the study area map
      OpenBackdropFile;
      RedrawMap;

      // Update the Overview Map
      UpdateOVmap;
      if OVmapForm.Visible then OVmapForm.Redraw;
      Uglobals.HasChanged := True;
    end;

  finally
    Free;
  end;
end;


procedure TMainForm.MnuBackdropUnloadClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Backdrop|Unload menu item.
// Resets the project's backdrop image to a no-image default.
//-----------------------------------------------------------------------------
begin
  MapForm.Map.Backdrop := UMap.DefMapBackdrop;
  OVmapForm.OVmap.Backdrop := MapForm.Map.Backdrop;
  MapForm.RedrawMap;
  OVmapForm.Redraw;
  Uglobals.HasChanged := True;
end;


procedure TMainForm.MnuBackdropAlignClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Backdrop|Align menu item.
// Allows the user to reposition the backdrop image over the study area map.
//-----------------------------------------------------------------------------
begin
  MapActionClick(MnuPan);
  MapForm.BeginAligning(Sender);
end;


procedure TMainForm.MnuBackdropResizeClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Backdrop|Resize menu item.
// Displays the Backdrop Dimensions form to resize the backdrop image's
// bounding area.
//-----------------------------------------------------------------------------
begin
  if FormExists('BackdropDimensionsForm') then Exit;
  with TBackdropDimensionsForm.Create(self) do Show;
end;


procedure TMainForm.MnuBackdropWatermarkClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Backdrop|Watermark menu item.
//-----------------------------------------------------------------------------
begin
  with MnuBackdropWatermark do
  begin
    Checked := not Checked;
    with MapForm do
    begin
      Map.Backdrop.Watermark := Checked;
      Map.RedrawBackdrop;
      RedrawMap;
    end;
  end;
  Uglobals.HasChanged := True;
end;


procedure TMainForm.MapActionClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Map menu items Full Extent, Rescale, Pan, and Zoom.
//-----------------------------------------------------------------------------
begin
  with MapForm do
  begin
    // Restore the Map window to it snormal display state
    Show;
    SetFocus;
    WindowState := wsNormal;

    // The Tag property of each menu item was set to a
    // constant corresponding to the action it controls.
    if Sender is TMenuItem then with TMenuItem(Sender) do
    case Tag of

      // Display map at full extent
      FULLEXTENT:
      begin
        MapForm.DrawFullExtent;
        SelectorButtonClick;
      end;

      // Panning & zooming handled by MapButtons 4-6
      PAN:     MapButtonClick(MapButton4);
      ZOOMIN:  MapButtonClick(MapButton5);
      ZOOMOUT: MapButtonClick(MapButton6);
    end;
  end;
end;


procedure TMainForm.MnuQueryClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Activates the Query dialog to highlight items on the map that
// meet a specific criteria when View|Query is selected.
//-----------------------------------------------------------------------------
begin
//  if not Assigned(QueryForm) then QueryForm := TQueryForm.Create(self);
  QueryForm.Visible := True;
end;


procedure TMainForm.MnuOVMapClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of the Overview Map when View|Overview Map selected.
//-----------------------------------------------------------------------------
begin
  MnuOVMap.Checked := not MnuOVMap.Checked;
  OVMapForm.Visible := MnuOVMap.Checked;
end;


procedure TMainForm.MnuObjectsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View|Objects menu item.
// Allows the user to toggle the display of certain categories of map objects.
//-----------------------------------------------------------------------------
begin
  MnuShowGages.Checked := MapForm.Map.Options.ShowGages;
  MnuShowSubcatch.Checked := MapForm.Map.Options.ShowSubcatchs;
  MnuShowNodes.Checked := MapForm.Map.Options.ShowNodes;
  MnuShowLinks.Checked := MapForm.Map.Options.ShowLinks;
  MnuShowLabels.Checked := MapForm.Map.Options.ShowLabels;
  MnuShowBackdrop.Checked := MapForm.Map.Backdrop.Visible;
  MnuShowBackdrop.Enabled := (Length(MapForm.Map.Backdrop.Filename) > 0);
end;


procedure TMainForm.MnuShowObjectsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Common OnClick handler for View/Objects/Gages..Labels.
//-----------------------------------------------------------------------------
begin
  MapForm.PopupShowObjectsClick(Sender);
end;


procedure TMainForm.MnuShowBackdropClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for View/Objects/Backdrop.
//-----------------------------------------------------------------------------
begin
  MapForm.PopupShowBackdropClick(Sender);
end;


procedure TMainForm.MnuLegendsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Enables/disables submenu items when View|Legends selected.
//-----------------------------------------------------------------------------
begin
  // Legend items are disabled if no view variable is selected
  // or if map is in Query mode
  MnuSubcatchLegend.Enabled   := (not Uglobals.QueryFlag) and
                                 (Uglobals.CurrentSubcatchVar <> NOVIEW);
  MnuNodeLegend.Enabled   := (not Uglobals.QueryFlag) and
                             (Uglobals.CurrentNodeVar <> NOVIEW);
  MnuLinkLegend.Enabled   := (not Uglobals.QueryFlag) and
                             (Uglobals.CurrentLinkVar <> NOVIEW);
  MnuTimeLegend.Enabled   := Uglobals.RunFlag;
  MnuModifyLegend.Enabled := (not Uglobals.QueryFlag);
end;


procedure TMainForm.MnuSubcatchLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Map's Subcatchment legend when
// View|Legends|Subcatch selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ToggleSubcatchLegend;
end;


procedure TMainForm.MnuLinkLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Map's Link legend when View|Legends|Link selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ToggleLinkLegend;
end;


procedure TMainForm.MnuNodeLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Map's Node legend when View|Legends|Node selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ToggleNodeLegend;
end;


procedure TMainForm.MnuTimeLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Map's Time legend when View|Legends|Time selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ToggleTimeLegend;
end;


procedure TMainForm.MnuModifyLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Enables/disables submenu items when View|Legends|Modify selected.
//-----------------------------------------------------------------------------
begin
  MnuModifySubcatchLegend.Enabled := (Uglobals.CurrentSubcatchVar <> NOVIEW);
  MnuModifyNodeLegend.Enabled := (Uglobals.CurrentNodeVar <> NOVIEW);
  MnuModifyLinkLegend.Enabled := (Uglobals.CurrentLinkVar <> NOVIEW);
end;


procedure TMainForm.MnuModifySubcatchLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays Legend Editor form when View|Legends|Modify|Subcatch is selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ModifySubcatchLegend;
end;


procedure TMainForm.MnuModifyNodeLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays Legend Editor form when View|Legends|Modify|Node is selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ModifyNodeLegend;
end;


procedure TMainForm.MnuModifyLinkLegendClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays Legend Editor form when View|Legends|Modify|Link is selected.
//-----------------------------------------------------------------------------
begin
  MapForm.ModifyLinkLegend;
end;


procedure TMainForm.MnuStdToolbarClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Standard Toolbar when View|Toolbars|Standard selected.
//-----------------------------------------------------------------------------
begin
  MnuStdToolbar.Checked := not MnuStdToolbar.Checked;
  StdToolBar.Visible := MnuStdToolbar.Checked;
end;


procedure TMainForm.MnuMapToolbarClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Map Toolabr when View|Toolbars|Map selected.
//-----------------------------------------------------------------------------
begin
  MnuMapToolbar.Checked := not MnuMapToolbar.Checked;
  MapToolbar.Visible := MnuMapToolbar.Checked;
end;


procedure TMainForm.MnuObjectToolbarClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Toggles display of Object Toolbar when View|Toolbars|Object selected.
//-----------------------------------------------------------------------------
begin
  MnuObjectToolbar.Checked := not MnuObjectToolbar.Checked;
  ObjectToolbar.Visible := MnuObjectToolbar.Checked;
end;


procedure TMainForm.StdToolBarClose(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClose handler for Standard Toolbar -
// removes checked status for View|Toolbars|Standard menu item.
//-----------------------------------------------------------------------------
begin
  MnuStdToolbar.Checked := False;
end;


procedure TMainForm.MapToolBarClose(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClose handler for Map Toolbar -
// removes checked status for View|Toolbars|Map menu item.
//-----------------------------------------------------------------------------
begin
  MnuMapToolbar.Checked := False;
end;


procedure TMainForm.ObjectToolbarClose(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClose handler for Object Toolbar -
// removes checked status for View|Toolbars|Object menu item.
//-----------------------------------------------------------------------------
begin
  MnuObjectToolbar.Checked := False;
end;


//=============================================================================
//                        Project Menu Handlers
//=============================================================================

procedure TMainForm.MnuProjectSummaryClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Project Summary form when Project|Summary is selected.
//-----------------------------------------------------------------------------
begin
  with TProjectSummaryForm.Create(self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;


procedure TMainForm.MnuProjectDefaultsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Project Defaults dialog when Project|Defaults is selected.
//-----------------------------------------------------------------------------
begin
  PropEditForm.Hide;
  with TDefaultsForm.Create(self) do
  try
    if (ShowModal = mrOK) and (Modified = True) then
    begin
      SetChangeFlags;
    end;
  finally
    Free;
  end;
end;


procedure TMainForm.MnuProjectDetailsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays current project data in the EPA SWMM input file format
// when Project|Details is selected.
//-----------------------------------------------------------------------------

// Completely revised to accommodate new DataViewForm and Tools unit.           //(5.0.010 - LR)

var
  Fname: String;
  OldTabDelimited: Boolean;
begin
  OldTabDelimited := Uglobals.TabDelimited;
  Uglobals.TabDelimited := False;
  Fname := Utools.GetTmpInpFileName;
  with TDataViewForm.Create(Self) do
  try
    SetDataFile(Fname);
    ShowModal;
  finally
    Free;
  end;
  Uglobals.TabDelimited := OldTabDelimited;
end;


procedure TMainForm.MnuProjectCalibDataClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Calibration Data dialog to register calibration data
// when Project|Calibration Data is selected.
//-----------------------------------------------------------------------------
begin
  with TCalibDataForm.Create(self) do
  try
    if (ShowModal = mrOK) then Uglobals.RegisterCalibData;
  finally
    Free;
  end;
end;


procedure TMainForm.MnuProjectRunSimulationClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Runs a simulation when Project|Run Simulation is selected.
//-----------------------------------------------------------------------------
begin
  RunSimulation;
end;


//=============================================================================
//                         Report Menu Handlers
//=============================================================================

procedure TMainForm.MnuReportClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Controls what kind of reports can be generated when Report menu is selected.
//-----------------------------------------------------------------------------
begin
  // Reports are available only if a successful analysis run has been made
  MnuReportStatus.Enabled := True;
  MnuReportStatistics.Enabled := Uglobals.RunFlag;

////  Graph menu option now always enabled. (LR - 9/5/05)  ////
  MnuReportGraph.Enabled := True;    //Uglobals.RunFlag;

  MnuReportTable.Enabled := Uglobals.RunFlag;

  // The Options menu item is enabled depending on type of active window
  MnuReportOptions.Enabled :=
    (ActiveMDIChild is TProfilePlotForm) or
    (ActiveMDIChild is TGraphForm) or
    ((ActiveMDIChild is TStatsReportForm) and
    (TStatsReportForm(ActiveMDIChild).PageControl1.ActivePageIndex = 2));
end;

procedure TMainForm.MnuReportStatusClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays a run's Status Report when Report|Status is selected.
//-----------------------------------------------------------------------------
begin
  // Check if Status Report form already exists
  if FormExists('StatusForm') then Exit;

  // Otherwise create it
  with TStatusForm.Create(self) do
  try
    Caption := TXT_STATUS_REPORT;
    RefreshStatusReport;
    SetFocus;
  finally
  end;
end;

/////////////////////////////////////////////////////////////////
////  New event handlers added starting here. (LR - 9/5/05)  ////

procedure TMainForm.MnuReportGraphClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Enables Time Series and Scatter plot options only if results are
// available when Report|Graph is selected.
//-----------------------------------------------------------------------------
begin
  MnuGraphScatter.Enabled := Uglobals.RunFlag;
  MnuGraphTimeSeries.Enabled := Uglobals.RunFlag;
end;


procedure TMainForm.TBGraphClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the graphing toolbar buttons.
//-----------------------------------------------------------------------------
begin
  if Sender = TBGraph then MnuGraphTimeSeriesClick(Sender);
  if Sender = TBProfile then MnuGraphProfileClick(Sender);
  if Sender = TBScatter then MnuGraphScatterClick(Sender);
end;

////  End of new event handlers. (LR - 9/5/05)  ////
////////////////////////////////////////////////////

////  The following 5 procedures were modified to recognize that the  ////
////  ReportSelectForm is now created at start up. (5.0.010 - LR)     ////

procedure TMainForm.MnuGraphTimeSeriesClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Report Selection dialog form for a Time Series Plot when
// Report|Graph|Time Series is selected.
//-----------------------------------------------------------------------------
begin
  if Assigned(ReportSelectForm) then with ReportSelectForm do
  try
    SetReportType(TIMESERIESPLOT);
    Show;
  finally
  end;
end;


procedure TMainForm.MnuGraphScatterClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Report Selection dialog form for a Scatter Plot when
// Report|Graph|Scatter is selected.
//-----------------------------------------------------------------------------
begin
  if Assigned(ReportSelectForm) then with ReportSelectForm do
  try
    SetReportType(SCATTERPLOT);
    Show;
  finally
  end;
end;


procedure TMainForm.MnuGraphProfileClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Report Selection dialog form for a Profile Plot when
// Report|Graph|Profile is selected.
//-----------------------------------------------------------------------------
begin
  if Assigned(ReportSelectForm) then with ReportSelectForm do
  try
    SetReportType(PROFILEPLOT);
    Show;
  finally
  end;
end;


procedure TMainForm.MnuTableByObjectClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Report Selection dialog form for a Table by Object when
// Report|Table|By Object is selected.
//-----------------------------------------------------------------------------
begin
  if Assigned(ReportSelectForm) then with ReportSelectForm do
  try
    SetReportType(TABLEBYOBJECT);
    Show;
  finally
  end;
end;


procedure TMainForm.MnuTableByVariableClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Report Selection dialog form for a Table by Variable when
// Report|Table|By Variable is selected.
//-----------------------------------------------------------------------------
begin
  if Assigned(ReportSelectForm) then with ReportSelectForm do
  try
    SetReportType(TABLEBYVARIABLE);
    Show;
  finally
  end;
end;


procedure TMainForm.MnuReportStatisticsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the Statistics Selection dialog form when
// Report|Statistics is selected.
//-----------------------------------------------------------------------------
begin
  if not FormExists('StatsSelectForm')
  then with TStatsSelectForm.Create(self) do
  try
    Show;
  finally
  end;
end;


procedure TMainForm.MnuReportCustomizeClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Report|Customize menu option.
//-----------------------------------------------------------------------------
begin
  if (ActiveMDIChild is TGraphForm)
  then with ActiveMDIChild as TGraphForm do SetGraphOptions
  else if (ActiveMDIChild is TProfilePlotForm)
  then with ActiveMDIChild as TProfilePlotForm do SetPlotOptions
  else if (ActiveMDIChild is TStatsReportForm)
  then with ActiveMDIChild as TStatsReportForm do SetGraphOptions;
end;


//=============================================================================
//                       Tool Menu Handlers
//=============================================================================

procedure TMainForm.MnuPreferencesClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays Preferences dialog form when Tools|Program Preferences selected.    //(5.0.010 - LR)
//-----------------------------------------------------------------------------
var
  OldBoldFonts: Boolean;
  OldLargeFonts: Boolean;
begin
  OldBoldFonts := Uglobals.BoldFonts;
  OldLargeFonts := Uglobals.LargeFonts;
  with TPreferencesForm.Create(self) do
  try
    if ShowModal = mrOK then MapForm.RedrawMap;
  finally
    Free;
  end;
  if (OldBoldFonts <> Uglobals.BoldFonts)
  or (OldLargeFonts <> Uglobals.LargeFonts) then
  begin
    SetItemsLabelFont;                                                          //(5.0.010 - LR)
    Uglobals.SetFont(MainForm);
    Uglobals.SetFont(PropEditForm);
    Uglobals.SetFont(ReportingForm);                                            //(5.0.014 - LR)
    Uglobals.SetFont(FindForm);                                                 //(5.0.014 - LR)
    Uglobals.SetFont(QueryForm);                                                //(5.0.014 - LR)
    Uglobals.SetFont(ReportSelectForm);                                         //(5.0.014 - LR)

    MapForm.SubcatchLegendPanel.Refresh;
    MapForm.NodeLegendPanel.Refresh;
    MapForm.LinkLegendPanel.Refresh;
    RefreshForms;
  end;
end;


procedure TMainForm.MnuViewOptionsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays Map Options dialog when Tools|Map Display Options selected.         //(5.0.010 - LR)
//-----------------------------------------------------------------------------
begin
  MapForm.SetMapOptions;
end;


////  New procedure added.  ////                                                //(5.0.010 - LR)
procedure TMainForm.MnuConfigureToolsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays a Tool Options dialog when Tools | Configure Tools selected.
//-----------------------------------------------------------------------------
begin
  with TToolOptionsForm.Create(Self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;


////  New procedure added.  ////                                                //(5.0.010 - LR)
procedure TMainForm.ToolItemClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Launches an application when it is selected from the Tools menu.
//-----------------------------------------------------------------------------
begin
  Utools.RunTool(TMenuItem(Sender).Caption);
end;


//=============================================================================
//                       Window Menu Handlers
//=============================================================================

procedure TMainForm.MnuWindowClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Window menu - disables the Close All & Tile
// menu items if the MapForm is the only MDI child window.
//-----------------------------------------------------------------------------
var
  Flag: Boolean;
begin
  if MDIChildCount <= 1 then Flag := False else Flag := True;
  MnuWindowCloseAll.Enabled := Flag;
  MnuWindowTile.Enabled := Flag;
end;


procedure TMainForm.MnuWindowCascadeClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Cascades MDI child windows when Window|Cascade is selected.
//-----------------------------------------------------------------------------
var
  DockHeight: Integer;
  DockWidth: Integer;
begin
  // Account for height & width of Toolbars
  DockHeight := 0;
  if DockAreaTop.Visible
  then DockHeight := DockHeight + DockAreaTop.Height;
  if DockAreaLeft.Visible
  then DockWidth := DockAreaLeft.Width
  else DockWidth := 0;

  // Arrange all MDI child windows
  if (MDIChildCount >= 1) then
  begin
    // First prevent windows from re-drawing and cascade them
    LockWindowUpdate(Handle);
    MapForm.RedrawOnResize := False;
    Cascade;
    MapForm.RedrawOnResize := True;

    // Place MapForm in upper left
    if Assigned(MapForm) then with MapForm do
      SetBounds(0, 0, MainForm.ClientWidth -
                MainForm.BrowserPanel1.Width - 8,
                MainForm.ClientHeight - DockHeight -
                MainForm.StatusBar.Height - 5);

    // Allow windows to re-draw themselves
    LockWindowUpdate(0);
  end;
end;


procedure TMainForm.MnuWindowTileClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Tiles MDI child windows when Window|Cascade is selected.
//-----------------------------------------------------------------------------
begin
  MapForm.WindowState := wsMinimized;
  Tile;
end;


procedure TMainForm.MnuWindowCloseAllClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Closes all MDI children (except the Map) when Window|Close All is selected.
//-----------------------------------------------------------------------------
begin
  CloseForms;
end;


procedure TMainForm.MnuAboutClick(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays About form when Help|About is selected.
//-----------------------------------------------------------------------------
begin
  with TAboutBoxForm.Create(self) do
  try
    ShowModal;
  finally
    Free;
  end;
end;


//=============================================================================
//                        Toolbar Button Handlers
//=============================================================================


////  New procedure added. (LR - 9/5/05)  ////
procedure TMainForm.FindPopupMenuPopup(Sender: TObject);
//-----------------------------------------------------------------------------
// OnPopup handler for the FindPopupMenu popup menu assigned to the TBFind
// tool button.
//-----------------------------------------------------------------------------
begin
  PopupFindText.Enabled := (ActiveMDIChild is TStatusForm);
end;


procedure TMainForm.TBOptionsClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Options toolbar button.
//-----------------------------------------------------------------------------
begin
  if (ActiveMDIChild is TMapForm)
  then with ActiveMDIChild as TMapForm do SetMapOptions
  else MnuReportCustomizeClick(Sender);
end;


procedure TMainForm.ObjectButtonClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the buttons on the Object Toolbar - invokes the
// ToolButtonClick procedure on the MapForm. The Tag property of each
// button stores the button's number.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  // Place all buttons on the Map Toolbar in the UP position
  for I := 1 to MAXMAPTOOLBARINDEX do
  begin
    with FindComponent('MapButton'+IntToStr(I)) as TToolbarButton97 do
    begin
      Down := False;
    end;
  end;

  // Place the selected button on the Object Toolbar in the DOWN position
  TToolbarButton97(Sender).Down := True;

  // Pass the button click on to the MapForm
  MapForm.ToolButtonClick(TToolbarButton97(Sender).Tag);
end;


procedure TMainForm.MapButton7Click(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for Full Extent button on the Map Toolbar.
//-----------------------------------------------------------------------------
begin
  MapActionClick(MnuFullExtent);
end;


procedure TMainForm.SelectorButtonClick;
//-----------------------------------------------------------------------------
// Activates the Select tool button on Map Toolbar.
//-----------------------------------------------------------------------------
begin
  MapButtonClick(MapButton1);
end;


procedure TMainForm.PanButtonClick;
//-----------------------------------------------------------------------------
// Activates the Pan tool button on Map Toolbar.
//-----------------------------------------------------------------------------
begin
  MapButton4.Down := True;
  MapForm.ToolButtonClick(MapButton4.Tag);
end;


procedure TMainForm.MapButtonClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Map Toolbar buttons - invokes the
// ToolButtonClick procedure on the MapForm. The Tag property of each
// MapButton stores the button's action code:
// MapButton   Tag   Action
// ----------  ----  -----------------------------
//    1        101    Activate Map Selection tool
//    2        102    Activate Vertex Selection tool
//    3        103    Activate Group Selection tool
//    4        104    Activate Map Panning tool
//    5        105    Activate Map Zoom In tool
//    6        106    Activate Map Zoom Out tool
//    8        108    Activate Map Ruler tool                                   //(5.0.012 - LR)
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  // Place all buttons on the Object Toolbar in the UP position
  for I := 1 to MAXOBJTOOLBARINDEX do
  begin
    with FindComponent('ObjectButton'+IntToStr(I)) as TToolbarButton97 do
      Down := False;
  end;

  // Place the selected Map Toolbar button in the DOWN position
  TToolbarButton97(Sender).Down := True;

  // Pass the button click on to the MapForm
  with MapForm do
  begin
    ToolButtonClick(TToolbarButton97(Sender).Tag);
  end;

  // Put Zoom Out button back in Up position                                    //(5.0.010 - LR)
  if TToolbarButton97(Sender).Tag = 106 then                                    //(5.0.010 - LR)
  begin                                                                         //(5.0.010 - LR)
    TToolbarButton97(Sender).Down := False;                                     //(5.0.010 - LR)
    SelectorButtonClick;                                                        //(5.0.010 - LR)
  end;                                                                          //(5.0.010 - LR)

end;


//=============================================================================
//                     Browser Panel Procedures
//
//  The Browser panel consists of two tabbed pages.
//
//  The Data page allows users to select an object category from the
//  ObjectTreeView control, and select a particular object in that
//  category from the ItemListBox control.
//
//  The Map page allows users to select variables to view on the MapForm
//  from the SubcatchViewBox, NodeViewBox, and LinkViewBox. A viewing
//  time period can be selected from the DateListBox, TimeListBox, or
//  ElapsedTimeSpin controls.
//
//  Most actions on the Browser panel are handled in the Ubrowser unit.
//=============================================================================

procedure TMainForm.BrowserBtnNewClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the New button on the Browser's Data page.
//-----------------------------------------------------------------------------
begin
  Ubrowser.BrowserNewObject;
end;


procedure TMainForm.BrowserBtnDeleteClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Delete button on the Browser's Data page.
//-----------------------------------------------------------------------------
begin
  Ubrowser.BrowserDeleteObject;
end;


procedure TMainForm.BrowserBtnEditClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Edit button on the Browser's Data page.
//-----------------------------------------------------------------------------
begin
  Ubrowser.BrowserEditObject;
end;


procedure TMainForm.BrowserBtnUpClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Up button on the Browser's Data page.
// Moves the current selection in the ItemListBox up one position
// in the list of objects of that type.
//-----------------------------------------------------------------------------
var
  ItemIndex: Integer;
begin
  if Uglobals.CurrentList < 0 then Exit;
  ItemIndex := Project.CurrentItem[Uglobals.CurrentList];
  with Project.Lists[Uglobals.CurrentList] do
    if ItemIndex > 0 then
    begin
      Exchange(ItemIndex, ItemIndex-1);
      ItemListBox.ItemIndex := ItemIndex-1;
      Uglobals.HasChanged := True;
    end;
end;


procedure TMainForm.BrowserBtnDownClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Down button on the Browser's Data page.
// Moves the current selection in the ItemListBox down one position
// in the list of objects of that type.
//-----------------------------------------------------------------------------
var
  ItemIndex: Integer;
begin
  if Uglobals.CurrentList < 0 then Exit;
  ItemIndex := Project.CurrentItem[Uglobals.CurrentList];
  with Project.Lists[Uglobals.CurrentList] do
    if ItemIndex < Count-1 then
    begin
      Exchange(ItemIndex, ItemIndex+1);
      ItemListBox.ItemIndex := ItemIndex+1;
      Uglobals.HasChanged := True;
    end;
end;


procedure TMainForm.BrowserBtnSortClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Sort button on the Browser's Data page.
// Sorts the items of the current object category by ID name.
//-----------------------------------------------------------------------------
begin
  Ubrowser.BrowserSortObjects;
end;


procedure TMainForm.ObjectTreeViewClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the ObjectTreeView on the Browser's Data page.
//-----------------------------------------------------------------------------
begin
  MainForm.SelectorButtonClick;
end;


procedure TMainForm.ObjectTreeViewChange(Sender: TObject; Node: TTreeNode);
//-----------------------------------------------------------------------------
// OnChange handler for the ObjectTreeView on the Browser's Data page.
//-----------------------------------------------------------------------------
var
  ObjType: Integer;
  ItemIndex: Integer;
  Caption: String;
begin
  // Determine which object category corresponds to the tree node selected
  // in the ObjectTreeView control
  ObjType := Ubrowser.GetObjectFromIndex(Node.AbsoluteIndex);

  // A general category (with no specific items) was selected
  if ObjType < 0 then
  begin
    ItemIndex := -1;
    Caption := '';
  end

  // A category with specific items was selected
  else
  begin
    ItemIndex := Project.CurrentItem[ObjType];
    Caption := Node.Text;
  end;

  // Update the display of items for the selected category
  //BrowserPanel3.Caption := Caption;
  ItemsLabel.Caption := Caption;
  Ubrowser.BrowserUpdate(ObjType, ItemIndex);
end;


procedure TMainForm.ItemListBoxClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the Browser panel's ItemListBox.
//-----------------------------------------------------------------------------
begin
  with ItemListBox do
  begin
    if ItemIndex >= 0 then
    begin
      Ubrowser.BrowserUpdate(Uglobals.CurrentList, ItemIndex);
      if Assigned(FindForm)
      then FindForm.SearchFor(Uglobals.CurrentList, ItemIndex);
    end;
  end;
end;


procedure TMainForm.ItemListBoxDblClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnDblClick handler for the Browser panel's ItemListBox --
// edits the item double-clicked on.
//-----------------------------------------------------------------------------
begin
  // End any drag operation begun on a MouseDown action
  ItemListBox.EndDrag(False);

  // Select & edit the list box item
  ItemListBoxClick(Sender);
  if ItemListBox.ItemIndex >= 0 then Ubrowser.BrowserEditObject;
end;


procedure TMainForm.ItemListBoxKeyPress(Sender: TObject; var Key: Char);
//-----------------------------------------------------------------------------
// OnKeyPress handler for the Browser panel's ItemListBox --
// edits the item if the Enter key was pressed.
//-----------------------------------------------------------------------------
begin
  if Key = #13 then
  begin
    ItemListBoxDblClick(Sender);
    Key := #0;
  end;
end;


procedure TMainForm.ItemListBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
//-----------------------------------------------------------------------------
// OnKeyDown handler for the Browser panel's ItemListBox --
// deletes the item if the Delete key was pressed or inserts a new item if
// the Insert key was pressed.
//-----------------------------------------------------------------------------
begin
  if Project.IsSortable(Uglobals.CurrentList) then case Key of
  VK_DELETE:  BrowserBtnDeleteClick(Sender);
  VK_INSERT:  BrowserBtnNewClick(Sender);
  end;
end;


procedure TMainForm.ItemListBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
//-----------------------------------------------------------------------------
// OnMouseDown handler for the Browser panel's ItemListBox. Allows an
// item to be dragged from the ItemListBox.
//-----------------------------------------------------------------------------
begin
  // Only visual objects can be dragged
  if (Button = mbLeft)
  and (Uglobals.CurrentList in
      [RAINGAGE, SUBCATCH, JUNCTION, OUTFALL, DIVIDER, STORAGE, MAPLABEL])
  then with Sender as TVirtualListBox do
  begin
    if ItemAtPos(Point(X, Y), True) >= 0 then BeginDrag(False);
  end;
end;


procedure TMainForm.ItemListBoxGetItem(Sender: TObject; Index: Integer;
  var Value: String; var aColor: TColor);
//-----------------------------------------------------------------------------
// OnGetItem handler for the Browser panel's ItemListBox --
// retrieves the ID name of an item with a given Index.
//-----------------------------------------------------------------------------
begin
  // Check for valid item index
  Value := '';
  aColor := clVLB;
  if (Index < 0)
  or (Index >= Project.Lists[Uglobals.CurrentList].Count)
  then exit;

  // Gray-out display of item name if its a node with no coordinates
  if Project.IsNode(Uglobals.CurrentList) then
  begin
    if (Project.GetNode(Uglobals.CurrentList, Index).X = MISSING)
    or (Project.GetNode(Uglobals.CurrentList, Index).Y = MISSING)
    then aColor := clGray;
  end;

  // Get database ID label of item at current index
  Value := Project.GetID(Uglobals.CurrentList, Index);
end;


procedure TMainForm.MapViewBoxChange(Sender: TObject);
//-----------------------------------------------------------------------------
// Generic OnChange handler for the combo boxes that select a
// map theme on the Browser panel's Map page.
//-----------------------------------------------------------------------------
begin
  if (Sender = SubcatchViewBox)
  and (SubcatchViewBox.ItemIndex <> Uglobals.CurrentSubcatchVar)
  then Ubrowser.ChangeMapTheme(SUBCATCHMENTS, SubcatchViewBox.ItemIndex)

  else if (Sender = NodeViewBox)
  and (NodeViewBox.ItemIndex <> Uglobals.CurrentNodeVar)
  then Ubrowser.ChangeMapTheme(NODES, NodeViewBox.ItemIndex)

  else if (Sender = LinkViewBox)
  and (LinkViewBox.ItemIndex <> Uglobals.CurrentLinkVar)
  then Ubrowser.ChangeMapTheme(LINKS, LinkViewBox.ItemIndex);
end;


//=============================================================================
//  The following procedures are event handlers for the Time/Date/ElapsedTime
//  controls on the Map page of the Browser panel.
//=============================================================================

procedure TMainForm.DateListBoxClick(Sender: TObject);
begin
  Ubrowser.ChangeDate(DateListBox.ItemIndex);
end;

procedure TMainForm.DateScrollBarChange(Sender: TObject);
begin
  DateListBox.ItemIndex := DateScrollBar.Position;
end;

procedure TMainForm.DateScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  if ScrollCode in
  [scLineUp, scLineDown, scPageUp, scPageDown, scPosition] then
  begin
    DateListBox.ItemIndex := ScrollPos;
    DateListBoxClick(Sender);
  end;
end;

procedure TMainForm.TimeListBoxClick(Sender: TObject);
begin
  Ubrowser.ChangeTimePeriod(TimeListBox.ItemIndex);
end;

procedure TMainForm.TimeScrollBarChange(Sender: TObject);
begin
  TimeListBox.ItemIndex := TimeScrollBar.Position;
end;

procedure TMainForm.TimeScrollBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  if ScrollCode in
    [scLineUp, scLineDown, scPageUp, scPageDown, scPosition] then
  begin
    TimeListBox.ItemIndex := ScrollPos;
    TimeListBoxClick(Sender);
  end;
end;

procedure TMainForm.ElapsedTimeUpDownChangingEx(Sender: TObject;
  var AllowChange: Boolean; NewValue: Smallint;
  Direction: TUpDownDirection);
begin
  if Direction = updUp then Ubrowser.IncreaseElapsedTime;
  if Direction = updDown then Ubrowser.DecreaseElapsedTime;
  AllowChange := False;
end;


//=============================================================================
//  The following procedures handle resizing of the Browser panel's
//  controls when the panel is re-sized by the user.
//=============================================================================

procedure TMainForm.BrowserPageControlResize(Sender: TObject);
var
  S: String;
begin
  ResizeControl(MapThemesBox);
  ResizeControl(MapTimePeriodBox);

  ResizeControl(SubcatchViewBox);
  ResizeControl(NodeViewBox);
  ResizeControl(LinkViewBox);
  ResizeControl(DateListBox);
  ResizeControl(DateScrollBar);
  ResizeControl(TimeListBox);
  ResizeControl(TimeScrollBar);
  DateScrollBar.Top := DateListBox.Top + DateListBox.Height + 2;
  TimeScrollBar.Top := TimeListBox.Top + TimeListBox.Height + 2;
  with ElapsedTimePanel do
  begin
    // Need to save & redisplay panel's display text since associating
    // the panel with ElapsedTimeUpDown causes it to display the
    // the latter's Position property
    if Enabled then S := Text else S := '';
    Width := Parent.ClientWidth - 2*Left - ElapsedTimeUpDown.Width;
    ElapsedTimeUpDown.Associate := ElapsedTimePanel;
    Text := S;
  end;
end;

procedure TMainForm.ResizeControl(aControl: TControl);
begin
  with aControl do
    Width := Parent.ClientWidth - 2*Left;
end;

procedure TMainForm.RecenterControl(aControl: TControl);
begin
  with aControl do
    Left := (Parent.ClientWidth - Width) div 2;
end;

////  New procedure added. (5.0.010 - LR)  ////
procedure TMainForm.SetItemsLabelFont;
begin
  with ItemsLabel.Font do
  begin
    Color := clBlue;
    if Uglobals.LargeFonts then Size := 10
    else Size := 8;
    if Uglobals.BoldFonts then Style := [fsBold]
    else Style := [];
  end;
end;


//=============================================================================
//              Most Recently Used (MRU) File Procedures
//=============================================================================

procedure TMainForm.MRUUpdate(Sender: TObject; const AddFileName: String);
//-----------------------------------------------------------------------------
// Updates the MRU list when a new file (AddFileName) is opened.
//-----------------------------------------------------------------------------

////  Size of MRU file list now limited to MAXMRUINDEX
////  as defined in the Uglobals.pas unit. (LR - 9/5/05)

var
  Index: Integer;
begin
  Index := 0;
  while Index < (MRUList.Count - 1) do
    if AddFileName = MRUList[Index]
    then MRUList.Delete(Index)
    else Index := Index + 1;
  while MRUList.Count > Uglobals.MAXMRUINDEX do
    MRUList.Delete(MRUList.Count - 1);
  while MRUList.Count < Uglobals.MAXMRUINDEX do
    MRUList.Add('');
  MRUList.Insert(0, AddFileName);
end;


procedure TMainForm.MRUDisplay(Sender: TObject);
//-----------------------------------------------------------------------------
// Displays the MRU file list on the File menu.
//-----------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////
////  This procedure was modified to accomodate the new
////  method of handling the most-recently-used file feature. (LR - 9/5/05)
///////////////////////////////////////////////////////////////////////////
var
  I: Integer;
begin
  for I := 0 to Uglobals.MAXMRUINDEX do
  begin
    MnuMRU[I].Caption := IntToStr(I) + ' ' + MRUList[I];
    MnuMRU[I].Visible := (MRUList[I] <> '');
  end;
end;


procedure TMainForm.MRUClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the File|MRU File menu item - opens the selected file.
//-----------------------------------------------------------------------------
var
  Index: Integer;
  Fname: String;
begin
  Index := TMenuItem(Sender).Tag;
  Fname := MRUList[Index];
  if not FileExists(Fname)
  then MessageDlg(MSG_NO_INPUT_FILE, mtInformation, [mbOK], 0)
  else if SaveFileDlg(Sender) <> mrCancel then
  begin
    ReadOnlyFlag := (HasAttr(Fname, faReadOnly));
    ProjectDir := ExtractFileDir(Fname);
    OpenFile(Sender, Fname);
  end;
end;


//=============================================================================
//                      File Open & Save Procedures
//=============================================================================

procedure TMainForm.ReadCmdLine;
//-----------------------------------------------------------------------------
//  Reads command line switches.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  // Examine each command line parameter
  I := 1;
  while (I < ParamCount) do
  begin

    // Check for a '/s' command switch
    if SameText(ParamStr(I), '/s') then
    begin

      // The next parameter is the EPASWMM INI directory
      if (I < ParamCount)
      and DirectoryExists(ParamStr(I+1))
      then IniFileDir := ParamStr(I+1);
      I := I + 2;
    end

    // Check for a '/f' command switch
    else if SameText(ParamStr(I), '/f') then
    begin

////  Updated for 5.0.014  ////
      // The next parameter is the start-up project file name
      if (I < ParamCount)and FileExists(ParamStr(I+1)) then
      begin
        InputFileName := ParamStr(I+1);
        if Length(ExtractFileDir(InputFileName)) = 0
        then InputFileName := GetCurrentDir + '\' + InputFileName;
      end;
      I := I + 2;
    end
    else I := I + 1;

  end;
end;

procedure TMainForm.OpenFile(Sender: TObject; const Fname: String);
//-----------------------------------------------------------------------------
// Opens an existing project file named Fname.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  // Close all output display forms
  CloseForms;

  // Re-set file names
  Uglobals.InputFileName := Fname;
  SetCurrentDir(ExtractFileDir(Fname));
  MRUUpdate(Self, Uglobals.InputFileName);
  Caption := TXT_MAIN_CAPTION + ' - ' + ExtractFileName(Uglobals.InputFileName);

  // Clear all existing data
  ClearAll;
  ShowRunStatus;

  // Import data from input file
  Uglobals.InputFileType := Uimport.OpenProject(Uglobals.InputFileName);
  StatusBar.Refresh;

  // If import was unsuccessful, then open a new, blank project
  if (Uglobals.InputFileType = iftNone) then
  begin
    MnuNewClick(Sender);
    Exit;
  end;

  // Check that any backdrop file named in the input file actually exists
  FindBackdropFile;

  // Reset printer's page properties
  PageSetup;

  // Initialize current item in each object category
  for I := 0 to MAXCLASS do
  begin
    if Project.Lists[I].Count > 0 then Project.CurrentItem[I] := 0;
  end;
  Uglobals.CurrentList := -1;

  // Re-scale and redraw the study area & overview maps
  MapForm.Map.Rescale;
  MapForm.OpenBackdropFile;
  MapForm.UpdateOVmap;

  // If previous results were saved then prepare the Browser to display them
  if Uglobals.ResultsSaved then
  begin
    Uglobals.RunFlag := Uoutput.GetRunFlag(Uglobals.InputFileName);
    if Uglobals.RunFlag then
    begin
      Ubrowser.InitMapPage;
      RefreshResults;
    end;
  end;
  RefreshMapForm;

  // Display Title/Notes as the current object category in the Browser
  Ubrowser.BrowserUpdate(NOTES, Project.CurrentItem[NOTES]);
end;


procedure TMainForm.FindBackdropFile;
//-----------------------------------------------------------------------------
// Let's the user search for a map backdrop file.
//-----------------------------------------------------------------------------
begin
  with MapForm.Map do
  begin
    if  (Length(Backdrop.Filename) > 0) then
    begin
      if not FileExists(Backdrop.Filename) then
      begin
        if MessageDlg(MSG_NO_BACKDROP + Backdrop.Filename + MSG_FIND_BACKDROP,
          mtError, [mbYes,mbNo],0) = mrYes then
        begin
          with OpenPictureDialog do
          begin
            Filename := ExtractFileName(Backdrop.Filename);
            if Execute then
            begin
              Backdrop.Filename := Filename;
              Backdrop.Visible := True;
            end
            else Backdrop := Umap.DefMapBackdrop;
          end;
        end
        else Backdrop := Umap.DefMapBackdrop;
      end
      else Backdrop.Visible := True;
    end;
  end;
end;


////  This function has been re-written.  ////                                  //(5.0.012 - LR)
function TMainForm.SaveFileDlg(Sender: TObject): Integer;
//-----------------------------------------------------------------------------
// Checks if user wants to save current project to file.
//-----------------------------------------------------------------------------
begin
  // If project data has changed then ask user to save input & results
  Result := mrNo;
  if not Uglobals.ReadOnlyFlag and Uglobals.HasChanged then
  begin

    // See if input data should be saved
    Result := MessageDlg(TXT_SAVE_CHANGES, mtConfirmation, mbYesNoCancel, 0);
    if Result = mrYes then
    begin
      MnuSaveClick(Sender);

      // See if most current results should be saved
      if Uglobals.RunFlag and not Uglobals.ResultsSaved then
      begin
        if Uglobals.AutoSave
        or (MessageDlg(TXT_SAVE_RESULTS, mtConfirmation, [mbYes, mbNo], 0) = mrYes)
        then Uexport.SaveResults(Uglobals.InputFileName);
      end;
    end;
  end

  // If project data not changed made, see if results should be saved
  else if Uglobals.RunFlag and not Uglobals.ResultsSaved then
  begin
    if Uglobals.AutoSave
    or (MessageDlg(TXT_SAVE_RESULTS, mtConfirmation, [mbYes, mbNo], 0) = mrYes)
    then Uexport.SaveResults(Uglobals.InputFileName);
  end;
end;


procedure TMainForm.SaveFile(Fname: String);
//-----------------------------------------------------------------------------
// Saves project data in text format to file Fname.
//-----------------------------------------------------------------------------
begin
  // Append .inp extension to file name if none exists
  if ExtractFileExt(Fname) = '' then Fname := Fname + '.inp';

  // Check if project file is read-only
  if ReadOnlyFlag
  and (CompareText(Fname, Uglobals.InputFileName) = 0)
  then MessageDlg(ExtractFileName(Uglobals.InputFileName) + MSG_READONLY,
                  mtInformation, [mbOK], 0)

  // Save project under new name
  else
  begin
    Uexport.SaveProject(Fname);
    Uglobals.InputFileName := Fname;
    Uglobals.InputFileType := iftINP;
    Caption := Txt_MAIN_CAPTION + ' - ' +
               ExtractFileName(Uglobals.InputFileName);
    MRUUpdate(Self, Uglobals.InputFileName);
    Uglobals.HasChanged := False;
    Uglobals.ReadOnlyFlag := False;
    if AutoBackup                                                               //(5.0.014 - LR)
    then CopyFile(PChar(Fname), PChar(ChangeFileExt(Fname, '.bak')), FALSE);    //(5.0.014 - LR)
  end;
end;


procedure TMainForm.ClearAll;
//-----------------------------------------------------------------------------
// Clears the entire project database.
// (Called when File|New or File|Open selected)
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  // Clear all output and input data
  Uoutput.ClearOutput;
  DeleteTempFiles;
  Uglobals.TempInputFile  := '';
  Uglobals.TempReportFile := '';
  Uglobals.TempOutputFile := '';
  Project.Clear;

  // Hide the Property Editor
  PropEditForm.Hide;

  // Read project defaults from the INI file
  Uinifile.ReadDefaults;
  Uimport.SetDefaultDates;

  // Reset the Auto-Length feature
  Uglobals.AutoLength := False;
  AutoLengthBtn.Caption := 'Auto-Length: Off';

  // Reset the Link Offsets option                                              //(5.0.012 - LR)
  Uupdate.UpdateOffsets;                                                        //(5.0.012 - LR)

  // Initialize the current item and next ID number for each object category
  Project.InitCurrentItems;

  // Reset the Browser panel and clear the MapForm
  Ubrowser.InitDataPage;
  Ubrowser.InitMapPage;
  MapForm.Map.Options := Umap.DefMapOptions;
  MapForm.ClearMap;
  MapForm.DrawSubcatchLegend;
  MapForm.DrawNodeLegend;
  MapForm.DrawLinkLegend;
  MapForm.WindowState := wsNormal;

  // Reset the printer's page layout
  InitPageLayout;

  // Clear all calibration file information
  for I := Low(CalibData) to High(CalibData) do
    Uglobals.CalibData[I].FileName := '';

  // Disable output reporting toolbar buttons
  TBGraph.Enabled := False;
  TBTable.Enabled := False;
  TBStats.Enabled := False;

////  New button added. (LR - 9/5/05)  ////
  TBScatter.Enabled := False;

  // Reset status flags
  Uglobals.HasChanged := False;
  Uglobals.UpdateFlag := False;
  Uglobals.ResultsSaved := False;
  MainForm.SelectorButtonClick;
end;


procedure TMainForm.CreateTempFiles;
//-----------------------------------------------------------------------------
// Creates temporary files that begin with the letters 'swmm'.
//-----------------------------------------------------------------------------
begin
  Uglobals.TempInputFile  := Uutils.GetTempFile(Uglobals.TempDir, 'swmm');
  Uglobals.TempReportFile := Uutils.GetTempFile(Uglobals.TempDir, 'swmm');
  Uglobals.TempOutputFile := Uutils.GetTempFile(Uglobals.TempDir, 'swmm');
end;


procedure TMainForm.DeleteTempFiles;
//-----------------------------------------------------------------------------
// Deletes temporary files.
//-----------------------------------------------------------------------------
begin
  SysUtils.DeleteFile(Uglobals.TempInputFile);
  if not Uglobals.ResultsSaved then
  begin
    SysUtils.DeleteFile(Uglobals.TempReportFile);
    SysUtils.DeleteFile(Uglobals.TempOutputFile);
  end;
end;


//=============================================================================
//                     Form-Related Procedures
//=============================================================================

function TMainForm.FormExists(const Name: String): Boolean;
//-----------------------------------------------------------------------------
// Checks if form with given name already exists.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  Result := False;
  for I := 0 to Screen.FormCount - 1 do
  begin
    if Screen.Forms[I].Name = Name then
    begin
      with Screen.Forms[I] do
      begin
        WindowState := wsNormal;
        Show;
        SetFocus;
      end;
      Result := True;
      Exit;
    end;
  end;
end;


procedure TMainForm.CreateReport(ReportSelection: TReportSelection);
//-----------------------------------------------------------------------------
// Creates a new graph or table form.
//-----------------------------------------------------------------------------
begin
  if ReportSelection.ReportType = TIMESERIESPLOT then
  with TGraphForm.Create(self) do
  try
    if CreateGraph(ReportSelection) then RefreshGraph
    else Close;
  finally
  end;

  if ReportSelection.ReportType = SCATTERPLOT then
  with TGraphForm.Create(self) do
  try
    if CreateGraph(ReportSelection) then RefreshGraph
    else Close;
  finally
  end;

  if ReportSelection.ReportType = PROFILEPLOT then
  with TProfilePlotForm.Create(self) do
  try
    if CreatePlot(ReportSelection) then Show
    else Close;
  finally
  end;

  if ReportSelection.ReportType in [TABLEBYVARIABLE, TABLEBYOBJECT] then
  with TTableForm.Create(self) do
  try
    if CreateTable(ReportSelection) then Show
    else Close;
  finally
  end;
  StatusBar.Refresh;
end;


procedure TMainForm.CloseForms;
//-----------------------------------------------------------------------------
// Closes all forms (except the MapForm).
//-----------------------------------------------------------------------------
var
  I : Integer;
begin
  // Close MDI child forms
  for I := MDIChildCount - 1 downto 0 do
  begin
    if (MDIChildren[I] is TMapForm) then continue;
    MDIChildren[I].Close;
    MDIChildren[I].Free;
  end;

  // Hide overview map if visible
  if MnuOVMap.Checked then
  begin
    MnuOVMap.Checked := False;
    OVMapForm.Hide;
  end;

  // Hide Map Query and Find forms if visible
  if Assigned(QueryForm) then with QueryForm do
  begin
    Close;
    Clear;
  end;
  if Assigned(FindForm) then with FindForm do
  begin
    Close;
    Clear;
  end;
  if Assigned(ReportingForm) then with ReportingForm do
  begin
    Close;
    Clear;
  end;

  // Remove other stay-on-top dialog forms if visible
  if FormExists('ReportSelectForm') then
    with Screen.ActiveForm as TReportSelectForm do Close;
  if FormExists('StatsSelectForm') then
    with Screen.ActiveForm as TStatsSelectForm do Close;
  if FormExists('BackdropDimensionsForm') then
    with Screen.ActiveForm as TBackdropDimensionsForm do Close;
end;


procedure TMainForm.RefreshMapForm;
//-----------------------------------------------------------------------------
// Refreshes the MapForm after new project data is retreived.
//-----------------------------------------------------------------------------
begin
  Uoutput.SetSubcatchColors;
  Uoutput.SetNodeColors;
  Uoutput.SetLinkColors;
  MapForm.RedrawMap;
end;


procedure TMainForm.RefreshForms;
//-----------------------------------------------------------------------------
// Refreshes all open output display forms after new analysis is made.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  Application.ProcessMessages;
  LockWindowUpdate(Handle);
  for I := 0 to MDIChildCount - 1 do
  begin

    if (MDIChildren[I] is TStatusForm) then
      with MDIChildren[I] as TStatusForm do RefreshStatusReport

    else if (MDIChildren[I] is TGraphForm) then
      with MDIChildren[I] as TGraphForm do RefreshGraph

    else if (MDIChildren[I] is TProfilePlotForm) then
      with MDIChildren[I] as TProfilePlotForm do RefreshPlot

    else if (MDIChildren[I] is TTableForm) then
      with MDIChildren[I] as TTableForm do RefreshTable

    else if (MDIChildren[I] is TStatsReportForm) then
      with MDIChildren[I] as TStatsReportForm do RefreshReport;
  end;
  LockWindowUpdate(0);
end;


procedure TMainForm.UpdateProfilePlots;
//-----------------------------------------------------------------------------
// Updates all profile plots when a new time period is selected.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  LockWindowUpdate(Handle);
  for I := MDIChildCount - 1 downto 0 do
  begin
   if MDIChildren[I] is TProfilePlotForm then
     with MDIChildren[I] as TProfilePlotForm do
       UpdatePlot(Uglobals.CurrentPeriod);
  end;
  LockWindowUpdate(0);
end;


//=============================================================================
//                       Printer Page Setup Procedures
//=============================================================================

procedure TMainForm.InitPageLayout;
//-----------------------------------------------------------------------------
// Initializes the printer's page layout.
//-----------------------------------------------------------------------------
begin
  with PageLayout do
  begin
    LMargin := 1.0;
    TMargin := 1.5;
    RMargin := 1.0;
    BMargin := 1.0;
  end;
  with PageSetupDialog do
  begin
    Header.Text := '';
    Header.Alignment := taCenter;
    Header.Enabled := True;
    Footer.Text := TXT_MAIN_CAPTION;
    Footer.Alignment := taLeftJustify;
    Footer.Enabled := True;
    PageNumbers := pnLowerRight;
  end;
  TitleAsHeader := True;
  Orientation := Ord(poPortrait);
end;


procedure TMainForm.PageSetup;
//-----------------------------------------------------------------------------
// Transfers current page margins & header/footer options to the
// Printer object.
//-----------------------------------------------------------------------------
var
  Y: Single;
  Justify: TJustify;
begin
  if Printer.Printers.Count > 0 then with thePrinter do
  begin
    // Set printer orientation
    SetOrientation(TPrinterOrientation(Orientation));

    // Set page margins
    with PageLayout do
      SetMargins(TMargin,BMargin,LMargin,RMargin);

    with PageSetupDialog do
    begin
      // Define header line (0.5 inches above top margin)
      Justify := TJustify(Ord(Header.Alignment));
      SetHeaderInformation(1,PageLayout.TMargin-0.5,Header.Text,Justify,
        'Arial',14,[fsBold]);
      SetHeaders(Header.Enabled);

      // Define footer line (0.5 inches from bottom of page)
      Justify := TJustify(Ord(Footer.Alignment));
      SetFooterInformation(1,GetPageHeight-0.5,Footer.Text,Justify,
        'Arial',10,[fsBold, fsItalic]);
      SetFooters(Footer.Enabled);

      // Set page number location
      Justify := jRight;
      if PageNumbers in [pnUpperLeft, pnLowerLeft] then Justify := jLeft;
      if PageNumbers in [pnUpperCenter, pnLowerCenter] then Justify := jCenter;
      Y := 0.5;
      if PageNumbers in [pnLowerLeft, pnLowerCenter, pnLowerRight] then
        Y := GetPageHeight-0.5;
      SetPageNumberInformation(Y,'Page ',Justify,'Arial',10,[]);
      SetPageNumbers(not (PageNumbers = pnNone));
    end;
  end;
end;


//=============================================================================
//                        Progress Bar Procedures
//=============================================================================

procedure TMainForm.ShowProgressBar(const Msg: String);
//-----------------------------------------------------------------------------
// Activates the ProgressBar by hiding the StatusPanel panel and making
// the ProgressPanel panel visible.
//-----------------------------------------------------------------------------
var
  W: Integer;
begin
  // Display a message on the ProgressBar's panel
  W := Canvas.TextWidth(Msg) + 2;
  ProgressPanel.Caption := Msg;

  // Position the ProgressBar to the right of the message
  ProgressBar.Left := W;
  ProgressBar.Position := 0;

  // Switch the visibility of the StatusPanel and the ProgressPanel panels
  StatusBar.Visible := False;
  ProgressPanel.Visible := True;
  ProgressPanel.Update;
end;


procedure TMainForm.HideProgressBar;
//-----------------------------------------------------------------------------
// Hides the ProgressBar by switching the visibility of the ProgressPanel
// panel and the StatusPanel panel.
//-----------------------------------------------------------------------------
begin
  ProgressPanel.Visible := False;
  StatusBar.Visible := True;
end;


procedure TMainForm.UpdateProgressBar(var Count: Integer;
  const StepSize: Integer);
//-----------------------------------------------------------------------------
// Updates the display of the ProgressBar's meter.
//-----------------------------------------------------------------------------
begin
  Inc(Count);
  if Count >= StepSize then
  begin
    Application.ProcessMessages;
    Count := 0;
    ProgressBar.StepIt;
  end;
end;


//=============================================================================
//                          Status Bar Procedures
//=============================================================================

//// ******** New Status Bar Procedures Start Here ******** //                  //(5.0.012 - LR)

procedure TMainForm.AutoLengthOnMnuClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the 'On' menu choice for the Auto-Length button
//-----------------------------------------------------------------------------
begin
  AutoLength := True;
  AutoLengthBtn.Caption := 'Auto-Length: On';
end;

procedure TMainForm.AutoLengthOffMnuClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the 'Off' menu choice for the Auto-Length button
//-----------------------------------------------------------------------------
begin
  AutoLength := False;
  AutoLengthBtn.Caption := 'Auto-Length: Off';
end;

procedure TMainForm.FlowUnitsMnuItemClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the menu attached to the Flow Units button
//-----------------------------------------------------------------------------
begin
  with Sender as TMenuItem do
  begin
    if not SameText(Caption, Project.Options.Data[FLOW_UNITS_INDEX]) then
    begin
      Project.Options.Data[FLOW_UNITS_INDEX] := Caption;
      Uupdate.UpdateUnits;
      SetChangeFlags;
    end;
  end;
end;

procedure TMainForm.OffsetsMnuItemClick(Sender: TObject);
//-----------------------------------------------------------------------------
// OnClick handler for the menu attached to the Offsets button
//-----------------------------------------------------------------------------
begin
  with Sender as TMenuItem do
  begin
    if Pos(Project.Options.Data[LINK_OFFSETS_INDEX], Caption) = 0 then
    begin
      OffsetsBtn.Caption := Caption;
      Project.Options.Data[LINK_OFFSETS_INDEX] := LinkOffsetsOptions[Tag];
      SetChangeFlags;
      Uupdate.UpdateDefOptions;
      Uupdate.UpdateLinkHints;
      if PropEditForm.Visible then PropEditForm.RefreshPropertyHint;
      Uupdate.UpdateOffsets;
    end;
  end;
end;

//// ******** New Status Bar Procedures End Here ******** //                    //(5.0.012 - LR)

procedure TMainForm.ShowRunStatus;
//-----------------------------------------------------------------------------
// Displays the analysis success or failure icon in the StatusPanel.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  if not Uglobals.RunFlag then I := 0
  else if not Uglobals.UpdateFlag then I := 1
  else I := 2;
  ImageList.GetBitmap(I, RunStatusImage.Picture.Bitmap);                        //(5.0.012 - LR)
  RunStatusImage.Hint := RunStatusHint[I];                                      //(5.0.012 - LR)
  StatusBar.Refresh;
end;


procedure TMainForm.SetChangeFlags;
//-----------------------------------------------------------------------------
// Updates change flags after a change is made to the project.
// HasChanged: True if database has changed.
// UpdateFlag: True if analysis results need updating.
//-----------------------------------------------------------------------------
begin
  Uglobals.HasChanged := True;
  if Uglobals.RunFlag and not Uglobals.UpdateFlag then
  begin
    Uglobals.UpdateFlag := True;
    ShowRunStatus;
  end;
end;


//=============================================================================
//                    Procedures for Running a Simulation
//=============================================================================

procedure Execute;
//-----------------------------------------------------------------------------
// Executes the command line (console) version of the SWMM engine.
// (Not currently used.)
//-----------------------------------------------------------------------------
var
  OldDir: String;
  CmdLine: String;
  S: TStringlist;
begin
  GetDir(0, OldDir);
  ChDir(TempDir);
  S := TStringlist.Create;
  try
    Uexport.ExportProject(S);
    Uexport.ExportTempDir(S);
    S.SaveToFile(TempInputFile);
  finally
    S.Free;
  end;
  CmdLine := EpaSwmmDir + 'swmm5.exe ' +
             ExtractFileName(TempInputFile) + ' ' +
             ExtractFileName(TempReportFile) + ' ' +
             ExtractFileName(TempOutputFile);
  Uutils.WinExecAndWait(CmdLine, '', SW_SHOWNORMAL, false);
  if GetFileSize(Uglobals.TempReportFile) <= 0
  then Uglobals.RunStatus := rsFailed
  else Uglobals.RunStatus := Uoutput.CheckRunStatus(Uglobals.TempOutputFile);
  if not (Uglobals.RunStatus in [rsSuccess, rsWarning])
  then ShowMessage('Run was unsuccessful.');
  ChDir(OldDir);
end;


procedure TMainForm.RunSimulation;
//-----------------------------------------------------------------------------
// Uses the external SWMM engine to run a simulation.
//-----------------------------------------------------------------------------
begin
  // Clear all previous results
  Uoutput.ClearOutput;
  Ubrowser.InitMapPage;
  Uglobals.RunStatus := rsNone;

  // Create a new set of temporary files
  Uglobals.ResultsSaved := False;
  DeleteTempFiles;
  CreateTempFiles;

  // Call the command line version of the engine (not currently used).
  //Execute;

  // Display the Simulation dialog form (which will call the DLL
  // version of the engine)
  with TSimulationForm.Create(self) do
  try
    ShowModal;
  finally
    Free;
  end;

  // Delete temporary files if the run ended prematurely
  if (Uglobals.RunStatus = rsShutdown) then DeleteTempFiles;

  // Set RunFlag if the run produced results
  if Uglobals.RunStatus in [rsSuccess, rsWarning]
  then Uglobals.RunFlag := True
  else Uglobals.RunFlag := False;

  // Display the Status Report if the run produced errors
  if Uglobals.RunStatus = rsError then MnuReportStatusClick(Self);

  // Refresh output results and any existing reporting forms
  RefreshResults;
  RefreshForms;
end;


procedure TMainForm.RefreshResults;
//-----------------------------------------------------------------------------
// Prepares the system to display the results of a simulation run.
//-----------------------------------------------------------------------------
begin
  // Enable reporting toolbar buttons
  TBGraph.Enabled := Uglobals.RunFlag;
  TBTable.Enabled := Uglobals.RunFlag;
  TBStats.Enabled := Uglobals.RunFlag;

////  New button added. (LR - 9/5/05)  ////  
  TBScatter.Enabled := Uglobals.RunFlag;

  // Do following if output results exist
  if Uglobals.RunFlag then
  begin

    // Read prolog portion of binary results file
    ShowRunStatus;
    Uoutput.GetBasicOutput;

    // Update Browser panel's map page and the MapForm's display
    Ubrowser.UpdateMapPage;
    Ubrowser.RefreshMapColors;
    Ubrowser.RefreshMap;
  end

  // Otherwise re-set map display themes to none
  // (xxxOUTVAR1 is index of the first map display variable that
  // comes from simulation results).
  else
  begin
    if   Uglobals.CurrentSubcatchVar >= SUBCATCHOUTVAR1
    then Uglobals.CurrentSubcatchVar := NOVIEW;
    if   Uglobals.CurrentNodeVar >= NODEOUTVAR1
    then Uglobals.CurrentNodeVar := NOVIEW;
    if   Uglobals.CurrentLinkVar >= LINKOUTVAR1
    then Uglobals.CurrentLinkVar := NOVIEW;
    if Assigned(QueryForm) then
    begin
      QueryForm.UpdateVariables;
      QueryForm.Clear;
    end;
    MapForm.RedrawMap;
  end;

// Refresh map legends
  MapForm.DrawSubcatchLegend;
  MapForm.DrawNodeLegend;
  MapForm.DrawLinkLegend;
end;


//=============================================================================
//                     Help System Procedures
//=============================================================================

procedure TMainForm.MnuHelpTopicsClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TMainForm.MnuHelpUnitsClick(Sender: TObject);
begin
  if Uglobals.UnitSystem = usUS then
    Application.HelpCommand(HELP_CONTEXT, 211800)
  else
    Application.HelpCommand(HELP_CONTEXT, 211810);
end;

procedure TMainForm.MnuHelpErrorsClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTEXT, 212990);
end;

procedure TMainForm.MnuHelpTutorialClick(Sender: TObject);
begin
  Application.HelpFile := EpaSwmmDir + TUTORFILE;
  //HelpRouter1.HelpType := htWinhelp;                                          //(5.0.014 - LR)
  Application.HelpCommand(HELP_CONTENTS,0);
  Application.HelpFile := EpaSwmmDir + HLPFILE;
  //HelpRouter1.HelpType := htHTMLhelp;                                         //(5.0.014 - LR)
end;

procedure TMainForm.MnuHowdoIClick(Sender: TObject);
begin
  HelpRouter1.HelpKeyword(TXT_HOW_DO_I);
end;

end.
