unit Fsummary;

{-------------------------------------------------------------------}
{                    Unit:    Fsummary.pas                          }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    5/6/05                                }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Dialog form that lists the number of each type of SWMM object   }
{   within the current project.                                     }
{-------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Uproject, Uglobals;

type
  TProjectSummaryForm = class(TForm)
    Memo1: TMemo;
    BtnOK: TButton;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    TSFcount: Integer;                 //Number of time series inflow nodes
    DWFcount: Integer;                 //Number of dry weather inflow nodes
    GWFcount: Integer;                 //Number of groundwater inflow nodes
    IIFcount: Integer;                 //Number of RDII inflow nodes
    TUcount:  Integer;                 //Number of WQ treatment nodes
    procedure UpdateNodePropertyCount(N: TNode);
    procedure GetInflowsCount;
  public
    { Public declarations }
  end;

//var
//  ProjectSummaryForm: TProjectSummaryForm;

implementation

{$R *.dfm}

const
  S: array[0..29] of String =
    (
     ' Raingages              ',
     ' Subcatchments          ',
     ' Aquifers               ',
     ' Snowpacks              ',
     ' RDII Hydrographs       ',
     ' Infiltration           ',
     ' ---------------------  ',
     ' Junction Nodes         ',
     ' Outfall Nodes          ',
     ' Flow Divider Nodes     ',
     ' Storage Unit Nodes     ',
     ' ---------------------  ',
     ' Conduit Links          ',
     ' Pump Links             ',
     ' Orifice Links          ',
     ' Weir Links             ',
     ' Outlet Links           ',
     ' ---------------------  ',
     ' Flow Units             ',
     ' Flow Routing           ',
     ' Control Rules          ',
     ' ---------------------  ',
     ' Pollutants             ',
     ' Land Uses              ',
     '----------------------  ',
     ' Time Series Inflows    ',
     ' Dry Weather Inflows    ',
     ' Groundwater Inflows    ',
     ' RDII Inflows           ',
     ' Treatment Units        '
     );


procedure TProjectSummaryForm.FormCreate(Sender: TObject);
//-----------------------------------------------------------------------------
// Form's OnCreate handler. Compiles the number of each type of object
// in the current project and displays them in a Memo control.
//-----------------------------------------------------------------------------
begin
//Set font style
  Uglobals.SetFont(self);
  Memo1.Font.Style := Font.Style;

// Display project statistics
  with Memo1.Lines do
  begin
    Add(S[0] + IntToStr(Project.Lists[RAINGAGE].Count));
    Add(S[1] + IntToStr(Project.Lists[SUBCATCH].Count));
    Add(S[2] + IntToStr(Project.Lists[AQUIFER].Count));
    Add(S[3] + IntToStr(Project.Lists[SNOWPACK].Count));
    Add(S[4] + IntToStr(Project.Lists[HYDROGRAPH].Count));
    Add(S[5] + Project.Options.Data[INFILTRATION_INDEX]);
    Add(S[6]);
    Add(S[7] + IntToStr(Project.Lists[JUNCTION].Count));
    Add(S[8] + IntToStr(Project.Lists[OUTFALL].Count));
    Add(S[9] + IntToStr(Project.Lists[DIVIDER].Count));
    Add(S[10] + IntToStr(Project.Lists[STORAGE].Count));
    Add(S[11]);
    Add(S[12] + IntToStr(Project.Lists[CONDUIT].Count));
    Add(S[13] + IntToStr(Project.Lists[PUMP].Count));
    Add(S[14] + IntToStr(Project.Lists[ORIFICE].Count));
    Add(S[15] + IntToStr(Project.Lists[WEIR].Count));
    Add(S[16] + IntToStr(Project.Lists[OUTLET].Count));
    Add(S[17]);
    Add(S[18] + Project.Options.Data[FLOW_UNITS_INDEX]);
    Add(S[19] + Project.Options.Data[ROUTING_MODEL_INDEX]);
    Add(S[20] + IntToStr(Project.Lists[CONTROL].Count));
    Add(S[21]);
    Add(S[22] + IntToStr(Project.Lists[POLLUTANT].Count));
    Add(S[23] + IntToStr(Project.Lists[LANDUSE].Count));
    GetInflowsCount();
    Add(S[24]);
    Add(S[25] + IntToStr(TSFcount));
    Add(S[26] + IntToStr(DWFcount));
    Add(S[27] + IntToStr(GWFcount));
    Add(S[28] + IntToStr(IIFcount));
    Add(S[29] + IntToStr(TUcount));
  end;
  Memo1.SelStart := 0;
end;

procedure TProjectSummaryForm.GetInflowsCount;
//-----------------------------------------------------------------------------
// Counts the numbers of inflows objects of various types and the
// number of treatment nodes.
//-----------------------------------------------------------------------------
var
  I, J: Integer;
begin
  TSFcount := 0;
  DWFcount := 0;
  GWFcount := 0;
  IIFcount := 0;
  TUcount  := 0;

  // Count number of groundwater inflow objects
  with Project.Lists[SUBCATCH] do
  begin
    for I := 0 to Count-1 do
      if TSubcatch(Objects[I]).Groundwater.Count > 1 then Inc(GWFcount);
  end;

  // Count numbers of nodes with various types of inflows or treatment
  for J := JUNCTION to STORAGE do
  begin
    with Project.Lists[J] do
      for I := 0 to Count-1 do UpdateNodePropertyCount(TNode(Objects[I]));
  end;
end;

procedure TProjectSummaryForm.UpdateNodePropertyCount(N: TNode);
//-----------------------------------------------------------------------------
// Adds to the count of nodal inflows and treatment if the node N
// has such objects.
//-----------------------------------------------------------------------------
begin
  if N.DXInflow.Count > 0 then Inc(TSFcount);
  if N.DWInflow.Count > 0 then Inc(DWFcount);
  if N.IIInflow.Count > 0 then Inc(IIFcount);
  if N.Treatment.Count > 0 then Inc(TUcount);
end;

end.
