unit Ugraph;
        
{-------------------------------------------------------------------}
{                    Unit:    Ugraph.pas                            }
{                    Project: EPA SWMM                              }
{                    Version: 5.0                                   }
{                    Date:    5/6/05                                }
{                             9/5/05                                }
{                             2/4/08    (5.0.012)                   }
{                             3/3/08    (5.0.013)                   }
{                    Author:  L. Rossman                            }
{                                                                   }
{   Delphi Pascal unit that contains general purpose graphing       }
{   routines used in conjunction with a TChart object.              }
{-------------------------------------------------------------------}

interface

uses
  Dialogs, Classes, SysUtils, Forms, Controls, Windows,
  Graphics, Clipbrd, TeEngine, Series, Chart, ChartDlg, PgSetup,
  Xprinter, Uglobals, Uutils;

procedure CopyTo(Chart: TChart);
function  CreateBarSeries(Chart: TChart; const Stitle: String): Boolean;
function  CreateFastLineSeries(Chart: TChart; const Stitle: String): TFastLineSeries;
function  CreateLineSeries(Chart: TChart; const Stitle: String): TLineSeries;
function  CreatePointSeries(Chart: TChart; const Stitle: String): Boolean;
procedure InitGraphOptions(Chart: TChart);
procedure InitLineSeriesOptions(Series: TLineSeries; I: Integer);
procedure InitFastLineSeriesOptions(Series: TFastLineSeries; I: Integer);
procedure Print(Chart: TChart; thePrinter: TPrintControl;
          Destination: TDestination);
procedure SetAxisScale(Axis: TChartAxis; Amin: Double; Amax: Double; Delta: Double);
procedure SetGraphOptions(theChart: TChart; theDialog: TChartDialog);

implementation

uses Dcopy;

procedure CopyToBitmap(const Fname: String; Chart: TChart); forward;
procedure CopyToMetafile(const Fname: String; Chart: TChart); forward;
procedure ChartToPicture(aChart: TChart; aPicture: TPicture); forward;
procedure CopyToString(const Fname: String; Chart: TChart); forward;
procedure InitAxisOptions(Axis: TChartAxis); forward;
procedure SaveDefaultOptions(Chart: TChart); forward;


procedure CopyTo(Chart: TChart);
//-----------------------------------------------------------------------------
//  Copies the Chart to a file or to the Clipboard.
//-----------------------------------------------------------------------------
begin
  // Create the CopyTo dialog form
  with TCopyToForm.Create(Application) do
  try

////  Added by FM - 9/5/05  ////
    Caption := 'Copy Chart';

    // Show the dialog and retrieve the name of a file (DestFileName)
    // (If name is empty then chart is copied to the Clipboard)
    if ShowModal = mrOK then
    begin

      // Copy the chart using the selected format
      case FormatGroup.ItemIndex of
      0: CopyToBitmap(DestFileName, Chart);
      1: CopyToMetafile(DestFileName, Chart);
      2: CopyToString(DestFileName, Chart);
      end;
    end;
  finally
    Free;
  end;
end;


function CreateBarSeries(Chart: TChart;  const Stitle: String): Boolean;
//-----------------------------------------------------------------------------
//  Creates a bar data series in a Chart.
//-----------------------------------------------------------------------------
var
  aSeries: TBarSeries;
begin
  aSeries := TBarSeries.Create(Chart);
  with aSeries do
  try
    ParentChart := Chart;
    Title := Stitle;
    ShowInLegend := False;
    BarBrush.Color := GraphOptions.AreaFillColor;
    BarBrush.Style := GraphOptions.AreaFillStyle;
    BarWidthPercent := 100;
  finally
    Result := Assigned(aSeries);
  end;
end;


function  CreateFastLineSeries(Chart: TChart; const Stitle: String): TFastLineSeries;
//-----------------------------------------------------------------------------
//  Creates a fast line data series in a Chart. (Used when plotting very
//  large time series data sets).
//-----------------------------------------------------------------------------
var
  aSeries: TFastLineSeries;
  I: Integer;
begin
  aSeries := TFastLineSeries.Create(Chart);
  with aSeries do
  try
    ParentChart := Chart;
    Title := Stitle;
    ShowInLegend := True;
    I := Chart.SeriesCount - 1;
    InitFastLineSeriesOptions(aSeries, I);
  finally
    Result := aSeries;
  end;
end;


function CreateLineSeries(Chart: TChart;  const Stitle: String): TLineSeries;
//-----------------------------------------------------------------------------
//  Creates a line data series in a Chart.
//-----------------------------------------------------------------------------
var
  aSeries: TLineSeries;
  I: Integer;
begin
  aSeries := TLineSeries.Create(Chart);
  with aSeries do
  try
    ParentChart := Chart;
    Title := Stitle;
    ShowInLegend := True;
    I := Chart.SeriesCount - 1;
    InitLineSeriesOptions(aSeries, I);
  finally
    Result := aSeries;
  end;
end;


function CreatePointSeries(Chart: TChart;  const Stitle: String): Boolean;
//-----------------------------------------------------------------------------
//  Creates a point data series in a Chart.
//-----------------------------------------------------------------------------
var
  aSeries : TPointSeries;
  N       : Integer;
begin
  aSeries := TPointSeries.Create(Chart);
  with aSeries do
  try
    ParentChart := Chart;
    Title := Stitle;
    ShowInLegend := False;
    Pointer.Visible := True;
    N := Chart.SeriesCount - 1;
    if N <= MAXSERIES then
    begin
      Pointer.Style := TSeriesPointerStyle(GraphOptions.PointStyle[N]);
      SeriesColor := GraphOptions.PointColor[N];
      Pointer.HorizSize := GraphOptions.PointSize[N];
      Pointer.VertSize := GraphOptions.PointSize[N];
    end;
  finally
    Result := Assigned(aSeries);
  end;
end;


procedure InitGraphOptions(Chart: TChart);
//-----------------------------------------------------------------------------
//  Assigns the display options contained in the global variable GraphOptions
//  to a Chart object.
//-----------------------------------------------------------------------------
begin
  with Chart do
  begin
    View3D := GraphOptions.View3D;
    Chart3DPercent := GraphOptions.Percent3D;
    Color := GraphOptions.PanelColor;
    BackColor := GraphOptions.BackColor;
    Legend.ColorWidth := GraphOptions.LegendWidth;
    Legend.LegendStyle :=lsSeries;
    Legend.Alignment := TLegendAlignment(GraphOptions.LegendPosition);
    Legend.Visible := GraphOptions.LegendVisible;
    Title.Text.Clear;
    Title.Font.Name := GraphOptions.TitleFontName;
    Title.Font.Size := GraphOptions.TitleFontSize;
    Title.Font.Style := [];
    if GraphOptions.TitleFontBold
    then Title.Font.Style := Title.Font.Style + [fsBold];
    if GraphOptions.TitleFontItalic
    then Title.Font.Style := Title.Font.Style + [fsItalic];
    InitAxisOptions(BottomAxis);
    InitAxisOptions(LeftAxis);
    Legend.Font.Assign(BottomAxis.Title.Font);
  end;
end;


procedure Print(Chart: TChart; thePrinter: TPrintControl;
  Destination: TDestination);
//-----------------------------------------------------------------------------
// Prints the Chart to Destination (printer or preview form).
//-----------------------------------------------------------------------------
var
  W, H : Single;
  Left, Top : Single;
  Width, Height: Single;
  aRatio: Single;
  aPicture : TPicture;
begin
  if Chart.Height > 0
  then aRatio := Chart.Width/Chart.Height
  else aRatio := 1.0;
  with PageLayout, thePrinter do
  begin
    Left := LMargin;
    Width := GetPageWidth - LMargin - RMargin;
    Height := GetPageHeight - TMargin - BMargin;
  end;
  aPicture := TPicture.Create;
  with thePrinter do
  try
    BeginJob;
    SetDestination(Destination);
    SetFontInformation('Times New Roman', 11, []);
    ChartToPicture(Chart, aPicture);
    Uutils.FitFrameToPage(aRatio, Width, Height, W, H);
    Top := GetYPos;
    Left := Left + (Width - W)/2;
    StretchGraphic(Left, Top, Left+W, Top+H, aPicture);
    EndJob;
  finally
    aPicture.Free;
  end;
end;


procedure SetAxisScale(Axis: TChartAxis; Amin: Double;  Amax: Double;
  Delta: Double);
//-----------------------------------------------------------------------------
//  Sets the axis scaling for an Axis component of a TChart object.
//-----------------------------------------------------------------------------
var
  Zmin, Zmax, Dz: Double;
begin
  if Amin >= Amax then
  begin
    if Axis.Horizontal then
    begin
      Zmin := Axis.ParentChart.MinXValue(Axis);
      Zmax := Axis.ParentChart.MaxXValue(Axis);
    end
    else
    begin
      Zmin := Axis.ParentChart.MinYValue(Axis);
      Zmax := Axis.ParentChart.MaxYValue(Axis);
    end;
  end
  else
  begin
    Zmin := Amin;
    Zmax := Amax;
  end;
  Dz := Delta;
  if Dz <= 0 then Uutils.AutoScale(Zmin, Zmax, Dz);

  // Note: need following line to prevent graph from updating itself
  //       with invalid min/max axis values.
  Application.ProcessMessages;

  with Axis do
  begin
    Automatic := False;
    SetMinMax(Zmin, Zmax);
    Increment := Dz;
  end;
end;


procedure SetGraphOptions(theChart: TChart; theDialog: TChartDialog);
//-----------------------------------------------------------------------------
//  Launches the Graph Options dialog which allows the user to change
//  theChart's display options.
//-----------------------------------------------------------------------------
begin
  with theDialog do
  begin
    Chart := theChart;
    BoldFont := BoldFonts;
    DefaultBox := True;
    Execute;
    if DefaultChecked then SaveDefaultOptions(theChart);
  end;
end;


//============================================================================


procedure ChartToPicture(aChart: TChart; aPicture: TPicture);
//-----------------------------------------------------------------------------
//  Copies the graphic content of a TeeChart chart object into a
//  TPicture object.
//-----------------------------------------------------------------------------
var
  mf: TMetafile;
  mfCanvas: TMetafileCanvas;
begin
  mf := TMetafile.Create;
  try
    mf.Width := aChart.Width;
    mf.Height := aChart.Height;
    mfCanvas := TMetafileCanvas.Create(mf,0);
      try
        aChart.PrintPartialCanvas(mfCanvas,Rect(0,0,aChart.Width,aChart.Height));
      finally
        mfCanvas.Free;
      end;
    aPicture.Graphic := mf;
  finally
    mf.Free;
  end;
end;


procedure CopyToBitmap(const Fname: String; Chart: TChart);
//-----------------------------------------------------------------------------
//  Copies Chart to file Fname (or to Clipboard) in bitmap format.
//-----------------------------------------------------------------------------
begin
  if Length(Fname) > 0
  then Chart.SaveToBitmapFile(Fname)
  else Chart.CopyToClipboardBitmap;
end;


procedure CopyToMetafile(const Fname: String; Chart: TChart);
//-----------------------------------------------------------------------------
// Copies Chart to file Fname (or to Clipboard) in enhanced metafile format.
//-----------------------------------------------------------------------------
begin
  if Length(Fname) > 0
  then Chart.SaveToMetafileEnh(Fname)
  else Chart.CopyToClipBoardMetaFile(True);
end;


procedure CopyToString(const Fname: String; Chart: TChart);
//-----------------------------------------------------------------------------
// Copies data behind Chart to file Fname or to the Clipboard.
//-----------------------------------------------------------------------------
var
  Slist: TStringList;
  I,J  : Integer;
  S,S1 : String;
begin
  // Create a stringlist to hold data points
  Slist := TStringList.Create;
  try
    with Chart do
    begin

    // Add chart's title to the stringlist
    Slist.Add(Title.Text[0]);

    // Add column headings to the stringlist
    S := Format('%-16s'+#9+'%-16s'+#9+'%-16s',
      ['Series', BottomAxis.Title.Caption, LeftAxis.Title.Caption]);
    Slist.Add(S);

    // Iterate over each data series in the chart
    for I := 0 to SeriesCount - 1 do
    begin
      if Series[I].Active then with Series[I] do
      begin
        S1 := Title;
        for J := 0 to Count-1 do
        begin

          // Build up a tab-delimited string of X, Y values
          S := Format('%-16s'+#9+'%-16.4f'+#9+'%-16.4f',
                 [S1,XValues.Value[J],YValues.Value[J]]);

          // Add the tab-delimited string to the stringlist
          Slist.Add(S);
        end;
      end;
    end;
  end;

  // Save the stringlist to file if a file name was supplied,
  // otherwise place text of the stringlist onto the Clipboard
  if Length(Fname) > 0
  then Slist.SaveToFile(Fname)
  else Clipboard.SetTextBuf(PChar(Slist.Text));

  // Free the stringlist.
  finally
    Slist.Free;
  end;
end;


procedure InitAxisOptions(Axis: TChartAxis);
//-----------------------------------------------------------------------------
// Initializes axis properties for the Axis component of a TChart object.
//-----------------------------------------------------------------------------
var
  I: Integer;
begin
  with Axis do
  begin
    Automatic := True;
    AutomaticMinimum := True;
    AutomaticMaximum := True;
    Grid.Color := clBlue;
    Grid.Style := TPenStyle(2);
    if Horizontal then I := 0
    else I := 1;
    Grid.Visible := GraphOptions.AxisGridStyle[I] > 0;
    Title.Font.Name := GraphOptions.AxisFontName;
    Title.Font.Size := GraphOptions.AxisFontSize;
    Title.Font.Style := [];
    if GraphOptions.AxisFontBold
    then Title.Font.Style := Title.Font.Style + [fsBold];
    if GraphOptions.AxisFontItalic
    then Title.Font.Style := Title.Font.Style + [fsItalic];
    LabelsFont.Assign(Title.Font);
  end;
end;


procedure InitFastLineSeriesOptions(Series: TFastLineSeries; I: Integer);
//-----------------------------------------------------------------------------
//  Assigns the display options in the global variable GraphOptions to
//  a fast line data series component of a TChart object.
//-----------------------------------------------------------------------------
begin
  with Series do
  begin
    SeriesColor := GraphOptions.LineColor[I];
    LinePen.Visible := GraphOptions.LineVisible[I];                    
    LinePen.Style := TPenStyle(GraphOptions.LineStyle[I]);
    LinePen.Width := 1;
  end;
end;


procedure InitLineSeriesOptions(Series: TLineSeries; I: Integer);
//-----------------------------------------------------------------------------
//  Assigns the display options in the global variable GraphOptions to
//  a line data series component of a TChart object.
//-----------------------------------------------------------------------------
var                                                                             //(5.0.013 - LR)
  Psize: Integer;                                                               //(5.0.013 - LR)
begin
  with Series do
  begin
    SeriesColor := GraphOptions.LineColor[I];
    LinePen.Visible := GraphOptions.LineVisible[I];
    LinePen.Style := TPenStyle(GraphOptions.LineStyle[I]);
    LinePen.Width := GraphOptions.LineWidth[I];
    Pointer.Visible := GraphOptions.PointVisible[I];
    Pointer.Style := TSeriesPointerStyle(GraphOptions.PointStyle[I]);
    Pointer.Brush.Color := GraphOptions.PointColor[I];
    Psize := GraphOptions.PointSize[I];                                         //(5.0.013 - LR)
    if Psize > 0 then                                                           //(5.0.013 - LR)
    begin                                                                       //(5.0.013 - LR)
      Pointer.ChangeHorizSize(Psize);                                           //(5.0.013 - LR)
      Pointer.ChangeVertSize(Psize);                                            //(5.0.013 - LR)
    end;                                                                        //(5.0.013 - LR)
  end;
end;


procedure SaveDefaultOptions(Chart: TChart);
//-----------------------------------------------------------------------------
//  Saves Chart's display options as the default display options.
//-----------------------------------------------------------------------------
var
  I: Integer;
  K: Integer;
begin
  with GraphOptions do
  begin
    View3D := Chart.View3D;
    Percent3D := Chart.Chart3DPercent;
    PanelColor := Chart.Color;
    BackColor := Chart.BackColor;
    LegendPosition := Ord(Chart.Legend.Alignment);
    LegendVisible := Chart.Legend.Visible;
    LegendColor := Chart.Legend.Color;
    LegendWidth := Chart.Legend.ColorWidth;
    LegendFramed := Chart.Legend.Frame.Visible;
    with Chart.Title.Font do
    begin
      TitleFontName := Name;
      TitleFontColor := Color;
      TitleFontSize := Size;
      TitleFontBold := (fsBold in Style);
      TitleFontItalic := (fsItalic in Style);
    end;
    with Chart.BottomAxis do
    begin
      AxisGridStyle[0] := Ord(Grid.Visible);
      with Title.Font do
      begin
        AxisFontName := Name;
        AxisFontSize := Size;
        AxisFontBold := (fsBold in Style);
        AxisFontItalic := (fsItalic in Style);
      end;
    end;
    DateTimeFormat := Chart.BottomAxis.DateTimeFormat;                          //(5.0.012 - LR)
    with Chart.LeftAxis do
    begin
      AxisGridStyle[1] := Ord(Grid.Visible);
    end;
    for I := 0 to Chart.SeriesCount-1 do
    begin
      if Chart.Series[I].Active then
      begin
        if Chart.Series[I] is TAreaSeries then
          with Chart.Series[I] as TAreaSeries do
        begin
          AreaFillColor := SeriesColor;
          AreaFillStyle := AreaBrush;
        end;
        if Chart.Series[I] is TLineSeries then
          with Chart.Series[I] as TLineSeries do
        begin
          K := Tag;
          LineColor[K] := SeriesColor;
          LineStyle[K] := Ord(LinePen.Style);
          LineWidth[K] := LinePen.Width;
          PointVisible[K] := Pointer.Visible;
          PointStyle[K] := Ord(Pointer.Style);
          PointColor[K] := Pointer.Brush.Color;
          PointSize[K] := Pointer.HorizSize;
          LineVisible[K] := LinePen.Visible;
          if (not LineVisible[K]) and (not PointVisible[K])
          then LineVisible[K] := True;
        end;
        if Chart.Series[I] is TPointSeries then
          with Chart.Series[I] as TPointSeries do
        begin
          PointStyle[MAXSERIES] := Ord(Pointer.Style);
          PointColor[MAXSERIES] := SeriesColor;
          PointSize[MAXSERIES] := Pointer.HorizSize;
        end;
      end;
    end;
  end;
end;

end.

